require('.classes-1')
require('.classes-2')
require('.classes-3')
require('.classes-4')
require('.classes-5')
require('.classes-6')
require('.classes-7')
require('.classes-8')
require('.classes-9')

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaResourceCategoryPrototype.help() end


----
--- @class LuaRoboportControlBehavior: LuaControlBehavior
--- Control behavior for roboports.
--- 
--- @field read_logistics boolean `true` if the roboport should report the logistics network content to the circuit network.
--- @field read_robot_stats boolean `true` if the roboport should report the robot statistics to the circuit network.
--- @field available_logistic_output_signal SignalID 
--- @field total_logistic_output_signal SignalID 
--- @field available_construction_output_signal SignalID 
--- @field total_construction_output_signal SignalID 
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaRoboportControlBehavior = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaRoboportControlBehavior.help() end


----
--- @class LuaSettings
--- Object containing mod settings of three distinct types: `startup`, `global`, and `player`. An instance of LuaSettings is available through the global object named `settings`.
--- 
--- @field startup table<string,ModSetting> The startup mod settings, indexed by prototype name.
--- @field global table<string,ModSetting> The current global mod settings, indexed by prototype name.  Even though this attribute is marked as read-only, individual settings can be changed by overwriting their [ModSetting](ModSetting) table. Mods can only change their own settings. Using the in-game console, all player settings can be changed.
--- @field player table<string,ModSetting> The default player mod settings for this map, indexed by prototype name.  Even though this attribute is marked as read-only, individual settings can be changed by overwriting their [ModSetting](ModSetting) table. Mods can only change their own settings. Using the in-game console, all player settings can be changed.
--- @field object_name string This object's name.
----
LuaSettings = { }

----
--- Gets the current per-player settings for the given player, indexed by prototype name. Returns the same structure as [LuaPlayer::mod_settings](LuaPlayer::mod_settings). This table becomes invalid if its associated player does.
--- 
--- Even though this attribute is marked as read-only, individual settings can be changed by overwriting their [ModSetting](ModSetting) table. Mods can only change their own settings. Using the in-game console, all player settings can be changed.
--- 
--- @param player PlayerIdentification 
--- @return table<string,ModSetting> 
----
function LuaSettings.get_player_settings(player) end


----
--- @class LuaShortcutPrototype
--- Prototype of a shortcut.
--- 
--- @field name string Name of this prototype.
--- @field order string The string used to alphabetically sort these prototypes. It is a simple string that has no additional semantic meaning.
--- @field localised_name LocalisedString 
--- @field localised_description LocalisedString 
--- @field action string 
--- @field item_to_spawn LuaItemPrototype? The item to create when this shortcut is used, if any.
--- @field technology_to_unlock LuaTechnologyPrototype? The technology that needs to be researched once (in any save) for this shortcut to be unlocked (in all saves).
--- @field toggleable boolean 
--- @field associated_control_input string? The control input that is associated with this shortcut, if any.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaShortcutPrototype = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaShortcutPrototype.help() end


----
--- @class LuaStorageTankControlBehavior: LuaControlBehavior
--- Control behavior for storage tanks.
--- 
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaStorageTankControlBehavior = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaStorageTankControlBehavior.help() end


----
--- @class LuaStyle
--- Style of a GUI element. All of the attributes listed here may be `nil` if not available for a particular GUI element.
--- 
--- @field gui LuaGui Gui of the [LuaGuiElement](LuaGuiElement) of this style.
--- @field name string Name of this style.
--- @field minimal_width int Minimal width ensures, that the widget will never be smaller than than that size. It can't be squashed to be smaller.
--- @field maximal_width int Maximal width ensures, that the widget will never be bigger than than that size. It can't be stretched to be bigger.
--- @field minimal_height int Minimal height ensures, that the widget will never be smaller than than that size. It can't be squashed to be smaller.
--- @field maximal_height int Maximal height ensures, that the widget will never be bigger than than that size. It can't be stretched to be bigger.
--- @field natural_width int Natural width specifies the width of the element tries to have, but it can still be squashed/stretched to have a smaller or bigger size.
--- @field natural_height int Natural height specifies the height of the element tries to have, but it can still be squashed/stretched to have a smaller or bigger size.
--- @field top_padding int 
--- @field right_padding int 
--- @field bottom_padding int 
--- @field left_padding int 
--- @field top_margin int 
--- @field right_margin int 
--- @field bottom_margin int 
--- @field left_margin int 
--- @field horizontal_align string? Horizontal align of the inner content of the widget, if any. Possible values are "left", "center" or "right".
--- @field vertical_align string? Vertical align of the inner content of the widget, if any. Possible values are "top", "center" or "bottom".
--- @field font_color Color 
--- @field font string 
--- @field top_cell_padding int Space between the table cell contents top and border.
--- @field right_cell_padding int Space between the table cell contents right and border.
--- @field bottom_cell_padding int Space between the table cell contents bottom and border.
--- @field left_cell_padding int Space between the table cell contents left and border.
--- @field horizontally_stretchable boolean? Whether the GUI element stretches its size horizontally to other elements. `nil` if this element does not support stretching.
--- @field vertically_stretchable boolean? Whether the GUI element stretches its size vertically to other elements. `nil` if this element does not support stretching.
--- @field horizontally_squashable boolean? Whether the GUI element can be squashed (by maximal width of some parent element) horizontally. `nil` if this element does not support squashing. This is mainly meant to be used for scroll-pane The default value is false.
--- @field vertically_squashable boolean? Whether the GUI element can be squashed (by maximal height of some parent element) vertically. `nil` if this element does not support squashing. This is mainly meant to be used for scroll-pane The default (parent) value for scroll pane is true, false otherwise.
--- @field rich_text_setting defines.rich_text_setting How this GUI element handles rich text.
--- @field hovered_font_color Color 
--- @field clicked_font_color Color 
--- @field disabled_font_color Color 
--- @field pie_progress_color Color 
--- @field clicked_vertical_offset int 
--- @field selected_font_color Color 
--- @field selected_hovered_font_color Color 
--- @field selected_clicked_font_color Color 
--- @field strikethrough_color Color 
--- @field horizontal_spacing int Horizontal space between individual cells.
--- @field vertical_spacing int Vertical space between individual cells.
--- @field use_header_filler boolean 
--- @field bar_width uint 
--- @field color Color 
--- @field column_alignments table<uint,Alignment> Array containing the alignment for every column of this table element. Even though this property is marked as read-only, the alignment can be changed by indexing the LuaCustomTable, like so:
--- @field single_line boolean 
--- @field extra_top_padding_when_activated int 
--- @field extra_bottom_padding_when_activated int 
--- @field extra_left_padding_when_activated int 
--- @field extra_right_padding_when_activated int 
--- @field extra_top_margin_when_activated int 
--- @field extra_bottom_margin_when_activated int 
--- @field extra_left_margin_when_activated int 
--- @field extra_right_margin_when_activated int 
--- @field stretch_image_to_widget_size boolean 
--- @field badge_font string 
--- @field badge_horizontal_spacing int 
--- @field default_badge_font_color Color 
--- @field selected_badge_font_color Color 
--- @field disabled_badge_font_color Color 
--- @field width int Sets both minimal and maximal width to the given value.
--- @field height int Sets both minimal and maximal height to the given value.
--- @field size int|int[] Sets both width and height to the given value. Also accepts an array with two values, setting width to the first and height to the second one.
--- @field padding int|int[] Sets top/right/bottom/left paddings to this value. An array with two values sets top/bottom padding to the first value and left/right padding to the second value. An array with four values sets top, right, bottom, left padding respectively.
--- @field margin int|int[] Sets top/right/bottom/left margins to this value. An array with two values sets top/bottom margin to the first value and left/right margin to the second value. An array with four values sets top, right, bottom, left margin respectively.
--- @field cell_padding int Space between the table cell contents and border. Sets top/right/bottom/left cell paddings to this value.
--- @field extra_padding_when_activated int|int[] Sets `extra_top/right/bottom/left_padding_when_activated` to this value. An array with two values sets top/bottom padding to the first value and left/right padding to the second value. An array with four values sets top, right, bottom, left padding respectively.
--- @field extra_margin_when_activated int|int[] Sets `extra_top/right/bottom/left_margin_when_activated` to this value. An array with two values sets top/bottom margin to the first value and left/right margin to the second value. An array with four values sets top, right, bottom, left margin respectively.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaStyle = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaStyle.help() end


----
--- @class LuaSurface
--- A "domain" of the world. Surfaces can only be created and deleted through the API. Surfaces are uniquely identified by their name. Every game contains at least the surface "nauvis".
--- 
--- @field name string The name of this surface. Names are unique among surfaces.
--- @field index uint This surface's index in [LuaGameScript::surfaces](LuaGameScript::surfaces) (unique ID). It is assigned when a surface is created, and remains so until it is [deleted](on_surface_deleted). Indexes of deleted surfaces can be reused.
--- @field map_gen_settings MapGenSettings The generation settings for this surface. These can be modified after surface generation, but note that this will not retroactively update the surface. To manually regenerate it, [LuaSurface::regenerate_entity](LuaSurface::regenerate_entity), [LuaSurface::regenerate_decorative](LuaSurface::regenerate_decorative), and [LuaSurface::delete_chunk](LuaSurface::delete_chunk) can be used.
--- @field generate_with_lab_tiles boolean When set to true, new chunks will be generated with lab tiles, instead of using the surface's map generation settings.
--- @field always_day boolean When set to true, the sun will always shine.
--- @field daytime double Current time of day, as a number in range [0, 1).
--- @field darkness float Amount of darkness at the current time, as a number in range [0, 1].
--- @field wind_speed double Current wind speed in tiles per tick.
--- @field wind_orientation RealOrientation Current wind direction.
--- @field wind_orientation_change double Change in wind orientation per tick.
--- @field peaceful_mode boolean Is peaceful mode enabled on this surface?
--- @field freeze_daytime boolean True if daytime is currently frozen.
--- @field ticks_per_day uint The number of ticks per day for this surface.
--- @field dusk double The daytime when dusk starts.
--- @field dawn double The daytime when dawn starts.
--- @field evening double The daytime when evening starts.
--- @field morning double The daytime when morning starts.
--- @field solar_power_multiplier double The multiplier of solar power on this surface. Cannot be less than 0.
--- @field min_brightness double The minimal brightness during the night. Defaults to `0.15`. This has an effect on both rendering and game mechanics such as biter spawns and solar power.
--- @field brightness_visual_weights ColorModifier Defines how surface daytime brightness influences each color channel of the current color lookup table (LUT).  The LUT is multiplied by `((1 - weight) + brightness * weight)` and result is clamped to range [0, 1].  Default is `{0, 0, 0}`, which means no influence.
--- @field show_clouds boolean If clouds are shown on this surface.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaSurface = { }

----
--- Adds the given script area.
--- 
--- @param area ScriptArea 
--- @return uint The id of the created area.
----
function LuaSurface.add_script_area(area) end

----
--- Adds the given script position.
--- 
--- @param area ScriptPosition 
--- @return uint The id of the created position.
----
function LuaSurface.add_script_position(area) end

----
--- Sets the given area to the checkerboard lab tiles.
--- 
--- @param area BoundingBox The tile area.
----
function LuaSurface.build_checkerboard(area) end

----
--- Send a group to build a new base.
--- 
--- @param position MapPosition Location of the new base.
--- @param unit_count uint Number of biters to send for the base-building task.
--- @param force ForceIdentification? Force the new base will belong to. Defaults to enemy.
----
function LuaSurface.build_enemy_base(position, unit_count, force) end

----
--- 
--- @param property_names string[] Names of properties (e.g. "elevation") to calculate
--- @param positions MapPosition[] Positions for which to calculate property values
--- @return table<string,double[]> Table of property value lists, keyed by property name
----
function LuaSurface.calculate_tile_properties(property_names, positions) end

----
--- @class LuaSurface.can_fast_replace_parameters
--- Parameters class for the method `LuaSurface.can_fast_replace`
--- 
--- @field name string Name of the entity to check.
--- @field position MapPosition Where the entity would be placed.
--- @field direction defines.direction? Direction the entity would be placed. Defaults to `north`.
--- @field force ForceIdentification? The force that would place the entity. Defaults to the `"neutral"` force.
----

----
--- If there exists an entity at the given location that can be fast-replaced with the given entity parameters.
--- 
--- @param args LuaSurface.can_fast_replace_parameters Table containing arguments for the method.
--- @see LuaSurface.can_fast_replace_parameters
--- @return boolean 
----
function LuaSurface.can_fast_replace(args) end

----
--- @class LuaSurface.can_place_entity_parameters
--- Parameters class for the method `LuaSurface.can_place_entity`
--- 
--- @field name string Name of the entity prototype to check.
--- @field position MapPosition Where the entity would be placed.
--- @field direction defines.direction? Direction of the placed entity. Defaults to `north`.
--- @field force ForceIdentification? The force that would place the entity. Defaults to the `"neutral"` force.
--- @field build_check_type defines.build_check_type? Which type of check should be carried out. Defaults to `ghost_revive`.
--- @field forced boolean? If `true`, entities that can be marked for deconstruction are ignored. Only used if `build_check_type` is either `manual_ghost`, `script_ghost` or `blueprint_ghost`. Defaults to `false`.
--- @field inner_name string? The prototype name of the entity contained in the ghost. Only used if `name` is `entity-ghost`.
----

----
--- Check for collisions with terrain or other entities.
--- 
--- @param args LuaSurface.can_place_entity_parameters Table containing arguments for the method.
--- @see LuaSurface.can_place_entity_parameters
--- @return boolean 
----
function LuaSurface.can_place_entity(args) end

----
--- @class LuaSurface.cancel_deconstruct_area_parameters
--- Parameters class for the method `LuaSurface.cancel_deconstruct_area`
--- 
--- @field area BoundingBox The area to cancel deconstruction orders in.
--- @field force ForceIdentification The force whose deconstruction orders to cancel.
--- @field player PlayerIdentification? The player to set the last_user to if any.
--- @field skip_fog_of_war boolean? If chunks covered by fog-of-war are skipped.
--- @field item LuaItemStack? The deconstruction item to use if any.
----

----
--- Cancel a deconstruction order.
--- 
--- @param args LuaSurface.cancel_deconstruct_area_parameters Table containing arguments for the method.
--- @see LuaSurface.cancel_deconstruct_area_parameters
----
function LuaSurface.cancel_deconstruct_area(args) end

----
--- @class LuaSurface.cancel_upgrade_area_parameters
--- Parameters class for the method `LuaSurface.cancel_upgrade_area`
--- 
--- @field area BoundingBox The area to cancel upgrade orders in.
--- @field force ForceIdentification The force whose upgrade orders to cancel.
--- @field player PlayerIdentification? The player to set the last_user to if any.
--- @field skip_fog_of_war boolean? If chunks covered by fog-of-war are skipped.
--- @field item LuaItemStack? The upgrade item to use if any.
----

----
--- Cancel a upgrade order.
--- 
--- @param args LuaSurface.cancel_upgrade_area_parameters Table containing arguments for the method.
--- @see LuaSurface.cancel_upgrade_area_parameters
----
function LuaSurface.cancel_upgrade_area(args) end

----
--- Clears this surface deleting all entities and chunks on it.
--- 
--- @param ignore_characters boolean? Whether characters on this surface that are connected to or associated with players should be ignored (not destroyed).
----
function LuaSurface.clear(ignore_characters) end

----
--- Clears all pollution on this surface.
--- 
----
function LuaSurface.clear_pollution() end

----
--- @class LuaSurface.clone_area_parameters
--- Parameters class for the method `LuaSurface.clone_area`
--- 
--- @field source_area BoundingBox 
--- @field destination_area BoundingBox 
--- @field destination_surface SurfaceIdentification? 
--- @field destination_force string|LuaForce? 
--- @field clone_tiles boolean? If tiles should be cloned
--- @field clone_entities boolean? If entities should be cloned
--- @field clone_decoratives boolean? If decoratives should be cloned
--- @field clear_destination_entities boolean? If the destination entities should be cleared
--- @field clear_destination_decoratives boolean? If the destination decoratives should be cleared
--- @field expand_map boolean? If the destination surface should be expanded when destination_area is outside current bounds. Default false.
--- @field create_build_effect_smoke boolean? If true, the building effect smoke will be shown around the new entities.
----

----
--- Clones the given area.
--- 
--- @param args LuaSurface.clone_area_parameters Table containing arguments for the method.
--- @see LuaSurface.clone_area_parameters
----
function LuaSurface.clone_area(args) end

----
--- @class LuaSurface.clone_brush_parameters
--- Parameters class for the method `LuaSurface.clone_brush`
--- 
--- @field source_offset TilePosition 
--- @field destination_offset TilePosition 
--- @field source_positions TilePosition[] 
--- @field destination_surface SurfaceIdentification? 
--- @field destination_force string|LuaForce? 
--- @field clone_tiles boolean? If tiles should be cloned
--- @field clone_entities boolean? If entities should be cloned
--- @field clone_decoratives boolean? If decoratives should be cloned
--- @field clear_destination_entities boolean? If the destination entities should be cleared
--- @field clear_destination_decoratives boolean? If the destination decoratives should be cleared
--- @field expand_map boolean? If the destination surface should be expanded when destination_area is outside current bounds. Default false.
--- @field manual_collision_mode boolean? If manual-style collision checks should be done.
--- @field create_build_effect_smoke boolean? If true, the building effect smoke will be shown around the new entities.
----

----
--- Clones the given area.
--- 
--- @param args LuaSurface.clone_brush_parameters Table containing arguments for the method.
--- @see LuaSurface.clone_brush_parameters
----
function LuaSurface.clone_brush(args) end

----
--- @class LuaSurface.clone_entities_parameters
--- Parameters class for the method `LuaSurface.clone_entities`
--- 
--- @field entities LuaEntity[] 
--- @field destination_offset Vector 
--- @field destination_surface SurfaceIdentification? 
--- @field destination_force ForceIdentification? 
--- @field snap_to_grid boolean? 
--- @field create_build_effect_smoke boolean? If true, the building effect smoke will be shown around the new entities.
----

----
--- Clones the given entities.
--- 
--- @param args LuaSurface.clone_entities_parameters Table containing arguments for the method.
--- @see LuaSurface.clone_entities_parameters
----
function LuaSurface.clone_entities(args) end

----
--- @class LuaSurface.count_entities_filtered_parameters
--- Parameters class for the method `LuaSurface.count_entities_filtered`
--- 
--- @field area BoundingBox? 
--- @field position MapPosition? 
--- @field radius double? If given with position, will count all entities within the radius of the position.
--- @field name string|string[]? 
--- @field type string|string[]? 
--- @field ghost_name string|string[]? 
--- @field ghost_type string|string[]? 
--- @field direction defines.direction[]|defines.direction? 
--- @field collision_mask CollisionMaskLayer|CollisionMaskLayer[]? 
--- @field force ForceIdentification|ForceIdentification[]? 
--- @field to_be_deconstructed boolean? 
--- @field to_be_upgraded boolean? 
--- @field limit uint? 
--- @field is_military_target boolean? 
--- @field invert boolean? Whether the filters should be inverted.
----

----
--- Count entities of given type or name in a given area. Works just like [LuaSurface::find_entities_filtered](LuaSurface::find_entities_filtered), except this only returns the count. As it doesn't construct all the wrapper objects, this is more efficient if one is only interested in the number of entities.
--- 
--- If no `area` or `position` are given, the entire surface is searched. If `position` is given, this returns the entities colliding with that position (i.e the given position is within the entity's collision box). If `position` and `radius` are given, this returns entities in the radius of the position. If `area` is specified, this returns entities colliding with that area.
--- 
--- @param args LuaSurface.count_entities_filtered_parameters Table containing arguments for the method.
--- @see LuaSurface.count_entities_filtered_parameters
--- @return uint 
----
function LuaSurface.count_entities_filtered(args) end

----
--- @class LuaSurface.count_tiles_filtered_parameters
--- Parameters class for the method `LuaSurface.count_tiles_filtered`
--- 
--- @field area BoundingBox? 
--- @field position MapPosition? Ignored if not given with radius.
--- @field radius double? If given with position, will return all entities within the radius of the position.
--- @field name string|string[]? 
--- @field force ForceIdentification|ForceIdentification[]? 
--- @field limit uint? 
--- @field has_hidden_tile boolean? 
--- @field has_tile_ghost boolean? Can be further filtered by supplying a `force` filter.
--- @field to_be_deconstructed boolean? Can be further filtered by supplying a `force` filter.
--- @field collision_mask CollisionMaskLayer|CollisionMaskLayer[]? 
--- @field invert boolean? If the filters should be inverted.
----

----
--- Count tiles of a given name in a given area. Works just like [LuaSurface::find_tiles_filtered](LuaSurface::find_tiles_filtered), except this only returns the count. As it doesn't construct all the wrapper objects, this is more efficient if one is only interested in the number of tiles.
--- 
--- If no `area` or `position` and `radius` is given, the entire surface is searched. If `position` and `radius` are given, only tiles within the radius of the position are included.
--- 
--- @param args LuaSurface.count_tiles_filtered_parameters Table containing arguments for the method.
--- @see LuaSurface.count_tiles_filtered_parameters
--- @return uint 
----
function LuaSurface.count_tiles_filtered(args) end

----
--- @class LuaSurface.create_decoratives_parameters
--- Parameters class for the method `LuaSurface.create_decoratives`
--- 
--- @field check_collision boolean? If collision should be checked against entities/tiles.
--- @field decoratives Decorative[] 
----

----
--- Adds the given decoratives to the surface.
--- 
--- @param args LuaSurface.create_decoratives_parameters Table containing arguments for the method.
--- @see LuaSurface.create_decoratives_parameters
----
function LuaSurface.create_decoratives(args) end

----
--- @class LuaSurface.create_entity_parameters
--- Parameters class for the method `LuaSurface.create_entity`
--- 
--- @field name string The entity prototype name to create.
--- @field position MapPosition Where to create the entity.
--- @field direction defines.direction? Desired orientation of the entity after creation.
--- @field force ForceIdentification? Force of the entity, default is enemy.
--- @field target MapPosition|LuaEntity? Entity with health for the new entity to target.
--- @field source MapPosition|LuaEntity? Source entity. Used for beams, projectiles, and highlight-boxes.
--- @field fast_replace boolean? If true, building will attempt to simulate fast-replace building.
--- @field player PlayerIdentification? If given set the last_user to this player. If fast_replace is true simulate fast replace using this player.
--- @field character LuaEntity? If fast_replace is true simulate fast replace using this character.
--- @field spill boolean? If false while fast_replace is true and player is nil any items from fast-replacing will be deleted instead of dropped on the ground.
--- @field raise_built boolean? If true; [defines.events.script_raised_built](defines.events.script_raised_built) will be fired on successful entity creation.
--- @field create_build_effect_smoke boolean? If false, the building effect smoke will not be shown around the new entity.
--- @field spawn_decorations boolean? If true, entity types that have spawn_decorations property will apply triggers defined in the property.
--- @field move_stuck_players boolean? If true, any characters that are in the way of the entity are teleported out of the way.
--- @field item LuaItemStack? If provided, the entity will attempt to pull stored values from this item (for example; creating a spidertron from a previously named and mined spidertron)
----

----
--- Create an entity on this surface.
--- 
--- @param args LuaSurface.create_entity_parameters Table containing arguments for the method.
--- @see LuaSurface.create_entity_parameters
--- @return LuaEntity? The created entity or `nil` if the creation failed.
----
function LuaSurface.create_entity(args) end

----
--- @class LuaSurface.create_particle_parameters
--- Parameters class for the method `LuaSurface.create_particle`
--- 
--- @field name string The particle name.
--- @field position MapPosition Where to create the particle.
--- @field movement Vector 
--- @field height float 
--- @field vertical_speed float 
--- @field frame_speed float 
----

----
--- Creates a particle at the given location
--- 
--- @param args LuaSurface.create_particle_parameters Table containing arguments for the method.
--- @see LuaSurface.create_particle_parameters
----
function LuaSurface.create_particle(args) end

----
--- @class LuaSurface.create_trivial_smoke_parameters
--- Parameters class for the method `LuaSurface.create_trivial_smoke`
--- 
--- @field name string The smoke prototype name to create.
--- @field position MapPosition Where to create the smoke.
----

----
--- 
--- @param args LuaSurface.create_trivial_smoke_parameters Table containing arguments for the method.
--- @see LuaSurface.create_trivial_smoke_parameters
----
function LuaSurface.create_trivial_smoke(args) end

----
--- @class LuaSurface.create_unit_group_parameters
--- Parameters class for the method `LuaSurface.create_unit_group`
--- 
--- @field position MapPosition Initial position of the new unit group.
--- @field force ForceIdentification? Force of the new unit group. Defaults to `"enemy"`.
----

----
--- Create a new unit group at a given position.
--- 
--- @param args LuaSurface.create_unit_group_parameters Table containing arguments for the method.
--- @see LuaSurface.create_unit_group_parameters
--- @return LuaUnitGroup 
----
function LuaSurface.create_unit_group(args) end

----
--- @class LuaSurface.deconstruct_area_parameters
--- Parameters class for the method `LuaSurface.deconstruct_area`
--- 
--- @field area BoundingBox The area to mark for deconstruction.
--- @field force ForceIdentification The force whose bots should perform the deconstruction.
--- @field player PlayerIdentification? The player to set the last_user to if any.
--- @field skip_fog_of_war boolean? If chunks covered by fog-of-war are skipped.
--- @field item LuaItemStack? The deconstruction item to use if any.
----

----
--- Place a deconstruction request.
--- 
--- @param args LuaSurface.deconstruct_area_parameters Table containing arguments for the method.
--- @see LuaSurface.deconstruct_area_parameters
----
function LuaSurface.deconstruct_area(args) end

----
--- Whether the given decorative prototype collides at the given position and direction.
--- 
--- @param prototype string The decorative prototype to check
--- @param position MapPosition The position to check
--- @return boolean 
----
function LuaSurface.decorative_prototype_collides(prototype, position) end

----
--- 
--- @param position ChunkPosition The chunk position to delete
----
function LuaSurface.delete_chunk(position) end

----
--- @class LuaSurface.destroy_decoratives_parameters
--- Parameters class for the method `LuaSurface.destroy_decoratives`
--- 
--- @field area BoundingBox? 
--- @field position TilePosition? 
--- @field name LuaDecorativePrototype|string|string[]|LuaDecorativePrototype[]? 
--- @field collision_mask CollisionMaskLayer|CollisionMaskLayer[]? 
--- @field from_layer string? 
--- @field to_layer string? 
--- @field exclude_soft boolean? Soft decoratives can be drawn over rails.
--- @field limit uint? 
--- @field invert boolean? If the filters should be inverted.
----

----
--- Removes all decoratives from the given area. If no area and no position are given, then the entire surface is searched.
--- 
--- @param args LuaSurface.destroy_decoratives_parameters Table containing arguments for the method.
--- @see LuaSurface.destroy_decoratives_parameters
----
function LuaSurface.destroy_decoratives(args) end

----
--- Sets the given script area to the new values.
--- 
--- @param id uint The area to edit.
--- @param area ScriptArea 
----
function LuaSurface.edit_script_area(id, area) end

----
--- Sets the given script position to the new values.
--- 
--- @param id uint The position to edit.
--- @param area ScriptPosition 
----
function LuaSurface.edit_script_position(id, area) end

----
--- Whether the given entity prototype collides at the given position and direction.
--- 
--- @param prototype EntityPrototypeIdentification The entity prototype to check
--- @param position MapPosition The position to check
--- @param use_map_generation_bounding_box boolean If the map generation bounding box should be used instead of the collision bounding box
--- @param direction defines.direction? 
--- @return boolean 
----
function LuaSurface.entity_prototype_collides(prototype, position, use_map_generation_bounding_box, direction) end

----
--- @class LuaSurface.find_decoratives_filtered_parameters
--- Parameters class for the method `LuaSurface.find_decoratives_filtered`
--- 
--- @field area BoundingBox? 
--- @field position TilePosition? 
--- @field name LuaDecorativePrototype|string|string[]|LuaDecorativePrototype[]? 
--- @field collision_mask CollisionMaskLayer|CollisionMaskLayer[]? 
--- @field from_layer string? 
--- @field to_layer string? 
--- @field exclude_soft boolean? Soft decoratives can be drawn over rails.
--- @field limit uint? 
--- @field invert boolean? If the filters should be inverted.
----

----
--- Find decoratives of a given name in a given area.
--- 
--- If no filters are given, returns all decoratives in the search area. If multiple filters are specified, returns only decoratives matching every given filter. If no area and no position are given, the entire surface is searched.
--- 
--- @param args LuaSurface.find_decoratives_filtered_parameters Table containing arguments for the method.
--- @see LuaSurface.find_decoratives_filtered_parameters
--- @return DecorativeResult[] 
----
function LuaSurface.find_decoratives_filtered(args) end

----
--- Find enemy units (entities with type "unit") of a given force within an area.
--- 
--- @param center MapPosition Center of the search area
--- @param radius double Radius of the circular search area
--- @param force string|LuaForce? Force to find enemies of. If not given, uses the player force.
--- @return LuaEntity[] 
----
function LuaSurface.find_enemy_units(center, radius, force) end

----
--- Find entities in a given area.
--- 
--- If no area is given all entities on the surface are returned.
--- 
--- @param area BoundingBox? 
--- @return LuaEntity[] 
----
function LuaSurface.find_entities(area) end

----
--- @class LuaSurface.find_entities_filtered_parameters
--- Parameters class for the method `LuaSurface.find_entities_filtered`
--- 
--- @field area BoundingBox? 
--- @field position MapPosition? Has precedence over area field.
--- @field radius double? 
--- @field name string|string[]? 
--- @field type string|string[]? 
--- @field ghost_name string|string[]? 
--- @field ghost_type string|string[]? 
--- @field direction defines.direction[]|defines.direction? 
--- @field collision_mask CollisionMaskLayer|CollisionMaskLayer[]? 
--- @field force ForceIdentification|ForceIdentification[]? 
--- @field to_be_deconstructed boolean? 
--- @field to_be_upgraded boolean? 
--- @field limit uint? 
--- @field is_military_target boolean? 
--- @field has_item_inside LuaItemPrototype? 
--- @field invert boolean? Whether the filters should be inverted.
----

----
--- Find all entities of the given type or name in the given area.
--- 
--- If no filters (`name`, `type`, `force`, etc.) are given, this returns all entities in the search area. If multiple filters are specified, only entities matching all given filters are returned.
--- 
--- - If no `area` or `position` are given, the entire surface is searched.
--- - If `position` is given, this returns the entities colliding with that position (i.e the given position is within the entity's collision box).
--- - If `position` and `radius` are given, this returns the entities within the radius of the position. Looks for the center of entities.
--- - If `area` is specified, this returns the entities colliding with that area.
--- 
--- @param args LuaSurface.find_entities_filtered_parameters Table containing arguments for the method.
--- @see LuaSurface.find_entities_filtered_parameters
--- @return LuaEntity[] 
----
function LuaSurface.find_entities_filtered(args) end

----
--- Find a specific entity at a specific position.
--- 
--- @param entity string Entity to look for.
--- @param position MapPosition Coordinates to look at.
--- @return LuaEntity? `nil` if no such entity is found.
----
function LuaSurface.find_entity(entity, position) end

----
--- Find the logistic network that covers a given position.
--- 
--- @param position MapPosition 
--- @param force ForceIdentification Force the logistic network should belong to.
--- @return LuaLogisticNetwork? The found network or `nil` if no such network was found.
----
function LuaSurface.find_logistic_network_by_position(position, force) end

----
--- Finds all of the logistics networks whose construction area intersects with the given position.
--- 
--- @param position MapPosition 
--- @param force ForceIdentification Force the logistic networks should belong to.
--- @return LuaLogisticNetwork[] 
----
function LuaSurface.find_logistic_networks_by_construction_area(position, force) end

----
--- @class LuaSurface.find_nearest_enemy_parameters
--- Parameters class for the method `LuaSurface.find_nearest_enemy`
--- 
--- @field position MapPosition Center of the search area.
--- @field max_distance double Radius of the circular search area.
--- @field force ForceIdentification? The force the result will be an enemy of. Uses the player force if not specified.
----

----
--- Find the enemy military target ([military entity](https://wiki.factorio.com/Military_units_and_structures)) closest to the given position.
--- 
--- @param args LuaSurface.find_nearest_enemy_parameters Table containing arguments for the method.
--- @see LuaSurface.find_nearest_enemy_parameters
--- @return LuaEntity? The nearest enemy military target or `nil` if no enemy could be found within the given area.
----
function LuaSurface.find_nearest_enemy(args) end

----
--- @class LuaSurface.find_nearest_enemy_entity_with_owner_parameters
--- Parameters class for the method `LuaSurface.find_nearest_enemy_entity_with_owner`
--- 
--- @field position MapPosition Center of the search area.
--- @field max_distance double Radius of the circular search area.
--- @field force ForceIdentification? The force the result will be an enemy of. Uses the player force if not specified.
----

----
--- Find the enemy entity-with-owner closest to the given position.
--- 
--- @param args LuaSurface.find_nearest_enemy_entity_with_owner_parameters Table containing arguments for the method.
--- @see LuaSurface.find_nearest_enemy_entity_with_owner_parameters
--- @return LuaEntity The nearest enemy entity-with-owner or `nil` if no enemy could be found within the given area.
----
function LuaSurface.find_nearest_enemy_entity_with_owner(args) end

----
--- Find a non-colliding position within a given radius.
--- 
--- @param name string Prototype name of the entity to find a position for. (The bounding box for the collision checking is taken from this prototype.)
--- @param center MapPosition Center of the search area.
--- @param radius double Max distance from `center` to search in. A radius of `0` means an infinitely-large search area.
--- @param precision double The step length from the given position as it searches, in tiles. Minimum value is `0.01`.
--- @param force_to_tile_center boolean? Will only check tile centers. This can be useful when your intent is to place a building at the resulting position, as they must generally be placed at tile centers. Default false.
--- @return MapPosition? The non-colliding position. May be `nil` if no suitable position was found.
----
function LuaSurface.find_non_colliding_position(name, center, radius, precision, force_to_tile_center) end

----
--- Find a non-colliding position within a given rectangle.
--- 
--- @param name string Prototype name of the entity to find a position for. (The bounding box for the collision checking is taken from this prototype.)
--- @param search_space BoundingBox The rectangle to search inside.
--- @param precision double The step length from the given position as it searches, in tiles. Minimum value is 0.01.
--- @param force_to_tile_center boolean? Will only check tile centers. This can be useful when your intent is to place a building at the resulting position, as they must generally be placed at tile centers. Default false.
--- @return MapPosition? The non-colliding position. May be `nil` if no suitable position was found.
----
function LuaSurface.find_non_colliding_position_in_box(name, search_space, precision, force_to_tile_center) end

----
--- @class LuaSurface.find_tiles_filtered_parameters
--- Parameters class for the method `LuaSurface.find_tiles_filtered`
--- 
--- @field area BoundingBox? 
--- @field position MapPosition? Ignored if not given with radius.
--- @field radius double? If given with position, will return all entities within the radius of the position.
--- @field name string|string[]? 
--- @field force ForceIdentification|ForceIdentification[]? 
--- @field limit uint? 
--- @field has_hidden_tile boolean? 
--- @field has_tile_ghost boolean? Can be further filtered by supplying a `force` filter.
--- @field to_be_deconstructed boolean? Can be further filtered by supplying a `force` filter.
--- @field collision_mask CollisionMaskLayer|CollisionMaskLayer[]? 
--- @field invert boolean? Whether the filters should be inverted.
----

----
--- Find all tiles of the given name in the given area.
--- 
--- If no filters are given, this returns all tiles in the search area.
--- 
--- If no `area` or `position` and `radius` is given, the entire surface is searched. If `position` and `radius` are given, only tiles within the radius of the position are included.
--- 
--- @param args LuaSurface.find_tiles_filtered_parameters Table containing arguments for the method.
--- @see LuaSurface.find_tiles_filtered_parameters
--- @return LuaTile[] 
----
function LuaSurface.find_tiles_filtered(args) end

----
--- @class LuaSurface.find_units_parameters
--- Parameters class for the method `LuaSurface.find_units`
--- 
--- @field area BoundingBox Box to find units within.
--- @field force string|LuaForce Force performing the search.
--- @field condition ForceCondition Only forces which meet the condition will be included in the search.
----

----
--- Find units (entities with type "unit") of a given force and force condition within a given area.
--- 
--- @param args LuaSurface.find_units_parameters Table containing arguments for the method.
--- @see LuaSurface.find_units_parameters
--- @return LuaEntity[] 
----
function LuaSurface.find_units(args) end

----
--- Blocks and generates all chunks that have been requested using all available threads.
--- 
----
function LuaSurface.force_generate_chunk_requests() end

----
--- Get an iterator going over every chunk on this surface.
--- 
--- @return LuaChunkIterator 
----
function LuaSurface.get_chunks() end

----
--- Gets the closest entity in the list to this position.
--- 
--- @param position MapPosition 
--- @param entities LuaEntity[] The Entities to check
--- @return LuaEntity? 
----
function LuaSurface.get_closest(position, entities) end

----
--- Gets all tiles of the given types that are connected horizontally or vertically to the given tile position including the given tile position.
--- 
--- @param position TilePosition The tile position to start at.
--- @param tiles string[] The tiles to search for.
--- @return TilePosition[] The resulting set of tiles.
----
function LuaSurface.get_connected_tiles(position, tiles) end

----
--- Returns all the military targets (entities with force) on this chunk for the given force.
--- 
--- @param position ChunkPosition The chunk's position.
--- @param force string|LuaForce Entities of this force will be returned.
--- @return LuaEntity[] 
----
function LuaSurface.get_entities_with_force(position, force) end

