require('.classes-1')
require('.classes-2')
require('.classes-3')

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaEquipmentGrid.help() end

----
--- @class LuaEquipmentGrid.move_parameters
--- Parameters class for the method `LuaEquipmentGrid.move`
--- 
--- @field equipment LuaEquipment The equipment to move
--- @field position EquipmentPosition Where to put it
----

----
--- Move an equipment within this grid.
--- 
--- @param args LuaEquipmentGrid.move_parameters Table containing arguments for the method.
--- @see LuaEquipmentGrid.move_parameters
--- @return boolean `true` if the equipment was successfully moved.
----
function LuaEquipmentGrid.move(args) end

----
--- @class LuaEquipmentGrid.put_parameters
--- Parameters class for the method `LuaEquipmentGrid.put`
--- 
--- @field name string Equipment prototype name
--- @field position EquipmentPosition? Grid position to put the equipment in.
--- @field by_player PlayerIdentification? If provided the action is done 'as' this player and [on_player_placed_equipment](on_player_placed_equipment) is triggered.
----

----
--- Insert an equipment into the grid.
--- 
--- @param args LuaEquipmentGrid.put_parameters Table containing arguments for the method.
--- @see LuaEquipmentGrid.put_parameters
--- @return LuaEquipment? The newly-added equipment, or `nil` if the equipment could not be added.
----
function LuaEquipmentGrid.put(args) end

----
--- @class LuaEquipmentGrid.take_parameters
--- Parameters class for the method `LuaEquipmentGrid.take`
--- 
--- @field position EquipmentPosition? Take the equipment that contains this position in the grid.
--- @field equipment LuaEquipment? Take this exact equipment.
--- @field by_player PlayerIdentification? If provided the action is done 'as' this player and [on_player_removed_equipment](on_player_removed_equipment) is triggered.
----

----
--- Remove an equipment from the grid.
--- 
--- @param args LuaEquipmentGrid.take_parameters Table containing arguments for the method.
--- @see LuaEquipmentGrid.take_parameters
--- @return SimpleItemStack? The removed equipment, or `nil` if no equipment was removed.
----
function LuaEquipmentGrid.take(args) end

----
--- Remove all equipment from the grid.
--- 
--- @param by_player PlayerIdentification? If provided, the action is done 'as' this player and [on_player_removed_equipment](on_player_removed_equipment) is triggered.
--- @return table<string,uint> Count of each removed equipment, indexed by their prototype names.
----
function LuaEquipmentGrid.take_all(by_player) end


----
--- @class LuaEquipmentGridPrototype
--- Prototype of an equipment grid.
--- 
--- @field name string Name of this prototype.
--- @field order string The string used to alphabetically sort these prototypes. It is a simple string that has no additional semantic meaning.
--- @field localised_name LocalisedString 
--- @field localised_description LocalisedString 
--- @field equipment_categories string[] Equipment category names for the [categories](LuaEquipmentPrototype::equipment_categories) that may be inserted into this equipment grid. The grid will accept any equipment that has at least one category in this list.
--- @field width uint 
--- @field height uint 
--- @field locked boolean If the player can move equipment into or out of this grid.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaEquipmentGridPrototype = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaEquipmentGridPrototype.help() end


----
--- @class LuaEquipmentPrototype
--- Prototype of a modular equipment.
--- 
--- @field name string Name of this prototype.
--- @field type string Type of this equipment prototype.
--- @field order string The string used to alphabetically sort these prototypes. It is a simple string that has no additional semantic meaning.
--- @field localised_name LocalisedString 
--- @field localised_description LocalisedString 
--- @field shape { height: uint, points: EquipmentPoint[], width: uint } Shape of this equipment prototype.
--- @field take_result LuaItemPrototype? The result item when taking this equipment out of an equipment grid, if any.
--- @field energy_production double The max power generated by this equipment.
--- @field shield float The shield value of this equipment. 0 for non-shield equipment.
--- @field energy_per_shield double The energy per shield point restored. 0 for non-shield equipment.
--- @field logistic_parameters { charge_approach_distance: float, charging_distance: float, charging_energy: double, charging_station_count: uint, charging_station_shift: Vector, charging_threshold_distance: float, construction_radius: float, logistic_radius: float, logistics_connection_distance: float, robot_limit: uint, robot_vertical_acceleration: float, robots_shrink_when_entering_and_exiting: boolean, spawn_and_station_height: float, spawn_and_station_shadow_height_offset: float, stationing_offset: Vector }? The logistic parameters for this roboport equipment.
--- @field energy_consumption double 
--- @field movement_bonus float? 
--- @field energy_source LuaElectricEnergySourcePrototype The energy source prototype for the equipment.
--- @field equipment_categories string[] Category names for this equipment. These [categories](LuaEquipmentGridPrototype::equipment_categories) will be used to determine whether this equipment is allowed in a particular equipment grid.
--- @field burner_prototype LuaBurnerPrototype? The burner energy source prototype this equipment uses, if any.
--- @field electric_energy_source_prototype LuaElectricEnergySourcePrototype? The electric energy source prototype this equipment uses, if any.
--- @field background_color Color The background color of this equipment prototype.
--- @field attack_parameters AttackParameters? The equipment attack parameters.
--- @field automatic boolean Whether this active defense equipment is automatic. Returns false if not active defense equipment.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaEquipmentPrototype = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaEquipmentPrototype.help() end


----
--- @class LuaFlowStatistics
--- Encapsulates statistic data for different parts of the game. In the context of flow statistics, `input` and `output` describe on which side of the associated GUI the values are shown. Input values are shown on the left side, output values on the right side.
--- 
--- Examples:
--- - The item production GUI shows "consumption" on the right, thus `output` describes the item consumption numbers. The same goes for fluid consumption.
--- - The kills GUI shows "losses" on the right, so `output` describes how many of the force's entities were killed by enemies.
--- - The electric network GUI shows "power consumption" on the left side, so in this case `input` describes the power consumption numbers.
--- 
--- @field input_counts table<string,double|uint64> List of input counts indexed by prototype name. Represents the data that is shown on the left side of the GUI for the given statistics.
--- @field output_counts table<string,double|uint64> List of output counts indexed by prototype name. Represents the data that is shown on the right side of the GUI for the given statistics.
--- @field force LuaForce? The force these statistics belong to. `nil` for pollution statistics.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaFlowStatistics = { }

----
--- Reset all the statistics data to 0.
--- 
----
function LuaFlowStatistics.clear() end

----
--- @class LuaFlowStatistics.get_flow_count_parameters
--- Parameters class for the method `LuaFlowStatistics.get_flow_count`
--- 
--- @field name string The prototype name.
--- @field input boolean Read the input values or the output values
--- @field precision_index defines.flow_precision_index The precision range to read.
--- @field sample_index uint16? The sample index to read from within the precision range. If not provided, the entire precision range is read. Must be between 1 and 300 where 1 is the most recent sample and 300 is the oldest.
--- @field count boolean? If true, the count of items/fluids/entities is returned instead of the per-time-frame value.
----

----
--- Gets the flow count value for the given time frame. If `sample_index` is not provided, then the value returned is the average across the provided precision time period. These are the values shown in the bottom section of the statistics GUIs.
--- 
--- Use `sample_index` to access the data used to generate the statistics graphs. Each precision level contains 300 samples of data so at a precision of 1 minute, each sample contains data averaged across 60s / 300 = 0.2s = 12 ticks.
--- 
--- All return values are normalized to be per-tick for electric networks and per-minute for all other types.
--- 
--- @param args LuaFlowStatistics.get_flow_count_parameters Table containing arguments for the method.
--- @see LuaFlowStatistics.get_flow_count_parameters
--- @return double 
----
function LuaFlowStatistics.get_flow_count(args) end

----
--- Gets the total input count for a given prototype.
--- 
--- @param name string The prototype name.
--- @return double|uint64 
----
function LuaFlowStatistics.get_input_count(name) end

----
--- Gets the total output count for a given prototype.
--- 
--- @param name string The prototype name.
--- @return double|uint64 
----
function LuaFlowStatistics.get_output_count(name) end

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaFlowStatistics.help() end

----
--- Adds a value to this flow statistics.
--- 
--- @param name string The prototype name.
--- @param count float The count: positive or negative determines if the value goes in the input or output statistics.
----
function LuaFlowStatistics.on_flow(name, count) end

----
--- Sets the total input count for a given prototype.
--- 
--- @param name string The prototype name.
--- @param count double|uint64 The new count. The type depends on the instance of the statistics.
----
function LuaFlowStatistics.set_input_count(name, count) end

----
--- Sets the total output count for a given prototype.
--- 
--- @param name string The prototype name.
--- @param count double|uint64 The new count. The type depends on the instance of the statistics.
----
function LuaFlowStatistics.set_output_count(name, count) end


----
--- @class LuaFluidBox
--- An array of fluid boxes of an entity. Entities may contain more than one fluid box, and some can change the number of fluid boxes -- for instance, an assembling machine will change its number of fluid boxes depending on its active recipe. See [Fluid](Fluid).
--- 
--- Do note that reading from a [LuaFluidBox](LuaFluidBox) creates a new table and writing will copy the given fields from the table into the engine's own fluid box structure. Therefore, the correct way to update a fluidbox of an entity is to read it first, modify the table, then write the modified table back. Directly accessing the returned table's attributes won't have the desired effect.
--- 
--- @field owner LuaEntity The entity that owns this fluidbox.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaFluidBox = { }

----
--- Flushes all fluid from this fluidbox and its fluid system.
--- 
--- @param index uint 
--- @param fluid FluidIdentification? If provided, only this fluid is flushed.
--- @return table<string,float> The removed fluid.
----
function LuaFluidBox.flush(index, fluid) end

----
--- The capacity of the given fluidbox index.
--- 
--- @param index uint 
--- @return double 
----
function LuaFluidBox.get_capacity(index) end

----
--- The fluidboxes to which the fluidbox at the given index is connected.
--- 
--- @param index uint 
--- @return LuaFluidBox[] 
----
function LuaFluidBox.get_connections(index) end

----
--- Get a fluid box filter
--- 
--- @param index uint The index of the filter to get.
--- @return FluidBoxFilter? The filter at the requested index, or `nil` if there isn't one.
----
function LuaFluidBox.get_filter(index) end

----
--- Flow through the fluidbox in the last tick. It is the larger of in-flow and out-flow.
--- 
--- @param index uint 
--- @return double 
----
function LuaFluidBox.get_flow(index) end

----
--- Gets unique fluid system identifier of selected fluid box. May return nil for fluid wagon, fluid turret's internal buffer or a fluidbox which does not belong to a fluid system
--- 
--- @param index uint 
--- @return uint 
----
function LuaFluidBox.get_fluid_system_id(index) end

----
--- Returns the fluid the fluidbox is locked onto
--- 
--- @param index uint 
--- @return string? `nil` if the fluidbox is not locked to any fluid.
----
function LuaFluidBox.get_locked_fluid(index) end

----
--- The prototype of this fluidbox index. If this is used on a fluidbox of a crafting machine which due to recipe was created by merging multiple prototypes, a table of prototypes that were merged will be returned instead
--- 
--- @param index uint 
--- @return LuaFluidBoxPrototype|LuaFluidBoxPrototype[] 
----
function LuaFluidBox.get_prototype(index) end

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaFluidBox.help() end

----
--- Set a fluid box filter.
--- 
--- @param index uint The index of the filter to set.
--- @param filter FluidBoxFilterSpec|nil The filter to set. Setting `nil` clears the filter.
--- @return boolean Whether the filter was set successfully.
----
function LuaFluidBox.set_filter(index, filter) end


----
--- @class LuaFluidBoxPrototype
--- A prototype of a fluidbox owned by some [LuaEntityPrototype](LuaEntityPrototype).
--- 
--- @field entity LuaEntityPrototype The entity that this belongs to.
--- @field index uint The index of this fluidbox prototype in the owning entity.
--- @field pipe_connections FluidBoxConnection[] The pipe connection points.
--- @field production_type string The production type. "input", "output", "input-output", or "none".
--- @field base_area double 
--- @field base_level double 
--- @field height double 
--- @field volume double 
--- @field filter LuaFluidPrototype? The filter, if any is set.
--- @field minimum_temperature double? The minimum temperature, if any is set.
--- @field maximum_temperature double? The maximum temperature, if any is set.
--- @field secondary_draw_orders int[] The secondary draw orders for the 4 possible connection directions.
--- @field render_layer string The render layer.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaFluidBoxPrototype = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaFluidBoxPrototype.help() end


----
--- @class LuaFluidEnergySourcePrototype
--- Prototype of a fluid energy source.
--- 
--- @field emissions double The emissions of this energy source in `pollution/Joule`. Multiplying it by energy consumption in `Watt` gives `pollution/second`.
--- @field render_no_network_icon boolean 
--- @field render_no_power_icon boolean 
--- @field effectivity double 
--- @field burns_fluid boolean 
--- @field scale_fluid_usage boolean 
--- @field destroy_non_fuel_fluid boolean 
--- @field fluid_usage_per_tick double 
--- @field smoke SmokeSource[] The smoke sources for this prototype, if any.
--- @field maximum_temperature double 
--- @field fluid_box LuaFluidBoxPrototype The fluid box for this energy source.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaFluidEnergySourcePrototype = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaFluidEnergySourcePrototype.help() end


----
--- @class LuaFluidPrototype
--- Prototype of a fluid.
--- 
--- @field name string Name of this prototype.
--- @field localised_name LocalisedString 
--- @field localised_description LocalisedString 
--- @field default_temperature double Default temperature of this fluid.
--- @field max_temperature double Maximum temperature this fluid can reach.
--- @field heat_capacity double The amount of energy in Joules required to heat one unit of this fluid by 1°C.
--- @field order string The string used to alphabetically sort these prototypes. It is a simple string that has no additional semantic meaning.
--- @field group LuaGroup Group of this prototype.
--- @field subgroup LuaGroup Subgroup of this prototype.
--- @field base_color Color 
--- @field flow_color Color 
--- @field gas_temperature double The temperature above which this fluid will be shown as gaseous inside tanks and pipes.
--- @field emissions_multiplier double A multiplier on the amount of emissions produced when this fluid is burnt in a generator. A value above `1.0` increases emissions and vice versa. The multiplier can't be negative.
--- @field fuel_value double The amount of energy in Joules one unit of this fluid will produce when burnt in a generator. A value of `0` means this fluid can't be used for energy generation. The value can't be negative.
--- @field hidden boolean Whether this fluid is hidden from the fluid and signal selectors.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaFluidPrototype = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaFluidPrototype.help() end


----
--- @class LuaFontPrototype
--- Prototype of a font.
--- 
--- @field name string Name of this prototype.
--- @field from string 
--- @field size int 
--- @field spacing float 
--- @field border boolean 
--- @field filtered boolean 
--- @field border_color Color? The border color, if any.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaFontPrototype = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaFontPrototype.help() end


----
--- @class LuaForce
--- `LuaForce` encapsulates data local to each "force" or "faction" of the game. Default forces are player, enemy and neutral. Players and mods can create additional forces (up to 64 total).
--- 
--- @field name string Name of the force.
--- @field technologies table<string,LuaTechnology> Technologies owned by this force, indexed by `name`.
--- @field recipes table<string,LuaRecipe> Recipes available to this force, indexed by `name`.
--- @field manual_mining_speed_modifier double Multiplier of the manual mining speed. Default value is `0`. The actual mining speed will be multiplied by `1 + manual_mining_speed_modifier`.
--- @field manual_crafting_speed_modifier double Multiplier of the manual crafting speed. Default value is `0`. The actual crafting speed will be multiplied by `1 + manual_crafting_speed_modifier`.
--- @field laboratory_speed_modifier double 
--- @field laboratory_productivity_bonus double 
--- @field worker_robots_speed_modifier double 
--- @field worker_robots_battery_modifier double 
--- @field worker_robots_storage_bonus double 
--- @field current_research LuaTechnology? The currently ongoing technology research, if any.
--- @field research_progress double Progress of current research, as a number in range [0, 1].
--- @field previous_research LuaTechnology? The previous research, if any.
--- @field inserter_stack_size_bonus double The inserter stack size bonus for non stack inserters
--- @field stack_inserter_capacity_bonus uint Number of items that can be transferred by stack inserters. When writing to this value, it must be >= 0 and <= 254.
--- @field character_trash_slot_count double Number of character trash slots.
--- @field maximum_following_robot_count uint Maximum number of follower robots.
--- @field following_robots_lifetime_modifier double Additional lifetime for following robots.
--- @field ghost_time_to_live uint The time, in ticks, before a placed ghost disappears.
--- @field players LuaPlayer[] Players belonging to this force.
--- @field ai_controllable boolean Enables some higher-level AI behaviour for this force. When set to `true`, biters belonging to this force will automatically expand into new territories, build new spawners, and form unit groups. By default, this value is `true` for the enemy force and `false` for all others.
--- @field logistic_networks table<string,LuaLogisticNetwork[]> List of logistic networks, grouped by surface.
--- @field item_production_statistics LuaFlowStatistics The item production statistics for this force.
--- @field fluid_production_statistics LuaFlowStatistics The fluid production statistics for this force.
--- @field kill_count_statistics LuaFlowStatistics The kill counter statistics for this force.
--- @field entity_build_count_statistics LuaFlowStatistics The entity build statistics for this force (built and mined)
--- @field character_running_speed_modifier double Modifies the running speed of all characters in this force by the given value as a percentage. Setting the running modifier to `0.5` makes the character run 50% faster. The minimum value of `-1` reduces the movement speed by 100%, resulting in a speed of `0`.
--- @field artillery_range_modifier double 
--- @field character_build_distance_bonus uint 
--- @field character_item_drop_distance_bonus uint 
--- @field character_reach_distance_bonus uint 
--- @field character_resource_reach_distance_bonus double 
--- @field character_item_pickup_distance_bonus double 
--- @field character_loot_pickup_distance_bonus double 
--- @field character_inventory_slots_bonus uint the number of additional inventory slots the character main inventory has.
--- @field deconstruction_time_to_live uint The time, in ticks, before a deconstruction order is removed.
--- @field character_health_bonus double 
--- @field max_successful_attempts_per_tick_per_construction_queue uint 
--- @field max_failed_attempts_per_tick_per_construction_queue uint 
--- @field zoom_to_world_enabled boolean Ability to use zoom-to-world on map.
--- @field zoom_to_world_ghost_building_enabled boolean Ability to build ghosts through blueprint or direct ghost placement, or "mine" ghosts when using zoom-to-world.
--- @field zoom_to_world_blueprint_enabled boolean Ability to create new blueprints using empty blueprint item when using zoom-to-world.
--- @field zoom_to_world_deconstruction_planner_enabled boolean Ability to use deconstruction planner when using zoom-to-world.
--- @field zoom_to_world_selection_tool_enabled boolean Ability to use custom selection tools when using zoom-to-world.
--- @field character_logistic_requests boolean `true` if character requester logistics is enabled.
--- @field rockets_launched uint The number of rockets launched.
--- @field items_launched table<string,uint> All of the items that have been launched in rockets. The attribute is a dictionary mapping the item prototype names to the launched amounts.
--- @field connected_players LuaPlayer[] The connected players belonging to this force.  This is primarily useful when you want to do some action against all online players of this force.
--- @field mining_drill_productivity_bonus double 
--- @field train_braking_force_bonus double 
--- @field evolution_factor double Evolution factor of this force.
--- @field evolution_factor_by_pollution double 
--- @field evolution_factor_by_time double 
--- @field evolution_factor_by_killing_spawners double 
--- @field friendly_fire boolean If friendly fire is enabled for this force.
--- @field share_chart boolean If sharing chart data is enabled for this force.
--- @field research_queue_enabled boolean Whether the research queue is available for this force.
--- @field index uint This force's index in [LuaGameScript::forces](LuaGameScript::forces) (unique ID). It is assigned when a force is created, and remains so until it is [merged](on_forces_merged) (ie. deleted). Indexes of merged forces can be reused.
--- @field research_queue TechnologyIdentification[] The research queue of this force. The first technology in the array is the currently active one. Reading this attribute gives an array of [LuaTechnology](LuaTechnology).  To write to this, the entire table must be written. Providing an empty table or `nil` will empty the research queue and cancel the current research. Writing to this when the research queue is disabled will simply set the last research in the table as the current research.
--- @field research_enabled boolean Whether research is enabled for this force, see [LuaForce::enable_research](LuaForce::enable_research) and [LuaForce::disable_research](LuaForce::disable_research)
--- @field custom_color Color? Custom color for this force. If specified, will take priority over other sources of the force color. Writing nil clears custom color. Will return nil if it was not specified or if was set to {0,0,0,0}
--- @field color Color Effective color of this force.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaForce = { }

----
--- Adds a custom chart tag to the given surface and returns the new tag or `nil` if the given position isn't valid for a chart tag.
--- 
--- @param surface SurfaceIdentification Which surface to add the tag to.
--- @param tag ChartTagSpec The tag to add.
--- @return LuaCustomChartTag? 
----
function LuaForce.add_chart_tag(surface, tag) end

----
--- Add this technology to the back of the research queue if the queue is enabled. Otherwise, set this technology to be researched now.
--- 
--- @param technology TechnologyIdentification 
--- @return boolean Whether the technology was successfully added.
----
function LuaForce.add_research(technology) end

----
--- Cancels pending chart requests for the given surface or all surfaces.
--- 
--- @param surface SurfaceIdentification? 
----
function LuaForce.cancel_charting(surface) end

----
--- Stop the research currently in progress. This will remove any dependent technologies from the research queue.
--- 
----
function LuaForce.cancel_current_research() end

----
--- Chart a portion of the map. The chart for the given area is refreshed; it creates chart for any parts of the given area that haven't been charted yet.
--- 
--- @param surface SurfaceIdentification 
--- @param area BoundingBox The area on the given surface to chart.
----
function LuaForce.chart(surface, area) end

----
--- Chart all generated chunks.
--- 
--- @param surface SurfaceIdentification? Which surface to chart or all if not given.
----
function LuaForce.chart_all(surface) end

----
--- Erases chart data for this force.
--- 
--- @param surface SurfaceIdentification? Which surface to erase chart data for or if not provided all surfaces charts are erased.
----
function LuaForce.clear_chart(surface) end

----
--- Disable all recipes and technologies. Only recipes and technologies enabled explicitly will be useable from this point.
--- 
----
function LuaForce.disable_all_prototypes() end

----
--- Disable research for this force.
--- 
----
function LuaForce.disable_research() end

----
--- Enables all recipes and technologies. The opposite of [LuaForce::disable_all_prototypes](LuaForce::disable_all_prototypes)
--- 
----
function LuaForce.enable_all_prototypes() end

----
--- Unlock all recipes.
--- 
----
function LuaForce.enable_all_recipes() end

----
--- Unlock all technologies.
--- 
----
function LuaForce.enable_all_technologies() end

----
--- Enable research for this force.
--- 
----
function LuaForce.enable_research() end

----
--- Finds all custom chart tags within the given bounding box on the given surface.
--- 
--- @param surface SurfaceIdentification 
--- @param area BoundingBox? 
--- @return LuaCustomChartTag[] 
----
function LuaForce.find_chart_tags(surface, area) end

----
--- 
--- @param position MapPosition Position to find a network for
--- @param surface SurfaceIdentification Surface to search on
--- @return LuaLogisticNetwork? The found network or `nil`.
----
function LuaForce.find_logistic_network_by_position(position, surface) end

----
--- 
--- @param ammo string Ammo category
--- @return double 
----
function LuaForce.get_ammo_damage_modifier(ammo) end

----
--- Is `other` force in this force's cease fire list?
--- 
--- @param other ForceIdentification 
--- @return boolean 
----
function LuaForce.get_cease_fire(other) end

----
--- Count entities of given type.
--- 
--- @param name string Prototype name of the entity.
--- @return uint Number of entities of given prototype belonging to this force.
----
function LuaForce.get_entity_count(name) end

----
--- Is `other` force in this force's friends list.
--- 
--- @param other ForceIdentification 
--- @return boolean 
----
function LuaForce.get_friend(other) end

----
--- 
--- @param ammo string Ammo category
--- @return double 
----
function LuaForce.get_gun_speed_modifier(ammo) end

----
--- Gets if the given recipe is explicitly disabled from being hand crafted.
--- 
--- @param recipe string|LuaRecipe 
--- @return boolean 
----
function LuaForce.get_hand_crafting_disabled_for_recipe(recipe) end

----
--- Gets the count of a given item launched in rockets.
--- 
--- @param item string The item to get
--- @return uint The count of the item that has been launched.
----
function LuaForce.get_item_launched(item) end

----
--- Gets the linked inventory for the given prototype and link ID if it exists or `nil`.
--- 
--- @param prototype EntityPrototypeIdentification 
--- @param link_id uint 
--- @return LuaInventory? 
----
function LuaForce.get_linked_inventory(prototype, link_id) end

----
--- Gets the saved progress for the given technology or `nil` if there is no saved progress.
--- 
--- @param technology TechnologyIdentification The technology
--- @return double? The progress as a percent.
----
function LuaForce.get_saved_technology_progress(technology) end

----
--- 
--- @param surface SurfaceIdentification 
--- @return MapPosition 
----
function LuaForce.get_spawn_position(surface) end

----
--- @class LuaForce.get_train_stops_parameters
--- Parameters class for the method `LuaForce.get_train_stops`
--- 
--- @field name string|string[]? The name(s) of the train stops. Not providing names will match any stop.
--- @field surface SurfaceIdentification? The surface to search. Not providing a surface will match stops on any surface.
----

----
--- Gets train stops matching the given filters.
--- 
--- @param args LuaForce.get_train_stops_parameters? Table containing arguments for the method.
--- @see LuaForce.get_train_stops_parameters
--- @return LuaEntity[] 
----
function LuaForce.get_train_stops(args) end

----
--- 
--- @param surface SurfaceIdentification? The surface to search. Not providing a surface will match trains on any surface.
--- @return LuaTrain[] 
----
function LuaForce.get_trains(surface) end

----
--- 
--- @param turret string Turret prototype name
--- @return double 
----
function LuaForce.get_turret_attack_modifier(turret) end

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaForce.help() end

----
--- Has a chunk been charted?
--- 
--- @param surface SurfaceIdentification 
--- @param position ChunkPosition Position of the chunk.
--- @return boolean 
----
function LuaForce.is_chunk_charted(surface, position) end

----
--- Is the given chunk currently charted and visible (not covered by fog of war) on the map.
--- 
--- @param surface SurfaceIdentification 
--- @param position ChunkPosition 
--- @return boolean 
----
function LuaForce.is_chunk_visible(surface, position) end

----
--- Is this force an enemy? This differs from `get_cease_fire` in that it is always false for neutral force. This is equivalent to checking the `enemy` ForceCondition.
--- 
--- @param other ForceIdentification 
--- @return boolean 
----
function LuaForce.is_enemy(other) end

----
--- Is this force a friend? This differs from `get_friend` in that it is always true for neutral force. This is equivalent to checking the `friend` ForceCondition.
--- 
--- @param other ForceIdentification 
--- @return boolean 
----
function LuaForce.is_friend(other) end

----
--- Is pathfinder busy? When the pathfinder is busy, it won't accept any more pathfinding requests.
--- 
--- @return boolean 
----
function LuaForce.is_pathfinder_busy() end

----
--- Kill all units and flush the pathfinder.
--- 
----
function LuaForce.kill_all_units() end

----
--- @class LuaForce.play_sound_parameters
--- Parameters class for the method `LuaForce.play_sound`
--- 
--- @field path SoundPath The sound to play.
--- @field position MapPosition? Where the sound should be played. If not given, it's played at the current position of each player.
--- @field volume_modifier double? The volume of the sound to play. Must be between 0 and 1 inclusive.
--- @field override_sound_type SoundType? The volume mixer to play the sound through. Defaults to the default mixer for the given sound type.
----

----
--- Play a sound for every player in this force.
--- 
--- @param args LuaForce.play_sound_parameters Table containing arguments for the method.
--- @see LuaForce.play_sound_parameters
----
function LuaForce.play_sound(args) end

----
--- Print text to the chat console of all players on this force.
--- 
--- @param message LocalisedString 
--- @param color Color? 
----
function LuaForce.print(message, color) end

----
--- Force a rechart of the whole chart.
--- 
----
function LuaForce.rechart() end

----
--- Research all technologies.
--- 
--- @param include_disabled_prototypes boolean? Whether technologies that are explicitly disabled in the prototype should also be researched. Defaults to `false`.
----
function LuaForce.research_all_technologies(include_disabled_prototypes) end

----
--- Reset everything. All technologies are set to not researched, all modifiers are set to default values.
--- 
----
function LuaForce.reset() end

----
--- Resets evolution for this force to zero.
--- 
----
function LuaForce.reset_evolution() end

----
--- Load the original version of all recipes from the prototypes.
--- 
----
function LuaForce.reset_recipes() end

----
--- Load the original versions of technologies from prototypes. Preserves research state of technologies.
--- 
----
function LuaForce.reset_technologies() end

----
--- Reapplies all possible research effects, including unlocked recipes. Any custom changes are lost. Preserves research state of technologies.
--- 
----
function LuaForce.reset_technology_effects() end

----
--- 
--- @param ammo string Ammo category
--- @param modifier double 
----
function LuaForce.set_ammo_damage_modifier(ammo, modifier) end

----
--- Add `other` force to this force's cease fire list. Forces on the cease fire list won't be targeted for attack.
--- 
--- @param other ForceIdentification 
--- @param cease_fire boolean 
----
function LuaForce.set_cease_fire(other, cease_fire) end

----
--- Add `other` force to this force's friends list. Friends have unrestricted access to buildings and turrets won't fire at them.
--- 
--- @param other ForceIdentification 
--- @param friend boolean 
----
function LuaForce.set_friend(other, friend) end

----
--- 
--- @param ammo string Ammo category
--- @param modifier double 
----
function LuaForce.set_gun_speed_modifier(ammo, modifier) end

----
--- Sets if the given recipe can be hand-crafted. This is used to explicitly disable hand crafting a recipe - it won't allow hand-crafting otherwise not hand-craftable recipes.
--- 
--- @param recipe string|LuaRecipe 
--- @param hand_crafting_disabled boolean 
----
function LuaForce.set_hand_crafting_disabled_for_recipe(recipe, hand_crafting_disabled) end

----
--- Sets the count of a given item launched in rockets.
--- 
--- @param item string The item to set
--- @param count uint The count to set
----
function LuaForce.set_item_launched(item, count) end

----
--- Sets the saved progress for the given technology. The technology must not be in progress, must not be completed, and the new progress must be < 100%.
--- 
--- @param technology TechnologyIdentification The technology
--- @param progress double Progress as a percent. Set to `nil` to remove the saved progress.
----
function LuaForce.set_saved_technology_progress(technology, progress) end

----
--- 
--- @param position MapPosition The new position on the given surface.
--- @param surface SurfaceIdentification Surface to set the spawn position for.
----
function LuaForce.set_spawn_position(position, surface) end

