require('.classes-1')
require('.classes-2')
require('.classes-3')
require('.classes-4')

----
--- 
--- @param turret string Turret prototype name
--- @param modifier double 
----
function LuaForce.set_turret_attack_modifier(turret, modifier) end

----
--- 
--- @param position ChunkPosition The chunk position to unchart.
--- @param surface SurfaceIdentification Surface to unchart on.
----
function LuaForce.unchart_chunk(position, surface) end


----
--- @class LuaFuelCategoryPrototype
--- Prototype of a fuel category.
--- 
--- @field name string Name of this prototype.
--- @field order string The string used to alphabetically sort these prototypes. It is a simple string that has no additional semantic meaning.
--- @field localised_name LocalisedString 
--- @field localised_description LocalisedString 
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaFuelCategoryPrototype = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaFuelCategoryPrototype.help() end


----
--- @class LuaGameScript
--- Main toplevel type, provides access to most of the API though its members. An instance of LuaGameScript is available as the global object named `game`.
--- 
--- @field object_name string This object's name.
--- @field player LuaPlayer? This property is only populated inside [custom command](LuaCommandProcessor) handlers and when writing [Lua console commands](https://wiki.factorio.com/Console#Scripting_and_cheat_commands). Returns the player that is typing the command, `nil` in all other instances.  See [LuaGameScript::players](LuaGameScript::players) for accessing all players.
--- @field players table<uint|string,LuaPlayer> Get a table of all the players that currently exist. This sparse table allows you to find players by indexing it with either their `name` or `index`. Iterating this table with `pairs()` will only iterate the array part of the table. Iterating with `ipairs()` will not work at all.  If only a single player is required, [LuaGameScript::get_player](LuaGameScript::get_player) should be used instead, as it avoids the unnecessary overhead of passing the whole table to Lua.
--- @field map_settings MapSettings The currently active set of map settings. Even though this property is marked as read-only, the members of the dictionary that is returned can be modified mid-game.
--- @field difficulty_settings DifficultySettings The currently active set of difficulty settings. Even though this property is marked as read-only, the members of the dictionary that is returned can be modified mid-game. This is however not recommended as different difficulties can have differing technology and recipe trees, which can cause problems for players.
--- @field difficulty defines.difficulty Current scenario difficulty.
--- @field forces table<uint|string,LuaForce> Get a table of all the forces that currently exist. This sparse table allows you to find forces by indexing it with either their `name` or `index`. Iterating this table with `pairs()` will only iterate the array part of the table. Iterating with `ipairs()` will not work at all.
--- @field entity_prototypes table<string,LuaEntityPrototype> A dictionary containing every LuaEntityPrototype indexed by `name`.
--- @field item_prototypes table<string,LuaItemPrototype> A dictionary containing every LuaItemPrototype indexed by `name`.
--- @field fluid_prototypes table<string,LuaFluidPrototype> A dictionary containing every LuaFluidPrototype indexed by `name`.
--- @field tile_prototypes table<string,LuaTilePrototype> A dictionary containing every LuaTilePrototype indexed by `name`.
--- @field equipment_prototypes table<string,LuaEquipmentPrototype> A dictionary containing every LuaEquipmentPrototype indexed by `name`.
--- @field damage_prototypes table<string,LuaDamagePrototype> A dictionary containing every LuaDamagePrototype indexed by `name`.
--- @field virtual_signal_prototypes table<string,LuaVirtualSignalPrototype> A dictionary containing every LuaVirtualSignalPrototype indexed by `name`.
--- @field equipment_grid_prototypes table<string,LuaEquipmentGridPrototype> A dictionary containing every LuaEquipmentGridPrototype indexed by `name`.
--- @field recipe_prototypes table<string,LuaRecipePrototype> A dictionary containing every LuaRecipePrototype indexed by `name`.
--- @field technology_prototypes table<string,LuaTechnologyPrototype> A dictionary containing every [LuaTechnologyPrototype](LuaTechnologyPrototype) indexed by `name`.
--- @field decorative_prototypes table<string,LuaDecorativePrototype> A dictionary containing every LuaDecorativePrototype indexed by `name`.
--- @field particle_prototypes table<string,LuaParticlePrototype> A dictionary containing every LuaParticlePrototype indexed by `name`.
--- @field autoplace_control_prototypes table<string,LuaAutoplaceControlPrototype> A dictionary containing every LuaAutoplaceControlPrototype indexed by `name`.
--- @field noise_layer_prototypes table<string,LuaNoiseLayerPrototype> A dictionary containing every LuaNoiseLayerPrototype indexed by `name`.
--- @field mod_setting_prototypes table<string,LuaModSettingPrototype> A dictionary containing every LuaModSettingPrototype indexed by `name`.
--- @field custom_input_prototypes table<string,LuaCustomInputPrototype> A dictionary containing every LuaCustomInputPrototype indexed by `name`.
--- @field ammo_category_prototypes table<string,LuaAmmoCategoryPrototype> A dictionary containing every LuaAmmoCategoryPrototype indexed by `name`.
--- @field named_noise_expressions table<string,LuaNamedNoiseExpression> A dictionary containing every LuaNamedNoiseExpression indexed by `name`.
--- @field item_subgroup_prototypes table<string,LuaGroup> A dictionary containing every ItemSubgroup indexed by `name`.
--- @field item_group_prototypes table<string,LuaGroup> A dictionary containing every ItemGroup indexed by `name`.
--- @field fuel_category_prototypes table<string,LuaFuelCategoryPrototype> A dictionary containing every LuaFuelCategoryPrototype indexed by `name`.
--- @field resource_category_prototypes table<string,LuaResourceCategoryPrototype> A dictionary containing every LuaResourceCategoryPrototype indexed by `name`.
--- @field achievement_prototypes table<string,LuaAchievementPrototype> A dictionary containing every LuaAchievementPrototype indexed by `name`.
--- @field module_category_prototypes table<string,LuaModuleCategoryPrototype> A dictionary containing every LuaModuleCategoryPrototype indexed by `name`.
--- @field equipment_category_prototypes table<string,LuaEquipmentCategoryPrototype> A dictionary containing every LuaEquipmentCategoryPrototype indexed by `name`.
--- @field trivial_smoke_prototypes table<string,LuaTrivialSmokePrototype> A dictionary containing every LuaTrivialSmokePrototype indexed by `name`.
--- @field shortcut_prototypes table<string,LuaShortcutPrototype> A dictionary containing every LuaShortcutPrototype indexed by `name`.
--- @field recipe_category_prototypes table<string,LuaRecipeCategoryPrototype> A dictionary containing every LuaRecipeCategoryPrototype indexed by `name`.
--- @field font_prototypes table<string,LuaFontPrototype> A dictionary containing every LuaFontPrototype indexed by `name`.
--- @field map_gen_presets table<string,MapGenPreset> A dictionary containing every MapGenPreset indexed by `name`.
--- @field console_command_used boolean Whether a console command has been used.
--- @field styles table<string,string> The styles that [LuaGuiElement](LuaGuiElement) can use, indexed by `name`.
--- @field tick uint Current map tick.
--- @field ticks_played uint The number of ticks since this game was 'created'. A game is 'created' either by using "new game" or "new game from scenario".
--- @field tick_paused boolean If the tick has been paused. This means that entity update has been paused.
--- @field ticks_to_run uint The number of ticks to be run while the tick is paused. When [LuaGameScript::tick_paused](LuaGameScript::tick_paused) is true, ticks_to_run behaves the following way: While this is > 0, the entity update is running normally and this value is decremented every tick. When this reaches 0, the game will pause again.
--- @field finished boolean True while the victory screen is shown.
--- @field finished_but_continuing boolean True after players finished the game and clicked "continue".
--- @field speed float Speed to update the map at. 1.0 is normal speed -- 60 UPS.
--- @field surfaces table<uint|string,LuaSurface> Get a table of all the surfaces that currently exist. This sparse table allows you to find surfaces by indexing it with either their `name` or `index`. Iterating this table with `pairs()` will only iterate the array part of the table. Iterating with `ipairs()` will not work at all.
--- @field active_mods table<string,string> The active mods versions. The keys are mod names, the values are the versions.
--- @field connected_players LuaPlayer[] The players that are currently online.  This is primarily useful when you want to do some action against all online players.
--- @field permissions LuaPermissionGroups 
--- @field backer_names table<uint,string> Array of the names of all the backers that supported the game development early on. These are used as names for labs, locomotives, radars, roboports, and train stops.
--- @field default_map_gen_settings MapGenSettings The default map gen settings for this save.
--- @field enemy_has_vision_on_land_mines boolean Determines if enemy land mines are completely invisible or not.
--- @field autosave_enabled boolean True by default. Can be used to disable autosaving. Make sure to turn it back on soon after.
--- @field draw_resource_selection boolean True by default. Can be used to disable the highlighting of resource patches when they are hovered on the map.
--- @field pollution_statistics LuaFlowStatistics The pollution statistics for this map.
--- @field max_force_distraction_distance double 
--- @field max_force_distraction_chunk_distance uint 
--- @field max_electric_pole_supply_area_distance float 
--- @field max_electric_pole_connection_distance double 
--- @field max_beacon_supply_area_distance double 
--- @field max_gate_activation_distance double 
--- @field max_inserter_reach_distance double 
--- @field max_pipe_to_ground_distance uint8 
--- @field max_underground_belt_distance uint8 
----
LuaGameScript = { }

----
--- Instruct the game to perform an auto-save.
--- 
--- @param name string? The autosave name if any. Saves will be named _autosave-*name* when provided.
----
function LuaGameScript.auto_save(name) end

----
--- Bans the given player from this multiplayer game. Does nothing if this is a single player game of if the player running this isn't an admin.
--- 
--- @param player PlayerIdentification The player to ban.
--- @param reason LocalisedString? The reason given if any.
----
function LuaGameScript.ban_player(player, reason) end

----
--- Run internal consistency checks. Allegedly prints any errors it finds.
--- 
----
function LuaGameScript.check_consistency() end

----
--- Goes over all items, entities, tiles, recipes, technologies among other things and logs if the locale is incorrect.
--- 
----
function LuaGameScript.check_prototype_translations() end

----
--- Counts how many distinct groups of pipes exist in the world.
--- 
----
function LuaGameScript.count_pipe_groups() end

----
--- Create a new force.
--- 
--- @param force string Name of the new force
--- @return LuaForce The force that was just created
----
function LuaGameScript.create_force(force) end

----
--- Creates an inventory that is not owned by any game object. It can be resized later with [LuaInventory::resize](LuaInventory::resize).
--- 
--- @param size uint16 The number of slots the inventory initially has.
--- @return LuaInventory 
----
function LuaGameScript.create_inventory(size) end

----
--- Creates a [LuaProfiler](LuaProfiler), which is used for measuring script performance.
--- 
--- @param stopped boolean? Create the timer stopped
--- @return LuaProfiler 
----
function LuaGameScript.create_profiler(stopped) end

----
--- Creates a deterministic standalone random generator with the given seed or if a seed is not provided the initial map seed is used.
--- 
--- @param seed uint? 
--- @return LuaRandomGenerator 
----
function LuaGameScript.create_random_generator(seed) end

----
--- Create a new surface.
--- 
--- @param name string Name of the new surface.
--- @param settings MapGenSettings? Map generation settings.
--- @return LuaSurface The surface that was just created.
----
function LuaGameScript.create_surface(name, settings) end

----
--- Base64 decodes and inflates the given string.
--- 
--- @param string string The string to decode.
--- @return string? The decoded string or `nil` if the decode failed.
----
function LuaGameScript.decode_string(string) end

----
--- Deletes the given surface and all entities on it.
--- 
--- @param surface LuaSurface|string The surface to be deleted. Currently the primary surface (1, 'nauvis') cannot be deleted.
----
function LuaGameScript.delete_surface(surface) end

----
--- Converts the given direction into the string version of the direction.
--- 
--- @param direction defines.direction 
----
function LuaGameScript.direction_to_string(direction) end

----
--- Disables replay saving for the current save file. Once done there's no way to re-enable replay saving for the save file without loading an old save.
--- 
----
function LuaGameScript.disable_replay() end

----
--- Disables tutorial triggers, that unlock new tutorials and show notices about unlocked tutorials.
--- 
----
function LuaGameScript.disable_tutorial_triggers() end

----
--- Deflates and base64 encodes the given string.
--- 
--- @param string string The string to encode.
--- @return string? The encoded string or `nil` if the encode failed.
----
function LuaGameScript.encode_string(string) end

----
--- Evaluate an expression, substituting variables as provided. For details on the formula, see the relevant page on the [Factorio wiki](https://wiki.factorio.com/Prototype/Technology#unit).
--- 
--- @param expression string The expression to evaluate.
--- @param variables table<string,double>? Variables to be substituted.
--- @return double 
----
function LuaGameScript.evaluate_expression(expression, variables) end

----
--- Force a CRC check. Tells all peers to calculate their current CRC, which are then compared to each other. If a mismatch is detected, the game desyncs and some peers are forced to reconnect.
--- 
----
function LuaGameScript.force_crc() end

----
--- Gets the number of entities that are active (updated each tick).
--- 
--- @param surface SurfaceIdentification? If given, only the entities active on this surface are counted.
--- @return uint 
----
function LuaGameScript.get_active_entities_count(surface) end

----
--- 
--- @param tag string 
--- @return LuaEntity? 
----
function LuaGameScript.get_entity_by_tag(tag) end

----
--- Returns a dictionary of all LuaAchievementPrototypes that fit the given filters. The prototypes are indexed by `name`.
--- 
--- @param filters AchievementPrototypeFilter[] 
--- @return table<string,LuaAchievementPrototype> 
----
function LuaGameScript.get_filtered_achievement_prototypes(filters) end

----
--- Returns a dictionary of all LuaDecorativePrototypes that fit the given filters. The prototypes are indexed by `name`.
--- 
--- @param filters DecorativePrototypeFilter[] 
--- @return table<string,LuaDecorativePrototype> 
----
function LuaGameScript.get_filtered_decorative_prototypes(filters) end

----
--- Returns a dictionary of all LuaEntityPrototypes that fit the given filters. The prototypes are indexed by `name`.
--- 
--- @param filters EntityPrototypeFilter[] 
--- @return table<string,LuaEntityPrototype> 
----
function LuaGameScript.get_filtered_entity_prototypes(filters) end

----
--- Returns a dictionary of all LuaEquipmentPrototypes that fit the given filters. The prototypes are indexed by `name`.
--- 
--- @param filters EquipmentPrototypeFilter[] 
--- @return table<string,LuaEquipmentPrototype> 
----
function LuaGameScript.get_filtered_equipment_prototypes(filters) end

----
--- Returns a dictionary of all LuaFluidPrototypes that fit the given filters. The prototypes are indexed by `name`.
--- 
--- @param filters FluidPrototypeFilter[] 
--- @return table<string,LuaFluidPrototype> 
----
function LuaGameScript.get_filtered_fluid_prototypes(filters) end

----
--- Returns a dictionary of all LuaItemPrototypes that fit the given filters. The prototypes are indexed by `name`.
--- 
--- @param filters ItemPrototypeFilter[] 
--- @return table<string,LuaItemPrototype> 
----
function LuaGameScript.get_filtered_item_prototypes(filters) end

----
--- Returns a dictionary of all LuaModSettingPrototypes that fit the given filters. The prototypes are indexed by `name`.
--- 
--- @param filters ModSettingPrototypeFilter[] 
--- @return table<string,LuaModSettingPrototype> 
----
function LuaGameScript.get_filtered_mod_setting_prototypes(filters) end

----
--- Returns a dictionary of all LuaRecipePrototypes that fit the given filters. The prototypes are indexed by `name`.
--- 
--- @param filters RecipePrototypeFilter[] 
--- @return table<string,LuaRecipePrototype> 
----
function LuaGameScript.get_filtered_recipe_prototypes(filters) end

----
--- Returns a dictionary of all LuaTechnologyPrototypes that fit the given filters. The prototypes are indexed by `name`.
--- 
--- @param filters TechnologyPrototypeFilter[] 
--- @return table<string,LuaTechnologyPrototype> 
----
function LuaGameScript.get_filtered_technology_prototypes(filters) end

----
--- Returns a dictionary of all LuaTilePrototypes that fit the given filters. The prototypes are indexed by `name`.
--- 
--- @param filters TilePrototypeFilter[] 
--- @return table<string,LuaTilePrototype> 
----
function LuaGameScript.get_filtered_tile_prototypes(filters) end

----
--- Gets the map exchange string for the map generation settings that were used to create this map.
--- 
--- @return string 
----
function LuaGameScript.get_map_exchange_string() end

----
--- Gets the given player or returns `nil` if no player is found.
--- 
--- @param player uint|string The player index or name.
--- @return LuaPlayer? 
----
function LuaGameScript.get_player(player) end

----
--- Gets the inventories created through [LuaGameScript::create_inventory](LuaGameScript::create_inventory)
--- 
--- @param mod string? The mod who's inventories to get. If not provided all inventories are returned.
--- @return table<string,LuaInventory[]> A mapping of mod name to array of inventories owned by that mod.
----
function LuaGameScript.get_script_inventories(mod) end

----
--- Gets the given surface or returns `nil` if no surface is found.
--- 
--- @param surface uint|string The surface index or name.
--- @return LuaSurface? 
----
function LuaGameScript.get_surface(surface) end

----
--- @class LuaGameScript.get_train_stops_parameters
--- Parameters class for the method `LuaGameScript.get_train_stops`
--- 
--- @field name string|string[]? The name(s) of the train stops. Not providing names will match any stop.
--- @field surface SurfaceIdentification? The surface to search. Not providing a surface will match stops on any surface.
--- @field force ForceIdentification? The force to search. Not providing a force will match stops in any force.
----

----
--- Gets train stops matching the given filters.
--- 
--- @param args LuaGameScript.get_train_stops_parameters? Table containing arguments for the method.
--- @see LuaGameScript.get_train_stops_parameters
--- @return LuaEntity[] 
----
function LuaGameScript.get_train_stops(args) end

----
--- Is this the demo version of Factorio?
--- 
--- @return boolean 
----
function LuaGameScript.is_demo() end

----
--- Is the map loaded is multiplayer?
--- 
--- @return boolean 
----
function LuaGameScript.is_multiplayer() end

----
--- Checks if the given SoundPath is valid.
--- 
--- @param sound_path SoundPath Path to the sound.
--- @return boolean 
----
function LuaGameScript.is_valid_sound_path(sound_path) end

----
--- Checks if the given SpritePath is valid and contains a loaded sprite. The existence of the image is not checked for paths of type `file`.
--- 
--- @param sprite_path SpritePath Path to the image.
--- @return boolean 
----
function LuaGameScript.is_valid_sprite_path(sprite_path) end

----
--- Convert a JSON string to a table.
--- 
--- @param json string The string to convert.
--- @return AnyBasic? The returned object, or `nil` if the JSON string was invalid.
----
function LuaGameScript.json_to_table(json) end

----
--- Kicks the given player from this multiplayer game. Does nothing if this is a single player game or if the player running this isn't an admin.
--- 
--- @param player PlayerIdentification The player to kick.
--- @param reason LocalisedString? The reason given if any.
----
function LuaGameScript.kick_player(player, reason) end

----
--- Marks two forces to be merged together. All players and entities in the source force will be reassigned to the target force. The source force will then be destroyed. Importantly, this does not merge technologies or bonuses, which are instead retained from the target force.
--- 
--- @param source ForceIdentification The force to remove.
--- @param destination ForceIdentification The force to reassign all entities to.
----
function LuaGameScript.merge_forces(source, destination) end

----
--- Mutes the given player. Does nothing if the player running this isn't an admin.
--- 
--- @param player PlayerIdentification The player to mute.
----
function LuaGameScript.mute_player(player) end

----
--- Convert a map exchange string to map gen settings and map settings.
--- 
--- @param map_exchange_string string 
--- @return MapExchangeStringData 
----
function LuaGameScript.parse_map_exchange_string(map_exchange_string) end

----
--- @class LuaGameScript.play_sound_parameters
--- Parameters class for the method `LuaGameScript.play_sound`
--- 
--- @field path SoundPath The sound to play.
--- @field position MapPosition? Where the sound should be played. If not given, it's played at the current position of each player.
--- @field volume_modifier double? The volume of the sound to play. Must be between 0 and 1 inclusive.
--- @field override_sound_type SoundType? The volume mixer to play the sound through. Defaults to the default mixer for the given sound type.
----

----
--- Play a sound for every player in the game.
--- 
--- @param args LuaGameScript.play_sound_parameters Table containing arguments for the method.
--- @see LuaGameScript.play_sound_parameters
----
function LuaGameScript.play_sound(args) end

----
--- Print text to the chat console all players.
--- 
--- @param message LocalisedString 
--- @param color Color? 
----
function LuaGameScript.print(message, color) end

----
--- Purges the given players messages from the game. Does nothing if the player running this isn't an admin.
--- 
--- @param player PlayerIdentification The player to purge.
----
function LuaGameScript.purge_player(player) end

----
--- Regenerate autoplacement of some entities on all surfaces. This can be used to autoplace newly-added entities.
--- 
--- @param entities string|string[] Prototype names of entity or entities to autoplace.
----
function LuaGameScript.regenerate_entity(entities) end

----
--- Forces a reload of all mods.
--- 
----
function LuaGameScript.reload_mods() end

----
--- Forces a reload of the scenario script from the original scenario location.
--- 
----
function LuaGameScript.reload_script() end

----
--- Remove players who are currently not connected from the map.
--- 
--- @param players LuaPlayer|string[]? List of players to remove. If not specified, remove all offline players.
----
function LuaGameScript.remove_offline_players(players) end

----
--- Remove a file or directory in the `script-output` folder, located in the game's [user data directory](https://wiki.factorio.com/User_data_directory). Can be used to remove files created by [LuaGameScript::write_file](LuaGameScript::write_file).
--- 
--- @param path string The path to the file or directory to remove, relative to `script-output`.
----
function LuaGameScript.remove_path(path) end

----
--- Reset scenario state (game_finished, player_won, etc.).
--- 
----
function LuaGameScript.reset_game_state() end

----
--- Resets the amount of time played for this map.
--- 
----
function LuaGameScript.reset_time_played() end

----
--- Saves the current configuration of Atlas to a file. This will result in huge file containing all of the game graphics moved to as small space as possible.
--- 
----
function LuaGameScript.save_atlas() end

----
--- Instruct the server to save the map.
--- 
--- @param name string? Save name. If not specified, writes into the currently-running save.
----
function LuaGameScript.server_save(name) end

----
--- @class LuaGameScript.set_game_state_parameters
--- Parameters class for the method `LuaGameScript.set_game_state`
--- 
--- @field game_finished boolean? 
--- @field player_won boolean? 
--- @field next_level string? 
--- @field can_continue boolean? 
--- @field victorious_force ForceIdentification? 
----

----
--- Set scenario state. Any parameters not provided do not change the current state.
--- 
--- @param args LuaGameScript.set_game_state_parameters Table containing arguments for the method.
--- @see LuaGameScript.set_game_state_parameters
----
function LuaGameScript.set_game_state(args) end

----
--- Forces the screenshot saving system to wait until all queued screenshots have been written to disk.
--- 
----
function LuaGameScript.set_wait_for_screenshots_to_finish() end

----
--- @class LuaGameScript.show_message_dialog_parameters
--- Parameters class for the method `LuaGameScript.show_message_dialog`
--- 
--- @field text LocalisedString What the dialog should say
--- @field image string? Path to an image to show on the dialog
--- @field point_to GuiArrowSpecification? If specified, dialog will show an arrow pointing to this place. When not specified, the arrow will point to the player's position. (Use `point_to={type="nowhere"}` to remove the arrow entirely.) The dialog itself will be placed near the arrow's target.
--- @field style string? The gui style to use for this speech bubble. Must be of type speech_bubble.
--- @field wrapper_frame_style string? Must be of type flow_style.
----

----
--- Show an in-game message dialog.
--- 
--- @param args LuaGameScript.show_message_dialog_parameters Table containing arguments for the method.
--- @see LuaGameScript.show_message_dialog_parameters
----
function LuaGameScript.show_message_dialog(args) end

----
--- Convert a table to a JSON string
--- 
--- @param data table 
--- @return string 
----
function LuaGameScript.table_to_json(data) end

----
--- @class LuaGameScript.take_screenshot_parameters
--- Parameters class for the method `LuaGameScript.take_screenshot`
--- 
--- @field player PlayerIdentification? The player to focus on. Defaults to the local player.
--- @field by_player PlayerIdentification? If defined, the screenshot will only be taken for this player.
--- @field surface SurfaceIdentification? If defined, the screenshot will be taken on this surface.
--- @field position MapPosition? If defined, the screenshot will be centered on this position. Otherwise, the screenshot will center on `player`.
--- @field resolution TilePosition? The maximum allowed resolution is 16384x16384 (8192x8192 when `anti_alias` is `true`), but the maximum recommended resolution is 4096x4096 (resp. 2048x2048).
--- @field zoom double? The map zoom to take the screenshot at. Defaults to `1`.
--- @field path string? The name of the image file. It should include a file extension indicating the desired format. Supports `.png`, `.jpg` /`.jpeg`, `.tga` and `.bmp`. Providing a directory path (ex. `"save/here/screenshot.png"`) will create the necessary folder structure in `script-output`. Defaults to `"screenshot.png"`.
--- @field show_gui boolean? Whether to include GUIs in the screenshot or not. Defaults to `false`.
--- @field show_entity_info boolean? Whether to include entity info ("Alt mode") or not. Defaults to `false`.
--- @field show_cursor_building_preview boolean? When `true` and when `player` is specified, the building preview for the item in the player's cursor will also be rendered. Defaults to `false`.
--- @field anti_alias boolean? Whether to render in double resolution and downscale the result (including GUI). Defaults to `false`.
--- @field quality int? The `.jpg` render quality as a percentage (from 0% to 100% inclusive), if used. A lower value means a more compressed image. Defaults to `80`.
--- @field allow_in_replay boolean? Whether to save the screenshot even during replay playback. Defaults to `false`.
--- @field daytime double? Overrides the current surface daytime for the duration of screenshot rendering.
--- @field water_tick uint? Overrides the tick of water animation, if animated water is enabled.
--- @field force_render boolean? Screenshot requests are processed in between game update and render. The game may skip rendering (ie. drop frames) if the previous frame has not finished rendering or the game simulation starts to fall below 60 updates per second. If `force_render` is set to `true`, the game won't drop frames and process the screenshot request at the end of the update in which the request was created. This is not honored on multiplayer clients that are catching up to server. Defaults to `false`.
----

----
--- Take a screenshot of the game and save it to the `script-output` folder, located in the game's [user data directory](https://wiki.factorio.com/User_data_directory). The name of the image file can be specified via the `path` parameter.
--- 
--- @param args LuaGameScript.take_screenshot_parameters Table containing arguments for the method.
--- @see LuaGameScript.take_screenshot_parameters
----
function LuaGameScript.take_screenshot(args) end

----
--- @class LuaGameScript.take_technology_screenshot_parameters
--- Parameters class for the method `LuaGameScript.take_technology_screenshot`
--- 
--- @field force ForceIdentification? The force whose technology to screenshot. If not given, the `"player`" force is used.
--- @field path string? The name of the image file. It should include a file extension indicating the desired format. Supports `.png`, `.jpg` /`.jpeg`, `.tga` and `.bmp`. Providing a directory path (ex. `"save/here/screenshot.png"`) will create the necessary folder structure in `script-output`. Defaults to `"technology-screenshot.png"`.
--- @field by_player PlayerIdentification? If given, the screenshot will only be taken for this player.
--- @field selected_technology TechnologyIdentification? The technology to highlight.
--- @field skip_disabled boolean? If `true`, disabled technologies will be skipped. Their successors will be attached to the disabled technology's parents. Defaults to `false`.
--- @field quality int? The `.jpg` render quality as a percentage (from 0% to 100% inclusive), if used. A lower value means a more compressed image. Defaults to `80`.
----

----
--- Take a screenshot of the technology screen and save it to the `script-output` folder, located in the game's [user data directory](https://wiki.factorio.com/User_data_directory). The name of the image file can be specified via the `path` parameter.
--- 
--- @param args LuaGameScript.take_technology_screenshot_parameters Table containing arguments for the method.
--- @see LuaGameScript.take_technology_screenshot_parameters
----
function LuaGameScript.take_technology_screenshot(args) end

----
--- Unbans the given player from this multiplayer game. Does nothing if this is a single player game of if the player running this isn't an admin.
--- 
--- @param player PlayerIdentification The player to unban.
----
function LuaGameScript.unban_player(player) end

----
--- Unmutes the given player. Does nothing if the player running this isn't an admin.
--- 
--- @param player PlayerIdentification The player to unmute.
----
function LuaGameScript.unmute_player(player) end

----
--- Write a file to the `script-output` folder, located in the game's [user data directory](https://wiki.factorio.com/User_data_directory). The name and file extension of the file can be specified via the `filename` parameter.
--- 
--- @param filename string The name of the file. Providing a directory path (ex. `"save/here/example.txt"`) will create the necessary folder structure in `script-output`.
--- @param data LocalisedString The content to write to the file.
--- @param append boolean? If `true`, `data` will be appended to the end of the file. Defaults to `false`, which will overwrite any pre-existing file with the new `data`.
--- @param for_player uint? If given, the file will only be written for this `player_index`. Providing `0` will only write to the server's output if present.
----
function LuaGameScript.write_file(filename, data, append, for_player) end


----
--- @class LuaGenericOnOffControlBehavior: LuaControlBehavior
--- An abstract base class for behaviors that support switching the entity on or off based on some condition.
--- 
--- @field disabled boolean If the entity is currently disabled because of the control behavior.
--- @field circuit_condition CircuitConditionDefinition The circuit condition. Writing `nil` clears the circuit condition.
--- @field logistic_condition CircuitConditionDefinition The logistic condition. Writing `nil` clears the logistic condition.
--- @field connect_to_logistic_network boolean `true` if this should connect to the logistic network.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaGenericOnOffControlBehavior = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaGenericOnOffControlBehavior.help() end


----
--- @class LuaGroup
--- Item group or subgroup.
--- 
--- @field name string? 
--- @field localised_name LocalisedString? Localised name of the group.
--- @field type string? 
--- @field group LuaGroup? The parent group, if any.
--- @field subgroups LuaGroup[]? Subgroups of this group. Can only be used on groups, not on subgroups.
--- @field order_in_recipe string? The additional order value used in recipe ordering. Can only be used on groups, not on subgroups.
--- @field order string The string used to alphabetically sort these prototypes. It is a simple string that has no additional semantic meaning.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaGroup = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaGroup.help() end


----
--- @class LuaGui
--- The root of the GUI. This type houses the root elements, `top`, `left`, `center`, `goal`, and `screen`, to which other elements can be added to be displayed on screen.
--- 
--- @field player LuaPlayer The player who owns this gui.
--- @field children table<string,LuaGuiElement> The children GUI elements mapped by name <> element.
--- @field top LuaGuiElement The top part of the GUI. It is a flow element inside a scroll pane element.
--- @field left LuaGuiElement The left part of the GUI. It is a flow element inside a scroll pane element.
--- @field center LuaGuiElement The center part of the GUI. It is a flow element.
--- @field goal LuaGuiElement The flow used in the objectives window. It is a flow element. The objectives window is only visible when the flow is not empty or the objective text is set.
--- @field screen LuaGuiElement For showing a GUI somewhere on the entire screen. It is an empty-widget element.
--- @field relative LuaGuiElement For showing a GUI somewhere relative to one of the game GUIs. It is an empty-widget element.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaGui = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaGui.help() end

----
--- Returns `true` if sprite_path is valid and contains loaded sprite, otherwise `false`. Sprite path of type `file` doesn't validate if file exists.
--- 
--- If you want to avoid needing a LuaGui object, [LuaGameScript::is_valid_sprite_path](LuaGameScript::is_valid_sprite_path) can be used instead.
--- 
--- @param sprite_path SpritePath Path to a image.
--- @return boolean 
----
function LuaGui.is_valid_sprite_path(sprite_path) end


----
--- @class LuaGuiElement
--- An element of a custom GUI. This type is used to represent any kind of a GUI element - labels, buttons and frames are all instances of this type. Just like [LuaEntity](LuaEntity), different kinds of elements support different attributes; attempting to access an attribute on an element that doesn't support it (for instance, trying to access the `column_count` of a `textfield`) will result in a runtime error.
--- 
--- The following types of GUI element are supported:
--- 
--- - `"button"`: A clickable element. Relevant event: [on_gui_click](on_gui_click)
--- - `"sprite-button"`: A `button` that displays a sprite rather than text. Relevant event: [on_gui_click](on_gui_click)
--- - `"checkbox"`: A clickable element with a check mark that can be turned off or on. Relevant event: [on_gui_checked_state_changed](on_gui_checked_state_changed)
--- - `"flow"`: An invisible container that lays out its children either horizontally or vertically.
--- - `"frame"`: A non-transparent box that contains other elements. It can have a title (set via the `caption` attribute). Just like a `flow`, it lays out its children either horizontally or vertically. Relevant event: [on_gui_location_changed](on_gui_location_changed)
--- - `"label"`: A piece of text.
--- - `"line"`: A horizontal or vertical separation line.
--- - `"progressbar"`: A partially filled bar that can be used to indicate progress.
--- - `"table"`: An invisible container that lays out its children in a specific number of columns. The width of each column is determined by the widest element it contains.
--- - `"textfield"`: A single-line box the user can type into. Relevant events: [on_gui_text_changed](on_gui_text_changed), [on_gui_confirmed](on_gui_confirmed)
--- - `"radiobutton"`: An element that is similar to a `checkbox`, but with a circular appearance. Clicking a selected radio button will not deselect it. Radio buttons are not linked to each other in any way. Relevant event: [on_gui_checked_state_changed](on_gui_checked_state_changed)
--- - `"sprite"`: An element that shows an image.
--- - `"scroll-pane"`: An invisible element that is similar to a `flow`, but has the ability to show and use scroll bars.
--- - `"drop-down"`: A drop-down containing strings of text. Relevant event: [on_gui_selection_state_changed](on_gui_selection_state_changed)
--- - `"list-box"`: A list of strings, only one of which can be selected at a time. Shows a scroll bar if necessary. Relevant event: [on_gui_selection_state_changed](on_gui_selection_state_changed)
--- - `"camera"`: A camera that shows the game at the given position on the given surface. It can visually track an [entity](LuaGuiElement::entity) that is set after the element has been created.
--- - `"choose-elem-button"`: A button that lets the player pick from a certain kind of prototype, with optional filtering. Relevant event: [on_gui_elem_changed](on_gui_elem_changed)
--- - `"text-box"`: A multi-line `textfield`. Relevant event: [on_gui_text_changed](on_gui_text_changed)
--- - `"slider"`: A horizontal number line which can be used to choose a number. Relevant event: [on_gui_value_changed](on_gui_value_changed)
--- - `"minimap"`: A minimap preview, similar to the normal player minimap. It can visually track an [entity](LuaGuiElement::entity) that is set after the element has been created.
--- - `"entity-preview"`: A preview of an entity. The [entity](LuaGuiElement::entity) has to be set after the element has been created.
--- - `"empty-widget"`: An empty element that just exists. The root GUI elements `screen` and `relative` are `empty-widget`s.
--- - `"tabbed-pane"`: A collection of `tab`s and their contents. Relevant event: [on_gui_selected_tab_changed](on_gui_selected_tab_changed)
--- - `"tab"`: A tab for use in a `tabbed-pane`.
--- - `"switch"`: A switch with three possible states. Can have labels attached to either side. Relevant event: [on_gui_switch_state_changed](on_gui_switch_state_changed)
--- 
--- Each GUI element allows access to its children by having them as attributes. Thus, one can use the `parent.child` syntax to refer to children. Lua also supports the `parent["child"]` syntax to refer to the same element. This can be used in cases where the child has a name that isn't a valid Lua identifier.
--- 
--- @field index uint The index of this GUI element (unique amongst the GUI elements of a LuaPlayer).
--- @field gui LuaGui The GUI this element is a child of.
--- @field parent LuaGuiElement? The direct parent of this element. `nil` if this is a top-level element.
--- @field name string The name of this element. `""` if no name was set.
--- @field caption LocalisedString The text displayed on this element. For frames, this is the "heading". For other elements, like buttons or labels, this is the content.
--- @field value double How much this progress bar is filled. It is a value in the range [0, 1].
--- @field direction string Direction of this element's layout. May be either `"horizontal"` or `"vertical"`.
--- @field style LuaStyle|string The style of this element. When read, this evaluates to a [LuaStyle](LuaStyle). For writing, it only accepts a string that specifies the textual identifier (prototype name) of the desired style.
--- @field visible boolean Sets whether this GUI element is visible or completely hidden, taking no space in the layout.
--- @field text string The text contained in this textfield or text-box.
--- @field children_names string[] Names of all the children of this element. These are the identifiers that can be used to access the child as an attribute of this element.
--- @field state boolean Is this checkbox or radiobutton checked?
--- @field player_index uint Index into [LuaGameScript::players](LuaGameScript::players) specifying the player who owns this element.
--- @field sprite SpritePath The sprite to display on this sprite-button or sprite in the default state.
--- @field resize_to_sprite boolean Whether the sprite widget should resize according to the sprite in it. Defaults to `true`.
--- @field hovered_sprite SpritePath The sprite to display on this sprite-button when it is hovered.
--- @field clicked_sprite SpritePath The sprite to display on this sprite-button when it is clicked.
--- @field tooltip LocalisedString 
--- @field horizontal_scroll_policy string Policy of the horizontal scroll bar. Possible values are `"auto"`, `"never"`, `"always"`, `"auto-and-reserve-space"`, `"dont-show-but-allow-scrolling"`.
--- @field vertical_scroll_policy string Policy of the vertical scroll bar. Possible values are `"auto"`, `"never"`, `"always"`, `"auto-and-reserve-space"`, `"dont-show-but-allow-scrolling"`.
--- @field type string The type of this GUI element.
--- @field children LuaGuiElement[] The child-elements of this GUI element.
--- @field items LocalisedString[] The items in this dropdown or listbox.
--- @field selected_index uint The selected index for this dropdown or listbox. Returns `0` if none is selected.
--- @field number double The number to be shown in the bottom right corner of this sprite-button. Set this to `nil` to show nothing.
--- @field show_percent_for_small_numbers boolean Related to the number to be shown in the bottom right corner of this sprite-button. When set to `true`, numbers that are non-zero and smaller than one are shown as a percentage rather than the value. For example, `0.5` will be shown as `50%` instead.
--- @field location GuiLocation? The location of this widget when stored in [LuaGui::screen](LuaGui::screen). `nil` if not set or not in [LuaGui::screen](LuaGui::screen).
--- @field auto_center boolean Whether this frame auto-centers on window resize when stored in [LuaGui::screen](LuaGui::screen).
--- @field badge_text LocalisedString The text to display after the normal tab text (designed to work with numbers)
--- @field position MapPosition The position this camera or minimap is focused on, if any.
--- @field surface_index uint The surface index this camera or minimap is using.
--- @field zoom double The zoom this camera or minimap is using. This value must be positive.
--- @field minimap_player_index uint The player index this minimap is using.
--- @field force string? The force this minimap is using, if any.
--- @field elem_type string The elem type of this choose-elem-button.
--- @field elem_value string|SignalID? The elem value of this choose-elem-button, if any.
--- @field elem_filters PrototypeFilter? The elem filters of this choose-elem-button, if any. The compatible type of filter is determined by `elem_type`.
--- @field selectable boolean Whether the contents of this text-box are selectable. Defaults to `true`.
--- @field word_wrap boolean Whether this text-box will word-wrap automatically. Defaults to `false`.
--- @field read_only boolean Whether this text-box is read-only. Defaults to `false`.
--- @field enabled boolean Whether this GUI element is enabled. Disabled GUI elements don't trigger events when clicked.
--- @field ignored_by_interaction boolean Whether this GUI element is ignored by interaction. This makes clicks on this element 'go through' to the GUI element or even the game surface below it.
--- @field locked boolean Whether this choose-elem-button can be changed by the player.
--- @field draw_vertical_lines boolean Whether this table should draw vertical grid lines.
--- @field draw_horizontal_lines boolean Whether this table should draw horizontal grid lines.
--- @field draw_horizontal_line_after_headers boolean Whether this table should draw a horizontal grid line below the first table row.
--- @field column_count uint The number of columns in this table.
--- @field vertical_centering boolean Whether the content of this table should be vertically centered. Overrides [LuaStyle::column_alignments](LuaStyle::column_alignments). Defaults to `true`.
--- @field slider_value double The value of this slider element.
--- @field mouse_button_filter MouseButtonFlags The mouse button filters for this button or sprite-button.
--- @field numeric boolean Whether this textfield is limited to only numberic characters.
--- @field allow_decimal boolean Whether this textfield (when in numeric mode) allows decimal numbers.
--- @field allow_negative boolean Whether this textfield (when in numeric mode) allows negative numbers.
--- @field is_password boolean Whether this textfield displays as a password field, which renders all characters as `*`.
--- @field lose_focus_on_confirm boolean Whether this textfield loses focus after [defines.events.on_gui_confirmed](defines.events.on_gui_confirmed) is fired.
--- @field clear_and_focus_on_right_click boolean Makes it so right-clicking on this textfield clears and focuses it.
--- @field drag_target LuaGuiElement? The `frame` that is being moved when dragging this GUI element, if any. This element needs to be a child of the `drag_target` at some level.
--- @field selected_tab_index uint? The selected tab index for this tabbed pane, if any.
--- @field tabs TabAndContent[] The tabs and contents being shown in this tabbed-pane.
--- @field entity LuaEntity? The entity associated with this entity-preview, camera, minimap, if any.
--- @field anchor GuiAnchor? The anchor for this relative widget, if any. Setting `nil` clears the anchor.
--- @field tags Tags The tags associated with this LuaGuiElement.
--- @field raise_hover_events boolean Whether this element will raise [on_gui_hover](on_gui_hover) and [on_gui_leave](on_gui_leave).
--- @field switch_state string The switch state (left, none, right) for this switch.
--- @field allow_none_state boolean Whether the `"none"` state is allowed for this switch.
--- @field left_label_caption LocalisedString The text shown for the left switch label.
--- @field left_label_tooltip LocalisedString The tooltip shown on the left switch label.
--- @field right_label_caption LocalisedString The text shown for the right switch label.
--- @field right_label_tooltip LocalisedString The tooltip shown on the right switch label.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaGuiElement = { }

----
--- @class LuaGuiElement.add_parameters
--- Parameters class for the method `LuaGuiElement.add`
--- 
--- @field type string The kind of element to add. Has to be one of the GUI element types listed at the top of this page.
--- @field name string? Name of the child element. It must be unique within the parent element.
--- @field caption LocalisedString? Text displayed on the child element. For frames, this is their title. For other elements, like buttons or labels, this is the content. Whilst this attribute may be used on all elements, it doesn't make sense for tables and flows as they won't display it.
--- @field tooltip LocalisedString? Tooltip of the child element.
--- @field enabled boolean? Whether the child element is enabled. Defaults to `true`.
--- @field visible boolean? Whether the child element is visible. Defaults to `true`.
--- @field ignored_by_interaction boolean? Whether the child element is ignored by interaction. Defaults to `false`.
--- @field style string? The name of the style prototype to apply to the new element.
--- @field tags Tags? [Tags](Tags) associated with the child element.
--- @field index uint? Location in its parent that the child element should slot into. By default, the child will be appended onto the end.
--- @field anchor GuiAnchor? Where to position the child element when in the `relative` element.
----

----
--- Add a new child element to this GuiElement.
--- 
--- @param args LuaGuiElement.add_parameters Table containing arguments for the method.
--- @see LuaGuiElement.add_parameters
--- @return LuaGuiElement The GUI element that was added.
----
function LuaGuiElement.add(args) end

----
--- Inserts a string at the end or at the given index of this dropdown or listbox.
--- 
--- @param string LocalisedString The text to insert.
--- @param index uint? The index at which to insert the item.
----
function LuaGuiElement.add_item(string, index) end

----
--- Adds the given tab and content widgets to this tabbed pane as a new tab.
--- 
--- @param tab LuaGuiElement The tab to add, must be a GUI element of type "tab".
--- @param content LuaGuiElement The content to show when this tab is selected. Can be any type of GUI element.
----
function LuaGuiElement.add_tab(tab, content) end

----
--- Moves this GUI element to the "front" so it will draw over other elements.
--- 
----
function LuaGuiElement.bring_to_front() end

