require('.classes-1')
require('.classes-2')
require('.classes-3')
require('.classes-4')
require('.classes-5')
require('.classes-6')

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaItemStack.help() end

----
--- Import a supported item (blueprint, blueprint-book, deconstruction-planner, upgrade-planner, item-with-tags) from a string.
--- 
--- @param data string The string to import
--- @return int 0 if the import succeeded with no errors. -1 if the import succeeded with errors. 1 if the import failed.
----
function LuaItemStack.import_stack(data) end

----
--- Is this blueprint item setup? I.e. is it a non-empty blueprint?
--- 
--- @return boolean 
----
function LuaItemStack.is_blueprint_setup() end

----
--- Removes a tag with the given name.
--- 
--- @param tag string 
--- @return boolean If the tag existed and was removed.
----
function LuaItemStack.remove_tag(tag) end

----
--- Set new entities to be a part of this blueprint.
--- 
--- @param entities BlueprintEntity[] The new blueprint entities.
----
function LuaItemStack.set_blueprint_entities(entities) end

----
--- Sets the given tag on the given blueprint entity index in this blueprint item.
--- 
--- @param index uint The entity index.
--- @param tag string The tag to set.
--- @param value AnyBasic The tag value to set or `nil` to clear the tag.
----
function LuaItemStack.set_blueprint_entity_tag(index, tag, value) end

----
--- Sets the tags on the given blueprint entity index in this blueprint item.
--- 
--- @param index uint The entity index
--- @param tags Tags 
----
function LuaItemStack.set_blueprint_entity_tags(index, tags) end

----
--- Set specific tiles in this blueprint.
--- 
--- @param tiles Tile[] Tiles to be a part of the blueprint.
----
function LuaItemStack.set_blueprint_tiles(tiles) end

----
--- Sets the entity filter at the given index for this deconstruction item.
--- 
--- @param index uint 
--- @param filter LuaEntityPrototype|string|LuaEntity|nil Writing `nil` removes the filter.
--- @return boolean Whether the new filter was successfully set (ie. was valid).
----
function LuaItemStack.set_entity_filter(index, filter) end

----
--- Sets the module filter at the given index for this upgrade item.
--- 
--- @param index uint The index of the mapper to set.
--- @param type string `from` or `to`.
--- @param filter UpgradeFilter The filter to set or `nil`
----
function LuaItemStack.set_mapper(index, type, filter) end

----
--- Set this item stack to another item stack.
--- 
--- @param stack ItemStackIdentification? Item stack to set it to. Omitting this parameter or passing `nil` will clear this item stack, as if [LuaItemStack::clear](LuaItemStack::clear) was called.
--- @return boolean Whether the stack was set successfully. Returns `false` if this stack was not [valid for write](LuaItemStack::can_set_stack).
----
function LuaItemStack.set_stack(stack) end

----
--- Sets the tag with the given name and value.
--- 
--- @param tag_name string 
--- @param tag AnyBasic 
----
function LuaItemStack.set_tag(tag_name, tag) end

----
--- Sets the tile filter at the given index for this deconstruction item.
--- 
--- @param index uint 
--- @param filter string|LuaTile|LuaTilePrototype Setting to nil erases the filter.
--- @return boolean Whether the new filter was successfully set (ie. was valid).
----
function LuaItemStack.set_tile_filter(index, filter) end

----
--- Swaps this item stack with the given item stack if allowed.
--- 
--- @param stack LuaItemStack 
--- @return boolean Whether the 2 stacks were swapped successfully.
----
function LuaItemStack.swap_stack(stack) end

----
--- Transfers the given item stack into this item stack.
--- 
--- @param stack ItemStackIdentification 
--- @return boolean `true` if the full stack was transferred.
----
function LuaItemStack.transfer_stack(stack) end

----
--- Use the capsule item with the entity as the source, targeting the given position.
--- 
--- @param entity LuaEntity The entity to use the capsule item with.
--- @param target_position MapPosition The position to use the capsule item with.
--- @return LuaEntity[] Array of the entities that were created by the capsule action.
----
function LuaItemStack.use_capsule(entity, target_position) end


----
--- @class LuaLampControlBehavior: LuaGenericOnOffControlBehavior
--- Control behavior for lamps.
--- 
--- @field use_colors boolean `true` if the lamp should set the color from the circuit network signals.
--- @field color Color? The color the lamp is showing, if any.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaLampControlBehavior = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaLampControlBehavior.help() end


----
--- @class LuaLazyLoadedValue
--- A lazily loaded value. For performance reasons, we sometimes return a custom lazily-loaded value type instead of the native Lua value. This custom type lazily constructs the necessary value when [LuaLazyLoadedValue::get](LuaLazyLoadedValue::get) is called, therefore preventing its unnecessary construction in some cases.
--- 
--- An instance of LuaLazyLoadedValue is only valid during the event it was created from and cannot be saved.
--- 
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaLazyLoadedValue = { }

----
--- Gets the value of this lazy loaded value.
--- 
--- @return Any 
----
function LuaLazyLoadedValue.get() end

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaLazyLoadedValue.help() end


----
--- @class LuaLogisticCell
--- Logistic cell of a particular [LuaEntity](LuaEntity). A "Logistic Cell" is the given name for settings and properties used by what would normally be seen as a "Roboport". A logistic cell however doesn't have to be attached to the roboport entity (the character has one for the personal roboport).
--- 
--- @field logistic_radius float Logistic radius of this cell.
--- @field logistics_connection_distance float Logistic connection distance of this cell.
--- @field construction_radius float Construction radius of this cell.
--- @field stationed_logistic_robot_count uint Number of stationed logistic robots in this cell.
--- @field stationed_construction_robot_count uint Number of stationed construction robots in this cell.
--- @field mobile boolean `true` if this is a mobile cell. In vanilla, only the logistic cell created by a character's personal roboport is mobile.
--- @field transmitting boolean `true` if this cell is active.
--- @field charge_approach_distance float Radius at which the robots hover when waiting to be charged.
--- @field charging_robot_count uint Number of robots currently charging.
--- @field to_charge_robot_count uint Number of robots waiting to charge.
--- @field owner LuaEntity This cell's owner.
--- @field logistic_network LuaLogisticNetwork? The network that owns this cell, if any.
--- @field neighbours LuaLogisticCell[] Neighbouring cells.
--- @field charging_robots LuaEntity[] Robots currently being charged.
--- @field to_charge_robots LuaEntity[] Robots waiting to charge.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaLogisticCell = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaLogisticCell.help() end

----
--- Is a given position within the construction range of this cell?
--- 
--- @param position MapPosition 
--- @return boolean 
----
function LuaLogisticCell.is_in_construction_range(position) end

----
--- Is a given position within the logistic range of this cell?
--- 
--- @param position MapPosition 
--- @return boolean 
----
function LuaLogisticCell.is_in_logistic_range(position) end

----
--- Are two cells neighbours?
--- 
--- @param other LuaLogisticCell 
--- @return boolean 
----
function LuaLogisticCell.is_neighbour_with(other) end


----
--- @class LuaLogisticContainerControlBehavior: LuaControlBehavior
--- Control behavior for logistic chests.
--- 
--- @field circuit_mode_of_operation defines.control_behavior.logistic_container.circuit_mode_of_operation The circuit mode of operations for the logistic container. Can only be set on containers whose [logistic_mode](LuaEntityPrototype::logistic_mode) is set to "requester".
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaLogisticContainerControlBehavior = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaLogisticContainerControlBehavior.help() end


----
--- @class LuaLogisticNetwork
--- A single logistic network of a given force on a given surface.
--- 
--- @field force LuaForce The force this logistic network belongs to.
--- @field available_logistic_robots uint Number of logistic robots available for a job.
--- @field all_logistic_robots uint The total number of logistic robots in the network (idle and active + in roboports).
--- @field available_construction_robots uint Number of construction robots available for a job.
--- @field all_construction_robots uint The total number of construction robots in the network (idle and active + in roboports).
--- @field robot_limit uint Maximum number of robots the network can work with. Currently only used for the personal roboport.
--- @field cells LuaLogisticCell[] All cells in this network.
--- @field providers LuaEntity[] All entities that have logistic provider points in this network.
--- @field empty_providers LuaEntity[] All entities that have empty logistic provider points in this network.
--- @field requesters LuaEntity[] All entities that have logistic requester points in this network.
--- @field storages LuaEntity[] All entities that have logistic storage points in this network.
--- @field logistic_members LuaEntity[] All other entities that have logistic points in this network (inserters mostly).
--- @field provider_points LuaLogisticPoint[] All things that have provider points in this network.
--- @field passive_provider_points LuaLogisticPoint[] All passive provider points in this network.
--- @field active_provider_points LuaLogisticPoint[] All active provider points in this network.
--- @field empty_provider_points LuaLogisticPoint[] All things that have empty provider points in this network.
--- @field requester_points LuaLogisticPoint[] All things that have requester points in this network.
--- @field storage_points LuaLogisticPoint[] All things that have storage points in this network.
--- @field robots LuaEntity[] All robots in this logistic network.
--- @field construction_robots LuaEntity[] All construction robots in this logistic network.
--- @field logistic_robots LuaEntity[] All logistic robots in this logistic network.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaLogisticNetwork = { }

----
--- Can the network satisfy a request for a given item and count.
--- 
--- @param item string Item name to check.
--- @param count uint? Count to check. Defaults to 1.
--- @param include_buffers boolean? Should buffers be considered? Defaults to false.
--- @return boolean Whether the network can satisfy the request.
----
function LuaLogisticNetwork.can_satisfy_request(item, count, include_buffers) end

----
--- Find logistic cell closest to a given position.
--- 
--- @param position MapPosition 
--- @return LuaLogisticCell? `nil` if no cell was found.
----
function LuaLogisticNetwork.find_cell_closest_to(position) end

----
--- Get item counts for the entire network, similar to how [LuaInventory::get_contents](LuaInventory::get_contents) does.
--- 
--- @return table<string,uint> A mapping of item prototype names to the number available in the network.
----
function LuaLogisticNetwork.get_contents() end

----
--- Count given or all items in the network or given members.
--- 
--- @param item string? Item name to count. If not given, gives counts of all items in the network.
--- @param member string? Logistic members to check, must be either `"storage"` or `"providers"`. If not given, gives count in the entire network.
--- @return int 
----
function LuaLogisticNetwork.get_item_count(item, member) end

----
--- Get the amount of items of the given type indexed by the storage member.
--- 
--- @param item string Item name to check.
--- @return table<string,uint> A mapping of member types ("storage", "passive-provider", "buffer", "active-provider") to the number available in the members.
----
function LuaLogisticNetwork.get_supply_counts(item) end

----
--- Gets the logistic points with of the given type indexed by the storage member.
--- 
--- @param item string Item name to check.
--- @return table<string,LuaLogisticPoint[]> A mapping of member types ("storage", "passive-provider", "buffer", "active-provider") to an array of LuaLogisticPoint.
----
function LuaLogisticNetwork.get_supply_points(item) end

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaLogisticNetwork.help() end

----
--- Insert items into the logistic network. This will actually insert the items into some logistic chests.
--- 
--- @param item ItemStackIdentification What to insert.
--- @param members string? Which logistic members to insert the items to. Must be `"storage"`, `"storage-empty"` (storage chests that are completely empty), `"storage-empty-slot"` (storage chests that have an empty slot), or `"requester"`. If not specified, inserts items into the logistic network in the usual order.
--- @return uint Number of items actually inserted.
----
function LuaLogisticNetwork.insert(item, members) end

----
--- Remove items from the logistic network. This will actually remove the items from some logistic chests.
--- 
--- @param item ItemStackIdentification What to remove.
--- @param members string? Which logistic members to remove from. Must be `"storage"`, `"passive-provider"`, `"buffer"`, or `"active-provider"`. If not specified, removes from the network in the usual order.
--- @return uint Number of items removed.
----
function LuaLogisticNetwork.remove_item(item, members) end

----
--- @class LuaLogisticNetwork.select_drop_point_parameters
--- Parameters class for the method `LuaLogisticNetwork.select_drop_point`
--- 
--- @field stack ItemStackIdentification Name of the item to select.
--- @field members string? When given, it will find from only the specific type of member. Must be `"storage"`, `"storage-empty"`, `"storage-empty-slot"` or `"requester"`. If not specified, selects with normal priorities.
----

----
--- Find a logistic point to drop the specific item stack.
--- 
--- @param args LuaLogisticNetwork.select_drop_point_parameters Table containing arguments for the method.
--- @see LuaLogisticNetwork.select_drop_point_parameters
--- @return LuaLogisticPoint? `nil` if no point was found.
----
function LuaLogisticNetwork.select_drop_point(args) end

----
--- @class LuaLogisticNetwork.select_pickup_point_parameters
--- Parameters class for the method `LuaLogisticNetwork.select_pickup_point`
--- 
--- @field name string Name of the item to select.
--- @field position MapPosition? When given, it will find the storage 'best' storage point from this position.
--- @field include_buffers boolean? Whether to consider buffer chests or not. Defaults to false. Only considered if selecting with position.
--- @field members string? When given, it will find from only the specific type of member. Must be `"storage"`, `"passive-provider"`, `"buffer"` or `"active-provider"`. If not specified, selects with normal priorities. Not considered if position is specified.
----

----
--- Find the 'best' logistic point with this item ID and from the given position or from given chest type.
--- 
--- @param args LuaLogisticNetwork.select_pickup_point_parameters Table containing arguments for the method.
--- @see LuaLogisticNetwork.select_pickup_point_parameters
--- @return LuaLogisticPoint? `nil` if no point was found.
----
function LuaLogisticNetwork.select_pickup_point(args) end


----
--- @class LuaLogisticPoint
--- Logistic point of a particular [LuaEntity](LuaEntity). A "Logistic point" is the name given for settings and properties used by requester, provider, and storage points in a given logistic network. These "points" don't have to be a logistic container but often are. One other entity that can own several points is the "character" character type entity.
--- 
--- @field owner LuaEntity The [LuaEntity](LuaEntity) owner of this [LuaLogisticPoint](LuaLogisticPoint).
--- @field logistic_network LuaLogisticNetwork 
--- @field logistic_member_index uint The Logistic member index of this logistic point.
--- @field filters LogisticFilter[]? The logistic filters for this logistic point, if this uses any.
--- @field mode defines.logistic_mode The logistic mode.
--- @field force LuaForce The force of this logistic point.
--- @field targeted_items_pickup table<string,uint> Items targeted to be picked up from this logistic point by robots. The attribute is a dictionary mapping the item prototype names to their item counts.
--- @field targeted_items_deliver table<string,uint> Items targeted to be dropped off into this logistic point by robots. The attribute is a dictionary mapping the item prototype names to their item counts.
--- @field exact boolean If this logistic point is using the exact mode. In exact mode robots never over-deliver requests.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaLogisticPoint = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaLogisticPoint.help() end


----
--- @class LuaMiningDrillControlBehavior: LuaGenericOnOffControlBehavior
--- Control behavior for mining drills.
--- 
--- @field circuit_enable_disable boolean `true` if this drill is enabled or disabled using the logistics or circuit condition.
--- @field circuit_read_resources boolean `true` if this drill should send the resources in the field to the circuit network. Which resources depends on [LuaMiningDrillControlBehavior::resource_read_mode](LuaMiningDrillControlBehavior::resource_read_mode)
--- @field resource_read_mode defines.control_behavior.mining_drill.resource_read_mode If the mining drill should send just the resources in its area or the entire field it's on to the circuit network.
--- @field resource_read_targets LuaEntity[] The resource entities that the mining drill will send information about to the circuit network or an empty array.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaMiningDrillControlBehavior = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaMiningDrillControlBehavior.help() end


----
--- @class LuaModSettingPrototype
--- Prototype of a mod setting.
--- 
--- @field name string Name of this prototype.
--- @field order string The string used to alphabetically sort these prototypes. It is a simple string that has no additional semantic meaning.
--- @field localised_name LocalisedString 
--- @field localised_description LocalisedString 
--- @field mod string The mod that owns this setting.
--- @field setting_type string 
--- @field default_value double|string|int|boolean The default value of this setting.
--- @field minimum_value double|int? The minimum value for this setting. `nil` if this setting type doesn't support a minimum.
--- @field maximum_value double|int? The maximum value for this setting. `nil` if this setting type doesn't support a maximum.
--- @field allowed_values double[]|int[]|string[]? The allowed values for this setting. `nil` if this setting doesn't use the a fixed set of values.
--- @field allow_blank boolean? Whether this string setting allows blank values. `nil` if not a string setting.
--- @field auto_trim boolean? Whether this string setting auto-trims values. `nil` if not a string setting
--- @field hidden boolean Whether this setting is hidden from the GUI.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaModSettingPrototype = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaModSettingPrototype.help() end


----
--- @class LuaModuleCategoryPrototype
--- Prototype of a module category.
--- 
--- @field name string Name of this prototype.
--- @field order string The string used to alphabetically sort these prototypes. It is a simple string that has no additional semantic meaning.
--- @field localised_name LocalisedString 
--- @field localised_description LocalisedString 
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaModuleCategoryPrototype = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaModuleCategoryPrototype.help() end


----
--- @class LuaNamedNoiseExpression
--- Prototype of a named noise expression.
--- 
--- @field name string Name of this prototype.
--- @field order string The string used to alphabetically sort these prototypes. It is a simple string that has no additional semantic meaning.
--- @field localised_name LocalisedString 
--- @field localised_description LocalisedString 
--- @field intended_property string Name of the property that this expression is intended to provide a value for, if any.
--- @field expression NoiseExpression The expression itself.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaNamedNoiseExpression = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaNamedNoiseExpression.help() end


----
--- @class LuaNoiseLayerPrototype
--- Prototype of a noise layer.
--- 
--- @field name string Name of this prototype.
--- @field order string The string used to alphabetically sort these prototypes. It is a simple string that has no additional semantic meaning.
--- @field localised_name LocalisedString 
--- @field localised_description LocalisedString 
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaNoiseLayerPrototype = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaNoiseLayerPrototype.help() end


----
--- @class LuaParticlePrototype
--- Prototype of an optimized particle.
--- 
--- @field name string Name of this prototype.
--- @field order string The string used to alphabetically sort these prototypes. It is a simple string that has no additional semantic meaning.
--- @field localised_name LocalisedString 
--- @field localised_description LocalisedString 
--- @field regular_trigger_effect TriggerEffectItem 
--- @field ended_in_water_trigger_effect TriggerEffectItem 
--- @field render_layer RenderLayer 
--- @field render_layer_when_on_ground RenderLayer 
--- @field life_time uint 
--- @field regular_trigger_effect_frequency uint 
--- @field movement_modifier_when_on_ground float 
--- @field movement_modifier float 
--- @field mining_particle_frame_speed float 
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaParticlePrototype = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaParticlePrototype.help() end


----
--- @class LuaPermissionGroup
--- A permission group that defines what players in this group are allowed to do.
--- 
--- @field name string The name of this group.
--- @field players LuaPlayer[] The players in this group.
--- @field group_id uint The group ID
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaPermissionGroup = { }

----
--- Adds the given player to this group.
--- 
--- @param player PlayerIdentification 
--- @return boolean Whether the player was added.
----
function LuaPermissionGroup.add_player(player) end

----
--- Whether this group allows the given action.
--- 
--- @param action defines.input_action The action in question.
--- @return boolean 
----
function LuaPermissionGroup.allows_action(action) end

----
--- Destroys this group.
--- 
--- @return boolean Whether the group was successfully destroyed.
----
function LuaPermissionGroup.destroy() end

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaPermissionGroup.help() end

----
--- Removes the given player from this group.
--- 
--- @param player PlayerIdentification 
--- @return boolean Whether the player was removed.
----
function LuaPermissionGroup.remove_player(player) end

----
--- Sets whether this group allows the performance the given action.
--- 
--- @param action defines.input_action The action in question.
--- @param allow_action boolean Whether to allow the specified action.
--- @return boolean Whether the value was successfully applied.
----
function LuaPermissionGroup.set_allows_action(action, allow_action) end


----
--- @class LuaPermissionGroups
--- All permission groups.
--- 
--- @field groups LuaPermissionGroup[] All of the permission groups.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaPermissionGroups = { }

----
--- Creates a new permission group.
--- 
--- @param name string? 
--- @return LuaPermissionGroup? `nil` if the calling player doesn't have permission to make groups.
----
function LuaPermissionGroups.create_group(name) end

----
--- Gets the permission group with the given name or group ID.
--- 
--- @param group string|uint 
--- @return LuaPermissionGroup? `nil` if there is no matching group.
----
function LuaPermissionGroups.get_group(group) end

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaPermissionGroups.help() end


----
--- @class LuaPlayer: LuaControl
--- A player in the game. Pay attention that a player may or may not have a character, which is the [LuaEntity](LuaEntity) of the little guy running around the world doing things.
--- 
--- @field character LuaEntity? The character attached to this player, if any. Returns `nil` when the player is disconnected (see [LuaPlayer::connected](LuaPlayer::connected)).
--- @field cutscene_character LuaEntity? When in a cutscene; the character this player would be using once the cutscene is over, if any. Returns `nil` when the player is disconnected (see [LuaPlayer::connected](LuaPlayer::connected)).
--- @field index uint This player's index in [LuaGameScript::players](LuaGameScript::players) (unique ID). It is assigned when a player is created, and remains so (even when the player is not [connected](LuaPlayer::connected)) until the player is irreversably [removed](on_player_removed). Indexes of removed players can be reused.
--- @field gui LuaGui 
--- @field opened_self boolean `true` if the player opened itself. I.e. if they opened the character or god-controller GUI.
--- @field controller_type defines.controllers 
--- @field stashed_controller_type defines.controllers? The stashed controller type, if any.
--- @field game_view_settings GameViewSettings The player's game view settings.
--- @field minimap_enabled boolean `true` if the minimap is visible.
--- @field color Color The color associated with the player. This will be used to tint the player's character as well as their buildings and vehicles.
--- @field chat_color Color The color used when this player talks in game.
--- @field name string The player's username.
--- @field tag string The tag that is shown after the player in chat and on the map.
--- @field connected boolean `true` if the player is currently connected to the game.
--- @field admin boolean `true` if the player is an admin.
--- @field entity_copy_source LuaEntity? The source entity used during entity settings copy-paste, if any.
--- @field afk_time uint How many ticks since the last action of this player
--- @field online_time uint How many ticks did this player spend playing this save (all sessions combined)
--- @field last_online uint At what tick this player was last online.
--- @field permission_group LuaPermissionGroup? The permission group this player is part of, if any.
--- @field mod_settings table<string,ModSetting> The current per-player settings for the this player, indexed by prototype name. Returns the same structure as [LuaSettings::get_player_settings](LuaSettings::get_player_settings). This table becomes invalid if its associated player does.  Even though this attribute is marked as read-only, individual settings can be changed by overwriting their [ModSetting](ModSetting) table. Mods can only change their own settings. Using the in-game console, all player settings can be changed.
--- @field ticks_to_respawn uint? The number of ticks until this player will respawn. `nil` if this player is not waiting to respawn.  Set to `nil` to immediately respawn the player.
--- @field display_resolution DisplayResolution The display resolution for this player.
--- @field display_scale double The display scale for this player.
--- @field blueprint_to_setup LuaItemStack The item stack containing a blueprint to be setup.
--- @field render_mode defines.render_mode The render mode of the player, like map or zoom to world. The render mode can be set using [LuaPlayer::open_map](LuaPlayer::open_map), [LuaPlayer::zoom_to_world](LuaPlayer::zoom_to_world) and [LuaPlayer::close_map](LuaPlayer::close_map).
--- @field spectator boolean If `true`, zoom-to-world noise effect will be disabled and environmental sounds will be based on zoom-to-world view instead of position of player's character.
--- @field show_on_map boolean If `true`, circle and name of given player is rendered on the map/chart.
--- @field remove_unfiltered_items boolean If items not included in this map editor infinity inventory filters should be removed.
--- @field infinity_inventory_filters InfinityInventoryFilter[] The filters for this map editor infinity inventory settings.
--- @field auto_sort_main_inventory boolean If the main inventory will be auto sorted.
--- @field hand_location ItemStackLocation? The original location of the item in the cursor, marked with a hand. `nil` if the cursor stack is empty. When writing, the specified inventory slot must be empty and the cursor stack must not be empty.
--- @field cursor_stack_temporary boolean Returns true if the current item stack in cursor will be destroyed after clearing the cursor. Manually putting it into inventory still preserves the item. If the cursor stack is not one of the supported types (blueprint, blueprint-book, deconstruction-planner, upgrade-planner), write operation will be silently ignored.
--- @field drag_target DragTarget? The wire drag target for this player, if any.
--- @field zoom double The player's zoom-level.
--- @field map_view_settings MapViewSettings The player's map view settings. To write to this, use a table containing the fields that should be changed.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaPlayer = { }

----
--- Gets a copy of the currently selected blueprint in the clipboard queue into the player's cursor, as if the player activated Paste.
--- 
----
function LuaPlayer.activate_paste() end

----
--- Adds an alert to this player for the given entity of the given alert type.
--- 
--- @param entity LuaEntity 
--- @param type defines.alert_type 
----
function LuaPlayer.add_alert(entity, type) end

----
--- Adds a custom alert to this player.
--- 
--- @param entity LuaEntity If the alert is clicked, the map will open at the position of this entity.
--- @param icon SignalID 
--- @param message LocalisedString 
--- @param show_on_map boolean 
----
function LuaPlayer.add_custom_alert(entity, icon, message, show_on_map) end

----
--- Adds the given recipe to the list of recipe notifications for this player.
--- 
--- @param recipe string Name of the recipe prototype to add.
----
function LuaPlayer.add_recipe_notification(recipe) end

----
--- Adds the given blueprint to this player's clipboard queue.
--- 
--- @param blueprint LuaItemStack The blueprint to add.
----
function LuaPlayer.add_to_clipboard(blueprint) end

----
--- Associates a character with this player.
--- 
--- @param character LuaEntity The character entity.
----
function LuaPlayer.associate_character(character) end

----
--- @class LuaPlayer.build_from_cursor_parameters
--- Parameters class for the method `LuaPlayer.build_from_cursor`
--- 
--- @field position MapPosition Where the entity would be placed
--- @field direction defines.direction? Direction the entity would be placed
--- @field alt boolean? If alt build should be used instead of normal build. Defaults to normal.
--- @field terrain_building_size uint? The size for building terrain if building terrain. Defaults to 2.
--- @field skip_fog_of_war boolean? If chunks covered by fog-of-war are skipped.
----

----
--- Builds whatever is in the cursor on the surface the player is on. The cursor stack will automatically be reduced as if the player built normally.
--- 
--- @param args LuaPlayer.build_from_cursor_parameters Table containing arguments for the method.
--- @see LuaPlayer.build_from_cursor_parameters
----
function LuaPlayer.build_from_cursor(args) end

----
--- @class LuaPlayer.can_build_from_cursor_parameters
--- Parameters class for the method `LuaPlayer.can_build_from_cursor`
--- 
--- @field position MapPosition Where the entity would be placed
--- @field direction defines.direction? Direction the entity would be placed
--- @field alt boolean? If alt build should be used instead of normal build. Defaults to normal.
--- @field terrain_building_size uint? The size for building terrain if building terrain. Defaults to 2.
--- @field skip_fog_of_war boolean? If chunks covered by fog-of-war are skipped.
----

----
--- Checks if this player can build what ever is in the cursor on the surface the player is on.
--- 
--- @param args LuaPlayer.can_build_from_cursor_parameters Table containing arguments for the method.
--- @see LuaPlayer.can_build_from_cursor_parameters
--- @return boolean 
----
function LuaPlayer.can_build_from_cursor(args) end

----
--- @class LuaPlayer.can_place_entity_parameters
--- Parameters class for the method `LuaPlayer.can_place_entity`
--- 
--- @field name string Name of the entity to check.
--- @field position MapPosition Where the entity would be placed.
--- @field direction defines.direction? Direction the entity would be placed. Defaults to `north`.
----

----
--- Checks if this player can build the given entity at the given location on the surface the player is on.
--- 
--- @param args LuaPlayer.can_place_entity_parameters Table containing arguments for the method.
--- @see LuaPlayer.can_place_entity_parameters
--- @return boolean 
----
function LuaPlayer.can_place_entity(args) end

----
--- Clear the chat console.
--- 
----
function LuaPlayer.clear_console() end

----
--- Invokes the "clear cursor" action on the player as if the user pressed it.
--- 
--- @return boolean Whether the cursor is now empty.
----
function LuaPlayer.clear_cursor() end

----
--- Clears all recipe notifications for this player.
--- 
----
function LuaPlayer.clear_recipe_notifications() end

----
--- Clears the players selection tool selection position.
--- 
----
function LuaPlayer.clear_selection() end

----
--- Queues request to switch to the normal game view from the map or zoom to world view. Render mode change requests are processed before rendering of the next frame.
--- 
----
function LuaPlayer.close_map() end

----
--- @class LuaPlayer.connect_to_server_parameters
--- Parameters class for the method `LuaPlayer.connect_to_server`
--- 
--- @field address string The server (address:port) if port is not given the default Factorio port is used.
--- @field name LocalisedString? The name of the server.
--- @field description LocalisedString? 
--- @field password string? The password if different from the one used to join this game. Note, if the current password is not empty but the one required to join the new server is an empty string should be given for this field.
----

----
--- Asks the player if they would like to connect to the given server.
--- 
--- @param args LuaPlayer.connect_to_server_parameters Table containing arguments for the method.
--- @see LuaPlayer.connect_to_server_parameters
----
function LuaPlayer.connect_to_server(args) end

----
--- Creates and attaches a character entity to this player.
--- 
--- @param character string? The character to create else the default is used.
--- @return boolean Whether the character was created.
----
function LuaPlayer.create_character(character) end

----
--- @class LuaPlayer.create_local_flying_text_parameters
--- Parameters class for the method `LuaPlayer.create_local_flying_text`
--- 
--- @field text LocalisedString The flying text to show.
--- @field position MapPosition? The location on the map at which to show the flying text.
--- @field create_at_cursor boolean? If `true`, the flying text is created at the player's cursor. Defaults to `false`.
--- @field color Color? The color of the flying text. Defaults to white text.
--- @field time_to_live uint? The amount of ticks that the flying text will be shown for. Defaults to `80`.
--- @field speed double? The speed at which the text rises upwards in tiles/second. Can't be a negative value.
----

----
--- Spawn flying text that is only visible to this player. Either `position` or `create_at_cursor` are required. When `create_at_cursor` is `true`, all parameters other than `text` are ignored.
--- 
--- @param args LuaPlayer.create_local_flying_text_parameters Table containing arguments for the method.
--- @see LuaPlayer.create_local_flying_text_parameters
----
function LuaPlayer.create_local_flying_text(args) end

----
--- Disables alerts for the given alert category.
--- 
--- @param alert_type defines.alert_type 
--- @return boolean Whether the alert type was disabled (false if it was already disabled).
----
function LuaPlayer.disable_alert(alert_type) end

----
--- Disable recipe groups.
--- 
----
function LuaPlayer.disable_recipe_groups() end

----
--- Disable recipe subgroups.
--- 
----
function LuaPlayer.disable_recipe_subgroups() end

