require('.classes-1')
require('.classes-2')
require('.classes-3')
require('.classes-4')
require('.classes-5')
require('.classes-6')
require('.classes-7')

----
--- Disassociates a character from this player. This is functionally the same as setting [LuaEntity::associated_player](LuaEntity::associated_player) to `nil`.
--- 
--- @param character LuaEntity The character entity
----
function LuaPlayer.disassociate_character(character) end

----
--- @class LuaPlayer.drag_wire_parameters
--- Parameters class for the method `LuaPlayer.drag_wire`
--- 
--- @field position MapPosition Position at which cursor was clicked. Used only to decide which side of arithmetic combinator, decider combinator or power switch is to be connected. Entity itself to be connected is based on the player's selected entity.
----

----
--- Start/end wire dragging at the specified location, wire type is based on the cursor contents
--- 
--- @param args LuaPlayer.drag_wire_parameters Table containing arguments for the method.
--- @see LuaPlayer.drag_wire_parameters
--- @return boolean `true` if the action did something
----
function LuaPlayer.drag_wire(args) end

----
--- Enables alerts for the given alert category.
--- 
--- @param alert_type defines.alert_type 
--- @return boolean Whether the alert type was enabled (false if it was already enabled).
----
function LuaPlayer.enable_alert(alert_type) end

----
--- Enable recipe groups.
--- 
----
function LuaPlayer.enable_recipe_groups() end

----
--- Enable recipe subgroups.
--- 
----
function LuaPlayer.enable_recipe_subgroups() end

----
--- Exit the current cutscene. Errors if not in a cutscene.
--- 
----
function LuaPlayer.exit_cutscene() end

----
--- Gets which quick bar page is being used for the given screen page or `nil` if not known.
--- 
--- @param index uint The screen page. Index 1 is the top row in the gui. Index can go beyond the visible number of bars on the screen to account for the interface config setting change.
--- @return uint8? 
----
function LuaPlayer.get_active_quick_bar_page(index) end

----
--- @class LuaPlayer.get_alerts_parameters
--- Parameters class for the method `LuaPlayer.get_alerts`
--- 
--- @field entity LuaEntity? 
--- @field prototype LuaEntityPrototype? 
--- @field position MapPosition? 
--- @field type defines.alert_type? 
--- @field surface SurfaceIdentification? 
----

----
--- Get all alerts matching the given filters, or all alerts if no filters are given.
--- 
--- @param args LuaPlayer.get_alerts_parameters Table containing arguments for the method.
--- @see LuaPlayer.get_alerts_parameters
--- @return table<uint,table<defines.alert_type,Alert[]>> A mapping of surface index to an array of arrays of [alerts](Alert) indexed by the [alert type](defines.alert_type).
----
function LuaPlayer.get_alerts(args) end

----
--- The characters associated with this player.
--- 
--- @return LuaEntity[] 
----
function LuaPlayer.get_associated_characters() end

----
--- Get the current goal description, as a localised string.
--- 
--- @return LocalisedString 
----
function LuaPlayer.get_goal_description() end

----
--- Gets the filter for this map editor infinity filters at the given index or `nil` if the filter index doesn't exist or is empty.
--- 
--- @param index uint The index to get.
--- @return InfinityInventoryFilter? 
----
function LuaPlayer.get_infinity_inventory_filter(index) end

----
--- Gets the quick bar filter for the given slot or `nil`.
--- 
--- @param index uint The slot index. 1 for the first slot of page one, 2 for slot two of page one, 11 for the first slot of page 2, etc.
--- @return LuaItemPrototype? 
----
function LuaPlayer.get_quick_bar_slot(index) end

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaPlayer.help() end

----
--- If the given alert type is currently enabled.
--- 
--- @param alert_type defines.alert_type 
--- @return boolean 
----
function LuaPlayer.is_alert_enabled(alert_type) end

----
--- If the given alert type is currently muted.
--- 
--- @param alert_type defines.alert_type 
--- @return boolean 
----
function LuaPlayer.is_alert_muted(alert_type) end

----
--- Is a custom Lua shortcut currently available?
--- 
--- @param prototype_name string Prototype name of the custom shortcut.
--- @return boolean 
----
function LuaPlayer.is_shortcut_available(prototype_name) end

----
--- Is a custom Lua shortcut currently toggled?
--- 
--- @param prototype_name string Prototype name of the custom shortcut.
--- @return boolean 
----
function LuaPlayer.is_shortcut_toggled(prototype_name) end

----
--- Jump to the specified cutscene waypoint. Only works when the player is viewing a cutscene.
--- 
--- @param waypoint_index uint 
----
function LuaPlayer.jump_to_cutscene_waypoint(waypoint_index) end

----
--- Logs a dictionary of chunks -> active entities for the surface this player is on.
--- 
----
function LuaPlayer.log_active_entity_chunk_counts() end

----
--- Logs a dictionary of active entities -> count for the surface this player is on.
--- 
----
function LuaPlayer.log_active_entity_counts() end

----
--- Mutes alerts for the given alert category.
--- 
--- @param alert_type defines.alert_type 
--- @return boolean Whether the alert type was muted (false if it was already muted).
----
function LuaPlayer.mute_alert(alert_type) end

----
--- Queues a request to open the map at the specified position. If the map is already opened, the request will simply set the position, scale, and entity to follow. Render mode change requests are processed before rendering of the next frame.
--- 
--- @param position MapPosition 
--- @param scale double? 
--- @param entity LuaEntity? The entity to follow. If not given the current entity being followed will be cleared.
----
function LuaPlayer.open_map(position, scale, entity) end

----
--- Invokes the "smart pipette" action on the player as if the user pressed it.
--- 
--- @param entity LuaEntityPrototype|string|LuaEntity 
--- @return boolean Whether the smart pipette found something to place.
----
function LuaPlayer.pipette_entity(entity) end

----
--- @class LuaPlayer.play_sound_parameters
--- Parameters class for the method `LuaPlayer.play_sound`
--- 
--- @field path SoundPath The sound to play.
--- @field position MapPosition? Where the sound should be played. If not given, it's played at the current position of the player.
--- @field volume_modifier double? The volume of the sound to play. Must be between 0 and 1 inclusive.
--- @field override_sound_type SoundType? The volume mixer to play the sound through. Defaults to the default mixer for the given sound type.
----

----
--- Play a sound for this player.
--- 
--- @param args LuaPlayer.play_sound_parameters Table containing arguments for the method.
--- @see LuaPlayer.play_sound_parameters
----
function LuaPlayer.play_sound(args) end

----
--- Print text to the chat console.
--- 
--- @param message LocalisedString 
--- @param color Color? 
----
function LuaPlayer.print(message, color) end

----
--- Print entity statistics to the player's console.
--- 
--- @param entities string[]? Entity prototypes to get statistics for. If not specified or empty, display statistics for all entities.
----
function LuaPlayer.print_entity_statistics(entities) end

----
--- Print LuaObject counts per mod.
--- 
----
function LuaPlayer.print_lua_object_statistics() end

----
--- Print construction robot job counts to the players console.
--- 
----
function LuaPlayer.print_robot_jobs() end

----
--- @class LuaPlayer.remove_alert_parameters
--- Parameters class for the method `LuaPlayer.remove_alert`
--- 
--- @field entity LuaEntity? 
--- @field prototype LuaEntityPrototype|string? 
--- @field position MapPosition? 
--- @field type defines.alert_type? 
--- @field surface SurfaceIdentification? 
--- @field icon SignalID? 
--- @field message LocalisedString? 
----

----
--- Removes all alerts matching the given filters or if an empty filters table is given all alerts are removed.
--- 
--- @param args LuaPlayer.remove_alert_parameters Table containing arguments for the method.
--- @see LuaPlayer.remove_alert_parameters
----
function LuaPlayer.remove_alert(args) end

----
--- Requests a translation for the given localised string. If the request is successful, the [on_string_translated](on_string_translated) event will be fired with the results.
--- 
--- @param localised_string LocalisedString 
--- @return uint? The unique ID for the requested translation.
----
function LuaPlayer.request_translation(localised_string) end

----
--- Requests translation for the given set of localised strings. If the request is successful, a [on_string_translated](on_string_translated) event will be fired for each string with the results.
--- 
--- @param localised_strings LocalisedString[] 
--- @return uint[]? The unique IDs for the requested translations.
----
function LuaPlayer.request_translations(localised_strings) end

----
--- Sets which quick bar page is being used for the given screen page.
--- 
--- @param screen_index uint The screen page. Index 1 is the top row in the gui. Index can go beyond the visible number of bars on the screen to account for the interface config setting change.
--- @param page_index uint The new quick bar page.
----
function LuaPlayer.set_active_quick_bar_page(screen_index, page_index) end

----
--- @class LuaPlayer.set_controller_parameters
--- Parameters class for the method `LuaPlayer.set_controller`
--- 
--- @field type defines.controllers Which controller to use.
--- @field character LuaEntity? Entity to control. Mandatory when `type` is [defines.controllers.character](defines.controllers.character), ignored otherwise.
--- @field waypoints CutsceneWaypoint? List of waypoints for the cutscene controller. This parameter is mandatory when `type` is [defines.controllers.cutscene](defines.controllers.cutscene).
--- @field start_position MapPosition? If specified and `type` is [defines.controllers.cutscene](defines.controllers.cutscene), the cutscene will start at this position. If not given the start position will be the player position.
--- @field start_zoom double? If specified and `type` is [defines.controllers.cutscene](defines.controllers.cutscene), the cutscene will start at this zoom level. If not given the start zoom will be the players zoom.
--- @field final_transition_time uint? If specified and `type` is [defines.controllers.cutscene](defines.controllers.cutscene), it is the time in ticks it will take for the camera to pan from the final waypoint back to the starting position. If not given the camera will not pan back to the start position/zoom.
--- @field chart_mode_cutoff double? If specified and `type` is [defines.controllers.cutscene](defines.controllers.cutscene), the game will switch to chart-mode (map zoomed out) rendering when the zoom level is less than this value.
----

----
--- Set the controller type of the player.
--- 
--- @param args LuaPlayer.set_controller_parameters Table containing arguments for the method.
--- @see LuaPlayer.set_controller_parameters
----
function LuaPlayer.set_controller(args) end

----
--- Setup the screen to be shown when the game is finished.
--- 
--- @param message LocalisedString Message to be shown.
--- @param file string? Path to image to be shown.
----
function LuaPlayer.set_ending_screen_data(message, file) end

----
--- Set the text in the goal window (top left).
--- 
--- @param text LocalisedString? The text to display. Lines can be delimited with `\n`. Passing an empty string or omitting this parameter entirely will make the goal window disappear.
--- @param only_update boolean? When `true`, won't play the "goal updated" sound.
----
function LuaPlayer.set_goal_description(text, only_update) end

----
--- Sets the filter for this map editor infinity filters at the given index.
--- 
--- @param index uint The index to set.
--- @param filter InfinityInventoryFilter|nil The new filter or `nil` to clear the filter.
----
function LuaPlayer.set_infinity_inventory_filter(index, filter) end

----
--- Sets the quick bar filter for the given slot.
--- 
--- @param index uint The slot index. 1 for the first slot of page one, 2 for slot two of page one, 11 for the first slot of page 2, etc.
--- @param filter LuaItemPrototype|string|LuaItemStack The filter or `nil`.
----
function LuaPlayer.set_quick_bar_slot(index, filter) end

----
--- Make a custom Lua shortcut available or unavailable.
--- 
--- @param prototype_name string Prototype name of the custom shortcut.
--- @param available boolean 
----
function LuaPlayer.set_shortcut_available(prototype_name, available) end

----
--- Toggle or untoggle a custom Lua shortcut
--- 
--- @param prototype_name string Prototype name of the custom shortcut.
--- @param toggled boolean 
----
function LuaPlayer.set_shortcut_toggled(prototype_name, toggled) end

----
--- Starts selection with selection tool from the specified position. Does nothing if the players cursor is not a selection tool.
--- 
--- @param position MapPosition The position to start selection from.
--- @param selection_mode string The type of selection to start. Can be `select`, `alternative-select`, `reverse-select`.
----
function LuaPlayer.start_selection(position, selection_mode) end

----
--- Toggles this player into or out of the map editor. Does nothing if this player isn't an admin or if the player doesn't have permission to use the map editor.
--- 
----
function LuaPlayer.toggle_map_editor() end

----
--- Unlock the achievements of the given player. This has any effect only when this is the local player, the achievement isn't unlocked so far and the achievement is of the type "achievement".
--- 
--- @param name string name of the achievement to unlock
----
function LuaPlayer.unlock_achievement(name) end

----
--- Unmutes alerts for the given alert category.
--- 
--- @param alert_type defines.alert_type 
--- @return boolean Whether the alert type was unmuted (false if it was wasn't muted).
----
function LuaPlayer.unmute_alert(alert_type) end

----
--- Uses the current item in the cursor if it's a capsule or does nothing if not.
--- 
--- @param position MapPosition Where the item would be used.
----
function LuaPlayer.use_from_cursor(position) end

----
--- Queues a request to zoom to world at the specified position. If the player is already zooming to world, the request will simply set the position, scale, and entity to follow. Render mode change requests are processed before rendering of the next frame.
--- 
--- @param position MapPosition 
--- @param scale double? 
--- @param entity LuaEntity? The entity to follow. If not given the current entity being followed will be cleared.
----
function LuaPlayer.zoom_to_world(position, scale, entity) end


----
--- @class LuaProfiler
--- An object used to measure script performance.
--- 
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaProfiler = { }

----
--- Add the duration of another timer to this timer. Useful to reduce start/stop overhead when accumulating time onto many timers at once.
--- 
--- @param other LuaProfiler The timer to add to this timer.
----
function LuaProfiler.add(other) end

----
--- Divides the current duration by a set value. Useful for calculating the average of many iterations.
--- 
--- @param number double The number to divide by. Must be > 0.
----
function LuaProfiler.divide(number) end

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaProfiler.help() end

----
--- Resets the clock, also restarting it.
--- 
----
function LuaProfiler.reset() end

----
--- Start the clock again, without resetting it.
--- 
----
function LuaProfiler.restart() end

----
--- Stops the clock.
--- 
----
function LuaProfiler.stop() end


----
--- @class LuaProgrammableSpeakerControlBehavior: LuaControlBehavior
--- Control behavior for programmable speakers.
--- 
--- @field circuit_parameters ProgrammableSpeakerCircuitParameters 
--- @field circuit_condition CircuitConditionDefinition 
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaProgrammableSpeakerControlBehavior = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaProgrammableSpeakerControlBehavior.help() end


----
--- @class LuaRCON
--- An interface to send messages to the calling RCON interface through the global object named `rcon`.
--- 
--- @field object_name string This object's name.
----
LuaRCON = { }

----
--- Print text to the calling RCON interface if any.
--- 
--- @param message LocalisedString 
----
function LuaRCON.print(message) end


----
--- @class LuaRailChainSignalControlBehavior: LuaControlBehavior
--- Control behavior for rail chain signals.
--- 
--- @field red_signal SignalID 
--- @field orange_signal SignalID 
--- @field green_signal SignalID 
--- @field blue_signal SignalID 
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaRailChainSignalControlBehavior = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaRailChainSignalControlBehavior.help() end


----
--- @class LuaRailPath
--- A rail path.
--- 
--- @field size uint The total number of rails in this path.
--- @field current uint The current rail index.
--- @field total_distance double The total path distance.
--- @field travelled_distance double The total distance travelled.
--- @field rails table<uint,LuaEntity> Array of the rails that this path travels over.
--- @field is_front boolean If the path goes from the front of the train
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaRailPath = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaRailPath.help() end


----
--- @class LuaRailSignalControlBehavior: LuaControlBehavior
--- Control behavior for rail signals.
--- 
--- @field red_signal SignalID 
--- @field orange_signal SignalID 
--- @field green_signal SignalID 
--- @field close_signal boolean If this will close the rail signal based off the circuit condition.
--- @field read_signal boolean If this will read the rail signal state.
--- @field circuit_condition CircuitConditionDefinition The circuit condition when controlling the signal through the circuit network.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaRailSignalControlBehavior = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaRailSignalControlBehavior.help() end


----
--- @class LuaRandomGenerator
--- A deterministic random generator independent from the core games random generator that can be seeded and re-seeded at will. This random generator can be saved and loaded and will maintain its state. Note this is entirely different from calling [math.random](Libraries.html#math.random)() and you should be sure you actually want to use this over calling `math.random()`. If you aren't sure if you need to use this over calling `math.random()` then you probably don't need to use this.
--- 
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaRandomGenerator = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaRandomGenerator.help() end

----
--- Re-seeds the random generator with the given value.
--- 
--- @param seed uint 
----
function LuaRandomGenerator.re_seed(seed) end


----
--- @class LuaRecipe
--- A crafting recipe. Recipes belong to forces (see [LuaForce](LuaForce)) because some recipes are unlocked by research, and researches are per-force.
--- 
--- @field name string Name of the recipe. This can be different than the name of the result items as there could be more recipes to make the same item.
--- @field localised_name LocalisedString Localised name of the recipe.
--- @field localised_description LocalisedString 
--- @field prototype LuaRecipePrototype The prototype for this recipe.
--- @field enabled boolean Can the recipe be used?
--- @field category string Category of the recipe.
--- @field ingredients Ingredient[] The ingredients to this recipe.
--- @field products Product[] The results/products of this recipe.
--- @field hidden boolean Is the recipe hidden? Hidden recipe don't show up in the crafting menu.
--- @field hidden_from_flow_stats boolean Is the recipe hidden from flow statistics?
--- @field energy double Energy required to execute this recipe. This directly affects the crafting time: Recipe's energy is exactly its crafting time in seconds, when crafted in an assembling machine with crafting speed exactly equal to one.
--- @field order string The string used to alphabetically sort these prototypes. It is a simple string that has no additional semantic meaning.
--- @field group LuaGroup Group of this recipe.
--- @field subgroup LuaGroup Subgroup of this recipe.
--- @field force LuaForce The force that owns this recipe.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaRecipe = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaRecipe.help() end

----
--- Reload the recipe from the prototype.
--- 
----
function LuaRecipe.reload() end


----
--- @class LuaRecipeCategoryPrototype
--- Prototype of a recipe category.
--- 
--- @field name string Name of this prototype.
--- @field order string The string used to alphabetically sort these prototypes. It is a simple string that has no additional semantic meaning.
--- @field localised_name LocalisedString 
--- @field localised_description LocalisedString 
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaRecipeCategoryPrototype = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaRecipeCategoryPrototype.help() end


----
--- @class LuaRecipePrototype
--- A crafting recipe prototype.
--- 
--- @field enabled boolean If this recipe prototype is enabled by default (enabled at the beginning of a game).
--- @field name string Name of the recipe. This can be different than the name of the result items as there could be more recipes to make the same item.
--- @field localised_name LocalisedString Localised name of the recipe.
--- @field localised_description LocalisedString 
--- @field category string Category of the recipe.
--- @field ingredients Ingredient[] The ingredients to this recipe.
--- @field products Product[] The results/products of this recipe.
--- @field main_product Product? The main product of this recipe, if any.
--- @field hidden boolean Is the recipe hidden? Hidden recipe don't show up in the crafting menu.
--- @field hidden_from_flow_stats boolean Is the recipe hidden from flow statistics (item/fluid production statistics)?
--- @field hidden_from_player_crafting boolean Is the recipe hidden from player crafting? The recipe will still show up for selection in machines.
--- @field always_show_made_in boolean Should this recipe always show "Made in" in the tooltip?
--- @field energy double Energy required to execute this recipe. This directly affects the crafting time: Recipe's energy is exactly its crafting time in seconds, when crafted in an assembling machine with crafting speed exactly equal to one.
--- @field order string The string used to alphabetically sort these prototypes. It is a simple string that has no additional semantic meaning.
--- @field group LuaGroup Group of this recipe.
--- @field subgroup LuaGroup Subgroup of this recipe.
--- @field request_paste_multiplier uint The multiplier used when this recipe is copied from an assembling machine to a requester chest. For each item in the recipe the item count * this value is set in the requester chest.
--- @field overload_multiplier uint Used to determine how many extra items are put into an assembling machine before it's considered "full enough".
--- @field allow_inserter_overload boolean If the recipe is allowed to have the extra inserter overload bonus applied (4 * stack inserter stack size).
--- @field allow_as_intermediate boolean If this recipe is enabled for the purpose of intermediate hand-crafting.
--- @field allow_intermediates boolean If this recipe is allowed to use intermediate recipes when hand-crafting.
--- @field show_amount_in_title boolean If the amount is shown in the recipe tooltip title when the recipe produces more than 1 product.
--- @field always_show_products boolean If the products are always shown in the recipe tooltip.
--- @field emissions_multiplier double The emissions multiplier for this recipe.
--- @field allow_decomposition boolean Is this recipe allowed to be broken down for the recipe tooltip "Total raw" calculations?
--- @field unlock_results boolean Is this recipe unlocks the result item(s) so they're shown in filter-select GUIs.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaRecipePrototype = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaRecipePrototype.help() end


----
--- @class LuaRemote
--- Registry of interfaces between scripts. An interface is simply a dictionary mapping names to functions. A script or mod can then register an interface with [LuaRemote](LuaRemote), after that any script can call the registered functions, provided it knows the interface name and the desired function name. An instance of LuaRemote is available through the global object named `remote`.
--- 
--- @field object_name string This object's name.
--- @field interfaces table<string,table<string,bool>> List of all registered interfaces. For each interface name, `remote.interfaces[name]` is a dictionary mapping the interface's registered functions to `true`.
----
LuaRemote = { }

----
--- Add a remote interface.
--- 
--- @param name string Name of the interface. If the name matches any existing interface, an error is thrown.
--- @param functions table<string,fun(): any> List of functions that are members of the new interface.
----
function LuaRemote.add_interface(name, functions) end

----
--- Call a function of an interface.
--- 
--- @param interface string Interface to look up `function` in.
--- @param function_ string Function name that belongs to the `interface`.
--- @return Any? 
----
function LuaRemote.call(interface, function_) end

----
--- Removes an interface with the given name.
--- 
--- @param name string Name of the interface.
--- @return boolean Whether the interface was removed. `False` if the interface didn't exist.
----
function LuaRemote.remove_interface(name) end


----
--- @class LuaRendering
--- Allows rendering of geometric shapes, text and sprites in the game world through the global object named `rendering`. Each render object is identified by an id that is universally unique for the lifetime of a whole game.
--- 
--- @field object_name string This object's name.
----
LuaRendering = { }

----
--- Reorder this object so that it is drawn in front of the already existing objects.
--- 
--- @param id uint64 
----
function LuaRendering.bring_to_front(id) end

----
--- Destroys all render objects. Passing an empty string (`""`)
--- 
--- @param mod_name string? If provided, only the render objects created by this mod are destroyed. An empty string (`""`) refers to all objects not belonging to a mod, such as those created using console commands.
----
function LuaRendering.clear(mod_name) end

----
--- Destroy the object with the given id.
--- 
--- @param id uint64 
----
function LuaRendering.destroy(id) end

----
--- @class LuaRendering.draw_animation_parameters
--- Parameters class for the method `LuaRendering.draw_animation`
--- 
--- @field animation string Name of an [animation prototype](https://wiki.factorio.com/Prototype/Animation).
--- @field orientation RealOrientation? The orientation of the animation. Default is 0.
--- @field x_scale double? Horizontal scale of the animation. Default is 1.
--- @field y_scale double? Vertical scale of the animation. Default is 1.
--- @field tint Color? 
--- @field render_layer RenderLayer? 
--- @field animation_speed double? How many frames the animation goes forward per tick. Default is 1.
--- @field animation_offset double? Offset of the animation in frames. Default is 0.
--- @field orientation_target MapPosition|LuaEntity? If given, the animation rotates so that it faces this target. Note that `orientation` is still applied to the animation.
--- @field orientation_target_offset Vector? Only used if `orientation_target` is a LuaEntity.
--- @field use_target_orientation boolean? Only used if `orientation_target` is a LuaEntity.
--- @field oriented_offset Vector? Offsets the center of the animation if `orientation_target` is given. This offset will rotate together with the animation.
--- @field target MapPosition|LuaEntity Center of the animation.
--- @field target_offset Vector? Only used if `target` is a LuaEntity.
--- @field surface SurfaceIdentification 
--- @field time_to_live uint? In ticks. Defaults to living forever.
--- @field forces ForceIdentification[]? The forces that this object is rendered to. Passing `nil` or an empty table will render it to all forces.
--- @field players PlayerIdentification[]? The players that this object is rendered to. Passing `nil` or an empty table will render it to all players.
--- @field visible boolean? If this is rendered to anyone at all. Defaults to true.
--- @field only_in_alt_mode boolean? If this should only be rendered in alt mode. Defaults to false.
----

----
--- Create an animation.
--- 
--- @param args LuaRendering.draw_animation_parameters Table containing arguments for the method.
--- @see LuaRendering.draw_animation_parameters
--- @return uint64 Id of the render object
----
function LuaRendering.draw_animation(args) end

----
--- @class LuaRendering.draw_arc_parameters
--- Parameters class for the method `LuaRendering.draw_arc`
--- 
--- @field color Color 
--- @field max_radius double The radius of the outer edge of the arc, in tiles.
--- @field min_radius double The radius of the inner edge of the arc, in tiles.
--- @field start_angle float Where the arc starts, in radian.
--- @field angle float The angle of the arc, in radian.
--- @field target MapPosition|LuaEntity 
--- @field target_offset Vector? Only used if `target` is a LuaEntity.
--- @field surface SurfaceIdentification 
--- @field time_to_live uint? In ticks. Defaults to living forever.
--- @field forces ForceIdentification[]? The forces that this object is rendered to. Passing `nil` or an empty table will render it to all forces.
--- @field players PlayerIdentification[]? The players that this object is rendered to. Passing `nil` or an empty table will render it to all players.
--- @field visible boolean? If this is rendered to anyone at all. Defaults to true.
--- @field draw_on_ground boolean? If this should be drawn below sprites and entities.
--- @field only_in_alt_mode boolean? If this should only be rendered in alt mode. Defaults to false.
----

----
--- Create an arc.
--- 
--- @param args LuaRendering.draw_arc_parameters Table containing arguments for the method.
--- @see LuaRendering.draw_arc_parameters
--- @return uint64 Id of the render object
----
function LuaRendering.draw_arc(args) end

----
--- @class LuaRendering.draw_circle_parameters
--- Parameters class for the method `LuaRendering.draw_circle`
--- 
--- @field color Color 
--- @field radius double In tiles.
--- @field width float? Width of the outline, used only if filled = false. Value is in pixels (32 per tile).
--- @field filled boolean If the circle should be filled.
--- @field target MapPosition|LuaEntity 
--- @field target_offset Vector? Only used if `target` is a LuaEntity.
--- @field surface SurfaceIdentification 
--- @field time_to_live uint? In ticks. Defaults to living forever.
--- @field forces ForceIdentification[]? The forces that this object is rendered to. Passing `nil` or an empty table will render it to all forces.
--- @field players PlayerIdentification[]? The players that this object is rendered to. Passing `nil` or an empty table will render it to all players.
--- @field visible boolean? If this is rendered to anyone at all. Defaults to true.
--- @field draw_on_ground boolean? If this should be drawn below sprites and entities.
--- @field only_in_alt_mode boolean? If this should only be rendered in alt mode. Defaults to false.
----

----
--- Create a circle.
--- 
--- @param args LuaRendering.draw_circle_parameters Table containing arguments for the method.
--- @see LuaRendering.draw_circle_parameters
--- @return uint64 Id of the render object
----
function LuaRendering.draw_circle(args) end

----
--- @class LuaRendering.draw_light_parameters
--- Parameters class for the method `LuaRendering.draw_light`
--- 
--- @field sprite SpritePath 
--- @field orientation RealOrientation? The orientation of the light. Default is 0.
--- @field scale float? Default is 1.
--- @field intensity float? Default is 1.
--- @field minimum_darkness float? The minimum darkness at which this light is rendered. Default is 0.
--- @field oriented boolean? If this light has the same orientation as the entity target, default is false. Note that `orientation` is still applied to the sprite.
--- @field color Color? Defaults to white (no tint).
--- @field target MapPosition|LuaEntity Center of the light.
--- @field target_offset Vector? Only used if `target` is a LuaEntity.
--- @field surface SurfaceIdentification 
--- @field time_to_live uint? In ticks. Defaults to living forever.
--- @field forces ForceIdentification[]? The forces that this object is rendered to. Passing `nil` or an empty table will render it to all forces.
--- @field players PlayerIdentification[]? The players that this object is rendered to. Passing `nil` or an empty table will render it to all players.
--- @field visible boolean? If this is rendered to anyone at all. Defaults to true.
--- @field only_in_alt_mode boolean? If this should only be rendered in alt mode. Defaults to false.
----

----
--- Create a light.
--- 
--- @param args LuaRendering.draw_light_parameters Table containing arguments for the method.
--- @see LuaRendering.draw_light_parameters
--- @return uint64 Id of the render object
----
function LuaRendering.draw_light(args) end

----
--- @class LuaRendering.draw_line_parameters
--- Parameters class for the method `LuaRendering.draw_line`
--- 
--- @field color Color 
--- @field width float In pixels (32 per tile).
--- @field gap_length double? Length of the gaps that this line has, in tiles. Default is 0.
--- @field dash_length double? Length of the dashes that this line has. Used only if gap_length > 0. Default is 0.
--- @field from MapPosition|LuaEntity 
--- @field from_offset Vector? Only used if `from` is a LuaEntity.
--- @field to MapPosition|LuaEntity 
--- @field to_offset Vector? Only used if `to` is a LuaEntity.
--- @field surface SurfaceIdentification 
--- @field time_to_live uint? In ticks. Defaults to living forever.
--- @field forces ForceIdentification[]? The forces that this object is rendered to. Passing `nil` or an empty table will render it to all forces.
--- @field players PlayerIdentification[]? The players that this object is rendered to. Passing `nil` or an empty table will render it to all players.
--- @field visible boolean? If this is rendered to anyone at all. Defaults to true.
--- @field draw_on_ground boolean? If this should be drawn below sprites and entities.
--- @field only_in_alt_mode boolean? If this should only be rendered in alt mode. Defaults to false.
----

----
--- Create a line.
--- 
--- @param args LuaRendering.draw_line_parameters Table containing arguments for the method.
--- @see LuaRendering.draw_line_parameters
--- @return uint64 Id of the render object
----
function LuaRendering.draw_line(args) end

