require('.classes-1')
require('.classes-2')
require('.classes-3')
require('.classes-4')
require('.classes-5')
require('.classes-6')
require('.classes-7')
require('.classes-8')
require('.classes-9')
require('.classes-10')

----
--- The hidden tile name.
--- 
--- @param position TilePosition The tile position.
--- @return string? `nil` if there isn't one for the given position.
----
function LuaSurface.get_hidden_tile(position) end

----
--- Gets the map exchange string for the current map generation settings of this surface.
--- 
--- @return string 
----
function LuaSurface.get_map_exchange_string() end

----
--- Get the pollution for a given position.
--- 
--- @param position MapPosition 
--- @return double 
----
function LuaSurface.get_pollution(position) end

----
--- Gets a random generated chunk position or 0,0 if no chunks have been generated on this surface.
--- 
--- @return ChunkPosition 
----
function LuaSurface.get_random_chunk() end

----
--- Gets the resource amount of all resources on this surface
--- 
--- @return table<string,uint> 
----
function LuaSurface.get_resource_counts() end

----
--- Gets the first script area by name or id.
--- 
--- @param key uint|string? The name or id of the area to get.
--- @return ScriptArea? 
----
function LuaSurface.get_script_area(key) end

----
--- Gets the script areas that match the given name or if no name is given all areas are returned.
--- 
--- @param name string? 
--- @return ScriptArea[] 
----
function LuaSurface.get_script_areas(name) end

----
--- Gets the first script position by name or id.
--- 
--- @param key uint|string? The name or id of the position to get.
--- @return ScriptPosition? 
----
function LuaSurface.get_script_position(key) end

----
--- Gets the script positions that match the given name or if no name is given all positions are returned.
--- 
--- @param name string? 
--- @return ScriptPosition[] 
----
function LuaSurface.get_script_positions(name) end

----
--- Gets the starting area radius of this surface.
--- 
--- @return double 
----
function LuaSurface.get_starting_area_radius() end

----
--- Get the tile at a given position. An alternative call signature for this method is passing it a single [TilePosition](TilePosition).
--- 
--- @param x int 
--- @param y int 
--- @return LuaTile 
----
function LuaSurface.get_tile(x, y) end

----
--- Gets the total amount of pollution on the surface by iterating over all of the chunks containing pollution.
--- 
--- @return double 
----
function LuaSurface.get_total_pollution() end

----
--- @class LuaSurface.get_train_stops_parameters
--- Parameters class for the method `LuaSurface.get_train_stops`
--- 
--- @field name string[]|string? The name(s) of the train stops. Not providing names will match any stop.
--- @field force ForceIdentification? The force to search. Not providing a force will match stops in any force.
----

----
--- Gets train stops matching the given filters.
--- 
--- @param args LuaSurface.get_train_stops_parameters? Table containing arguments for the method.
--- @see LuaSurface.get_train_stops_parameters
--- @return LuaEntity[] 
----
function LuaSurface.get_train_stops(args) end

----
--- 
--- @param force ForceIdentification? The force to search. Not providing a force will match trains in any force.
--- @return LuaTrain[] 
----
function LuaSurface.get_trains(force) end

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaSurface.help() end

----
--- Is a given chunk generated?
--- 
--- @param position ChunkPosition The chunk's position.
--- @return boolean 
----
function LuaSurface.is_chunk_generated(position) end

----
--- @class LuaSurface.play_sound_parameters
--- Parameters class for the method `LuaSurface.play_sound`
--- 
--- @field path SoundPath The sound to play.
--- @field position MapPosition? Where the sound should be played. If not given, it's played at the current position of each player.
--- @field volume_modifier double? The volume of the sound to play. Must be between 0 and 1 inclusive.
--- @field override_sound_type SoundType? The volume mixer to play the sound through. Defaults to the default mixer for the given sound type.
----

----
--- Play a sound for every player on this surface.
--- 
--- @param args LuaSurface.play_sound_parameters Table containing arguments for the method.
--- @see LuaSurface.play_sound_parameters
----
function LuaSurface.play_sound(args) end

----
--- Spawn pollution at the given position.
--- 
--- @param source MapPosition Where to spawn the pollution.
--- @param amount double How much pollution to add.
----
function LuaSurface.pollute(source, amount) end

----
--- Print text to the chat console of all players on this surface.
--- 
--- @param message LocalisedString 
--- @param color Color? 
----
function LuaSurface.print(message, color) end

----
--- Regenerate autoplacement of some decoratives on this surface. This can be used to autoplace newly-added decoratives.
--- 
--- @param decoratives string[]|string? Prototype names of decorative or decoratives to autoplace. When `nil` all decoratives with an autoplace are used.
--- @param chunks ChunkPosition[]? The chunk positions to regenerate the entities on. If not given all chunks are regenerated. Note chunks with status < entities are ignored.
----
function LuaSurface.regenerate_decorative(decoratives, chunks) end

----
--- Regenerate autoplacement of some entities on this surface. This can be used to autoplace newly-added entities.
--- 
--- @param entities string[]|string? Prototype names of entity or entities to autoplace. When `nil` all entities with an autoplace are used.
--- @param chunks ChunkPosition[]? The chunk positions to regenerate the entities on. If not given all chunks are regenerated. Note chunks with status < entities are ignored.
----
function LuaSurface.regenerate_entity(entities, chunks) end

----
--- Removes the given script area.
--- 
--- @param id uint 
--- @return boolean If the area was actually removed. False when it didn't exist.
----
function LuaSurface.remove_script_area(id) end

----
--- Removes the given script position.
--- 
--- @param id uint 
--- @return boolean If the position was actually removed. False when it didn't exist.
----
function LuaSurface.remove_script_position(id) end

----
--- @class LuaSurface.request_path_parameters
--- Parameters class for the method `LuaSurface.request_path`
--- 
--- @field bounding_box BoundingBox The dimensions of the object that's supposed to travel the path.
--- @field collision_mask CollisionMaskWithFlags|string[] The list of masks the `bounding_box` collides with.
--- @field start MapPosition The position from which to start pathfinding.
--- @field goal MapPosition The position to find a path to.
--- @field force ForceIdentification The force for which to generate the path, determining which gates can be opened for example.
--- @field radius double? How close the pathfinder needs to get to its `goal` (in tiles). Defaults to `1`.
--- @field pathfind_flags PathfinderFlags? Flags that affect pathfinder behavior.
--- @field can_open_gates boolean? Whether the path request can open gates. Defaults to `false`.
--- @field path_resolution_modifier int? Defines how coarse the pathfinder's grid is, where smaller values mean a coarser grid. Defaults to `0`, which equals a resolution of `1x1` tiles, centered on tile centers. Values range from `-8` to `8` inclusive, where each integer increment doubles/halves the resolution. So, a resolution of `-8` equals a grid of `256x256` tiles, and a resolution of `8` equals `1/256` of a tile.
--- @field entity_to_ignore LuaEntity? Makes the pathfinder ignore collisions with this entity if it is given.
----

----
--- Generates a path with the specified constraints (as an array of [PathfinderWaypoints](PathfinderWaypoint)) using the unit pathfinding algorithm. This path can be used to emulate pathing behavior by script for non-unit entities, such as vehicles. If you want to command actual units (such as biters or spitters) to move, use [LuaEntity::set_command](LuaEntity::set_command) instead.
--- 
--- The resulting path is ultimately returned asynchronously via [on_script_path_request_finished](on_script_path_request_finished).
--- 
--- @param args LuaSurface.request_path_parameters Table containing arguments for the method.
--- @see LuaSurface.request_path_parameters
--- @return uint A unique handle to identify this call when [on_script_path_request_finished](on_script_path_request_finished) fires.
----
function LuaSurface.request_path(args) end

----
--- Request that the game's map generator generate chunks at the given position for the given radius on this surface.
--- 
--- @param position MapPosition Where to generate the new chunks.
--- @param radius uint The chunk radius from `position` to generate new chunks in.
----
function LuaSurface.request_to_generate_chunks(position, radius) end

----
--- Set generated status of a chunk. Useful when copying chunks.
--- 
--- @param position ChunkPosition The chunk's position.
--- @param status defines.chunk_generated_status The chunk's new status.
----
function LuaSurface.set_chunk_generated_status(position, status) end

----
--- Set the hidden tile for the specified position. While during normal gameplay only [non-mineable](LuaTilePrototype::mineable_properties) tiles can become hidden, this method allows any kind of tile to be set as the hidden one.
--- 
--- @param position TilePosition The tile position.
--- @param tile LuaTilePrototype|string The new hidden tile or `nil` to clear the hidden tile.
----
function LuaSurface.set_hidden_tile(position, tile) end

----
--- @class LuaSurface.set_multi_command_parameters
--- Parameters class for the method `LuaSurface.set_multi_command`
--- 
--- @field command Command 
--- @field unit_count uint Number of units to give the command to.
--- @field force ForceIdentification? Force of the units this command is to be given to. If not specified, uses the enemy force.
--- @field unit_search_distance uint? Radius to search for units. The search area is centered on the destination of the command.
----

----
--- Give a command to multiple units. This will automatically select suitable units for the task.
--- 
--- @param args LuaSurface.set_multi_command_parameters Table containing arguments for the method.
--- @see LuaSurface.set_multi_command_parameters
--- @return uint Number of units actually sent. May be less than `count` if not enough units were available.
----
function LuaSurface.set_multi_command(args) end

----
--- Set tiles at specified locations. Can automatically correct the edges around modified tiles.
--- 
--- Placing a [mineable](LuaTilePrototype::mineable_properties) tile on top of a non-mineable one will turn the latter into the [LuaTile::hidden_tile](LuaTile::hidden_tile) for that tile. Placing a mineable tile on a mineable one or a non-mineable tile on a non-mineable one will not modify the hidden tile. This restriction can however be circumvented by using [LuaSurface::set_hidden_tile](LuaSurface::set_hidden_tile).
--- 
--- @param tiles Tile[] 
--- @param correct_tiles boolean? If `false`, the correction logic is not applied to the changed tiles. Defaults to `true`.
--- @param remove_colliding_entities boolean|string? `true`, `false`, or `abort_on_collision`. Defaults to `true`.
--- @param remove_colliding_decoratives boolean? `true` or `false`. Defaults to `true`.
--- @param raise_event boolean? `true` or `false`. Defaults to `false`.
----
function LuaSurface.set_tiles(tiles, correct_tiles, remove_colliding_entities, remove_colliding_decoratives, raise_event) end

----
--- Spill items on the ground centered at a given location.
--- 
--- @param position MapPosition Center of the spillage
--- @param items ItemStackIdentification Items to spill
--- @param enable_looted boolean? When true, each created item will be flagged with the [LuaEntity::to_be_looted](LuaEntity::to_be_looted) flag.
--- @param force LuaForce|string? When provided (and not `nil`) the items will be marked for deconstruction by this force.
--- @param allow_belts boolean? Whether items can be spilled onto belts. Defaults to `true`.
--- @return LuaEntity[] The created item-on-ground entities.
----
function LuaSurface.spill_item_stack(position, items, enable_looted, force, allow_belts) end

----
--- @class LuaSurface.upgrade_area_parameters
--- Parameters class for the method `LuaSurface.upgrade_area`
--- 
--- @field area BoundingBox The area to mark for upgrade.
--- @field force ForceIdentification The force whose bots should perform the upgrade.
--- @field player PlayerIdentification? The player to set the last_user to if any.
--- @field skip_fog_of_war boolean? If chunks covered by fog-of-war are skipped.
--- @field item LuaItemStack The upgrade item to use.
----

----
--- Place an upgrade request.
--- 
--- @param args LuaSurface.upgrade_area_parameters Table containing arguments for the method.
--- @see LuaSurface.upgrade_area_parameters
----
function LuaSurface.upgrade_area(args) end


----
--- @class LuaTechnology
--- One research item.
--- 
--- @field force LuaForce The force this technology belongs to.
--- @field name string Name of this technology.
--- @field localised_name LocalisedString Localised name of this technology.
--- @field localised_description LocalisedString 
--- @field prototype LuaTechnologyPrototype The prototype of this technology.
--- @field enabled boolean Can this technology be researched?
--- @field visible_when_disabled boolean If this technology will be visible in the research GUI even though it is disabled.
--- @field upgrade boolean Is this an upgrade-type research?
--- @field researched boolean Has this technology been researched? Switching from `false` to `true` will trigger the technology advancement perks; switching from `true` to `false` will reverse them.
--- @field prerequisites table<string,LuaTechnology> Prerequisites of this technology. The result maps technology name to the [LuaTechnology](LuaTechnology) object.
--- @field research_unit_ingredients Ingredient[] The types of ingredients that labs will require to research this technology.
--- @field effects TechnologyModifier[] Effects applied when this technology is researched.
--- @field research_unit_count uint The number of research units required for this technology.
--- @field research_unit_energy double Amount of energy required to finish a unit of research.
--- @field order string The string used to alphabetically sort these prototypes. It is a simple string that has no additional semantic meaning.
--- @field level uint The current level of this technology. For level-based technology writing to this is the same as researching the technology to the previous level. Writing the level will set [LuaTechnology::enabled](LuaTechnology::enabled) to `true`.
--- @field research_unit_count_formula string? The count formula, if this research has any. See the [wiki](https://wiki.factorio.com/Prototype/Technology#Technology_data) for details.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaTechnology = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaTechnology.help() end

----
--- Reload this technology from its prototype.
--- 
----
function LuaTechnology.reload() end


----
--- @class LuaTechnologyPrototype
--- A Technology prototype.
--- 
--- @field name string Name of this technology.
--- @field localised_name LocalisedString Localised name of this technology.
--- @field localised_description LocalisedString 
--- @field enabled boolean If this technology prototype is enabled by default (enabled at the beginning of a game).
--- @field hidden boolean If this technology prototype is hidden.
--- @field visible_when_disabled boolean If this technology will be visible in the research GUI even though it is disabled.
--- @field ignore_tech_cost_multiplier boolean If this technology ignores the technology cost multiplier setting.
--- @field upgrade boolean If the is technology prototype is an upgrade to some other technology.
--- @field prerequisites table<string,LuaTechnologyPrototype> Prerequisites of this technology. The result maps technology name to the [LuaTechnologyPrototype](LuaTechnologyPrototype) object.
--- @field research_unit_ingredients Ingredient[] The types of ingredients that labs will require to research this technology.
--- @field effects TechnologyModifier[] Effects applied when this technology is researched.
--- @field research_unit_count uint The number of research units required for this technology.
--- @field research_unit_energy double Amount of energy required to finish a unit of research.
--- @field order string The string used to alphabetically sort these prototypes. It is a simple string that has no additional semantic meaning.
--- @field level uint The level of this research.
--- @field max_level uint The max level of this research.
--- @field research_unit_count_formula string? The count formula, if this research has any. See the [wiki](https://wiki.factorio.com/Prototype/Technology#Technology_data) for details.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaTechnologyPrototype = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaTechnologyPrototype.help() end


----
--- @class LuaTile
--- A single "square" on the map.
--- 
--- @field name string Prototype name of this tile. E.g. `"sand-3"` or `"grass-2"`.
--- @field prototype LuaTilePrototype 
--- @field position TilePosition The position this tile references.
--- @field hidden_tile string? The name of the [LuaTilePrototype](LuaTilePrototype) hidden under this tile, if any. During normal gameplay, only [non-mineable](LuaTilePrototype::mineable_properties) tiles can become hidden. This can however be circumvented with [LuaSurface::set_hidden_tile](LuaSurface::set_hidden_tile).
--- @field surface LuaSurface The surface this tile is on.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaTile = { }

----
--- Cancels deconstruction if it is scheduled, does nothing otherwise.
--- 
--- @param force ForceIdentification The force who did the deconstruction order.
--- @param player PlayerIdentification? The player to set the last_user to if any.
----
function LuaTile.cancel_deconstruction(force, player) end

----
--- What type of things can collide with this tile?
--- 
--- @param layer CollisionMaskLayer 
--- @return boolean 
----
function LuaTile.collides_with(layer) end

----
--- Gets all tile ghosts on this tile.
--- 
--- @param force ForceIdentification? Get tile ghosts of this force.
--- @return LuaTile[] 
----
function LuaTile.get_tile_ghosts(force) end

----
--- Does this tile have any tile ghosts on it.
--- 
--- @param force ForceIdentification? Check for tile ghosts of this force.
--- @return boolean 
----
function LuaTile.has_tile_ghost(force) end

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaTile.help() end

----
--- Orders deconstruction of this tile by the given force.
--- 
--- @param force ForceIdentification The force whose robots are supposed to do the deconstruction.
--- @param player PlayerIdentification? The player to set the last_user to if any.
--- @return LuaEntity? The deconstructible tile proxy created, if any.
----
function LuaTile.order_deconstruction(force, player) end

----
--- Is this tile marked for deconstruction?
--- 
--- @param force ForceIdentification? The force who did the deconstruction order.
--- @return boolean 
----
function LuaTile.to_be_deconstructed(force) end


----
--- @class LuaTilePrototype
--- Prototype of a tile.
--- 
--- @field name string Name of this prototype.
--- @field order string The string used to alphabetically sort these prototypes. It is a simple string that has no additional semantic meaning.
--- @field localised_name LocalisedString 
--- @field localised_description LocalisedString 
--- @field collision_mask CollisionMask The collision mask this tile uses
--- @field collision_mask_with_flags CollisionMaskWithFlags 
--- @field layer uint 
--- @field autoplace_specification AutoplaceSpecification? Autoplace specification for this prototype, if any.
--- @field walking_speed_modifier float 
--- @field vehicle_friction_modifier float 
--- @field map_color Color 
--- @field decorative_removal_probability float The probability that decorative entities will be removed from on top of this tile when this tile is generated.
--- @field automatic_neighbors boolean 
--- @field allowed_neighbors table<string,LuaTilePrototype> 
--- @field needs_correction boolean If this tile needs correction logic applied when it's generated in the world..
--- @field mineable_properties { minable: boolean, mining_particle: string, mining_time: double, products: Product[] } 
--- @field next_direction LuaTilePrototype? The next direction of this tile, if any. Used when a tile has multiple directions (such as hazard concrete)
--- @field items_to_place_this ItemStackDefinition[]? Items that when placed will produce this tile, if any. Construction bots will choose the first item in the list to build this tile.
--- @field can_be_part_of_blueprint boolean False if this tile is not allowed in blueprints regardless of the ability to build it.
--- @field emissions_per_second double Amount of pollution emissions per second this tile will absorb.
--- @field check_collision_with_entities boolean True if building this tile should check for colliding entities above and prevent building if such are found. Also during mining tiles above this tile checks for entities colliding with this tile and prevents mining if such are found.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaTilePrototype = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaTilePrototype.help() end


----
--- @class LuaTrain
--- A train. Trains are a sequence of connected rolling stocks -- locomotives and wagons.
--- 
--- @field manual_mode boolean When `true`, the train is explicitly controlled by the player or script. When `false`, the train moves autonomously according to its schedule.
--- @field speed double Current speed.
--- @field max_forward_speed double Current max speed when moving forward, depends on locomotive prototype and fuel.
--- @field max_backward_speed double Current max speed when moving backwards, depends on locomotive prototype and fuel.
--- @field weight double The weight of this train.
--- @field carriages LuaEntity[] The rolling stocks this train is composed of, with the numbering starting at the [front](LuaTrain::front_stock) of the train.
--- @field locomotives table<string,LuaEntity[]> Arrays of locomotives. The result is two arrays, indexed by `"front_movers"` and `"back_movers"` containing the locomotives. E.g. `{front_movers={loco1, loco2}, back_movers={loco3}}`.
--- @field cargo_wagons LuaEntity[] The cargo carriages the train contains.
--- @field fluid_wagons LuaEntity[] The fluid carriages the train contains.
--- @field schedule TrainSchedule? This train's current schedule, if any. Set to `nil` to clear.
--- @field state defines.train_state This train's current state.
--- @field front_rail LuaEntity? The rail at the front end of the train, if any.
--- @field back_rail LuaEntity? The rail at the back end of the train, if any.
--- @field rail_direction_from_front_rail defines.rail_direction 
--- @field rail_direction_from_back_rail defines.rail_direction 
--- @field front_stock LuaEntity? The front stock of this train, if any. The front of the train is in the direction that a majority of locomotives are pointing in. If it's a tie, the North and West directions take precedence.
--- @field back_stock LuaEntity? The back stock of this train, if any. The back of the train is at the opposite end of the [front](LuaTrain::front_stock).
--- @field station LuaEntity? The train stop this train is stopped at, if any.
--- @field has_path boolean If this train has a path.
--- @field path_end_rail LuaEntity? The destination rail this train is currently pathing to, if any.
--- @field path_end_stop LuaEntity? The destination train stop this train is currently pathing to, if any.
--- @field id uint The unique train ID.
--- @field passengers LuaPlayer[] The player passengers on the train
--- @field riding_state RidingState The riding state of this train.
--- @field killed_players table<uint,uint> The players killed by this train.  The keys are the player indices, the values are how often this train killed that player.
--- @field kill_count uint The total number of kills by this train.
--- @field path LuaRailPath? The path this train is using, if any.
--- @field signal LuaEntity? The signal this train is arriving or waiting at, if any.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaTrain = { }

----
--- Clears all fluids in this train.
--- 
----
function LuaTrain.clear_fluids_inside() end

----
--- Clear all items in this train.
--- 
----
function LuaTrain.clear_items_inside() end

----
--- Get a mapping of the train's inventory.
--- 
--- @return table<string,uint> The counts, indexed by item names.
----
function LuaTrain.get_contents() end

----
--- Gets a mapping of the train's fluid inventory.
--- 
--- @return table<string,double> The counts, indexed by fluid names.
----
function LuaTrain.get_fluid_contents() end

----
--- Get the amount of a particular fluid stored in the train.
--- 
--- @param fluid string? Fluid name to count. If not given, counts all fluids.
--- @return double 
----
function LuaTrain.get_fluid_count(fluid) end

----
--- Get the amount of a particular item stored in the train.
--- 
--- @param item string? Item name to count. If not given, counts all items.
--- @return uint 
----
function LuaTrain.get_item_count(item) end

----
--- Gets all rails under the train.
--- 
--- @return LuaEntity[] 
----
function LuaTrain.get_rails() end

----
--- Go to the station specified by the index in the train's schedule.
--- 
--- @param index uint 
----
function LuaTrain.go_to_station(index) end

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaTrain.help() end

----
--- Insert a stack into the train.
--- 
--- @param stack ItemStackIdentification 
----
function LuaTrain.insert(stack) end

----
--- Inserts the given fluid into the first available location in this train.
--- 
--- @param fluid Fluid 
--- @return double The amount inserted.
----
function LuaTrain.insert_fluid(fluid) end

----
--- Checks if the path is invalid and tries to re-path if it isn't.
--- 
--- @param force boolean? Forces the train to re-path regardless of the current path being valid or not.
--- @return boolean If the train has a path after the repath attempt.
----
function LuaTrain.recalculate_path(force) end

----
--- Remove some fluid from the train.
--- 
--- @param fluid Fluid 
--- @return double The amount of fluid actually removed.
----
function LuaTrain.remove_fluid(fluid) end

----
--- Remove some items from the train.
--- 
--- @param stack ItemStackIdentification The amount and type of items to remove
--- @return uint Number of items actually removed.
----
function LuaTrain.remove_item(stack) end


----
--- @class LuaTrainStopControlBehavior: LuaGenericOnOffControlBehavior
--- Control behavior for train stops.
--- 
--- @field send_to_train boolean `true` if the train stop should send the circuit network contents to the train to use.
--- @field read_from_train boolean `true` if the train stop should send the train contents to the circuit network.
--- @field read_stopped_train boolean `true` if the train stop should send the stopped train id to the circuit network.
--- @field set_trains_limit boolean `true` if the trains_limit_signal is used to set a limit of trains incoming for train stop.
--- @field read_trains_count boolean `true` if the train stop should send amount of incoming trains to the circuit network.
--- @field enable_disable boolean `true` if the train stop is enabled/disabled through the circuit network.
--- @field stopped_train_signal SignalID The signal that will be sent when using the send-train-id option.
--- @field trains_count_signal SignalID The signal that will be sent when using the read-trains-count option.
--- @field trains_limit_signal SignalID The signal to be used by set-trains-limit to limit amount of incoming trains
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaTrainStopControlBehavior = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaTrainStopControlBehavior.help() end


----
--- @class LuaTransportBeltControlBehavior: LuaGenericOnOffControlBehavior
--- Control behavior for transport belts.
--- 
--- @field enable_disable boolean If the belt will be enabled/disabled based off the circuit network.
--- @field read_contents boolean If the belt will read the contents and send them to the circuit network.
--- @field read_contents_mode defines.control_behavior.transport_belt.content_read_mode The read mode for the belt.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaTransportBeltControlBehavior = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaTransportBeltControlBehavior.help() end


----
--- @class LuaTransportLine
--- One line on a transport belt.
--- 
--- @field owner LuaEntity The entity this transport line belongs to.
--- @field output_lines LuaTransportLine[] The transport lines that this transport line outputs items to or an empty table if none.
--- @field input_lines LuaTransportLine[] The transport lines that this transport line is fed by or an empty table if none.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaTransportLine = { }

----
--- Can an item be inserted at a given position?
--- 
--- @param position float Where to insert an item.
--- @return boolean 
----
function LuaTransportLine.can_insert_at(position) end

----
--- Can an item be inserted at the back of this line?
--- 
--- @return boolean 
----
function LuaTransportLine.can_insert_at_back() end

----
--- Remove all items from this transport line.
--- 
----
function LuaTransportLine.clear() end

----
--- Get counts of all items on this line, similar to how [LuaInventory::get_contents](LuaInventory::get_contents) does.
--- 
--- @return table<string,uint> The counts, indexed by item names.
----
function LuaTransportLine.get_contents() end

----
--- Count some or all items on this line, similar to how [LuaInventory::get_item_count](LuaInventory::get_item_count) does.
--- 
--- @param item string? Prototype name of the item to count. If not specified, count all items.
--- @return uint 
----
function LuaTransportLine.get_item_count(item) end

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaTransportLine.help() end

----
--- Insert items at a given position.
--- 
--- @param position float Where on the line to insert the items.
--- @param items ItemStackIdentification Items to insert.
--- @return boolean Were the items inserted successfully?
----
function LuaTransportLine.insert_at(position, items) end

----
--- Insert items at the back of this line.
--- 
--- @param items ItemStackIdentification 
--- @return boolean Were the items inserted successfully?
----
function LuaTransportLine.insert_at_back(items) end

----
--- Returns whether the associated internal transport line of this line is the same as the others associated internal transport line.
--- 
--- @param other LuaTransportLine 
--- @return boolean 
----
function LuaTransportLine.line_equals(other) end

----
--- Remove some items from this line.
--- 
--- @param items ItemStackIdentification Items to remove.
--- @return uint Number of items actually removed.
----
function LuaTransportLine.remove_item(items) end


----
--- @class LuaTrivialSmokePrototype
--- Prototype of a trivial smoke.
--- 
--- @field name string Name of this prototype.
--- @field order string The string used to alphabetically sort these prototypes. It is a simple string that has no additional semantic meaning.
--- @field localised_name LocalisedString 
--- @field localised_description LocalisedString 
--- @field color Color 
--- @field start_scale double 
--- @field end_scale double 
--- @field movement_slow_down_factor double 
--- @field duration uint 
--- @field spread_duration uint 
--- @field fade_away_duration uint 
--- @field fade_in_duration uint 
--- @field glow_fade_away_duration uint 
--- @field cyclic boolean 
--- @field affected_by_wind boolean 
--- @field show_when_smoke_off boolean 
--- @field glow_animation boolean 
--- @field render_layer RenderLayer 
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaTrivialSmokePrototype = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaTrivialSmokePrototype.help() end


----
--- @class LuaUnitGroup
--- A collection of units moving and attacking together. The engine creates autonomous unit groups to attack polluted areas. The script can create and control such groups as well. Groups can accept commands in the same manner as regular units.
--- 
--- @field members LuaEntity[] Members of this group.
--- @field position MapPosition Group position. This can have different meanings depending on the group state. When the group is gathering, the position is the place of gathering. When the group is moving, the position is the expected position of its members along the path. When the group is attacking, it is the average position of its members.
--- @field state defines.group_state Whether this group is gathering, moving or attacking.
--- @field force LuaForce The force of this unit group.
--- @field surface LuaSurface The surface of this unit group.
--- @field group_number uint The group number for this unit group.
--- @field is_script_driven boolean Whether this unit group is controlled by a script or by the game engine. This can be changed using [LuaUnitGroup::set_autonomous](LuaUnitGroup::set_autonomous).
--- @field command Command? The command given to this group, if any.
--- @field distraction_command Command? The distraction command given to this group, if any.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaUnitGroup = { }

----
--- Make a unit a member of this group. Has the same effect as giving a `group_command` with this group to the unit.
--- 
--- @param unit LuaEntity 
----
function LuaUnitGroup.add_member(unit) end

----
--- Dissolve this group. Its members won't be destroyed, they will be merely unlinked from this group.
--- 
----
function LuaUnitGroup.destroy() end

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaUnitGroup.help() end

----
--- Make this group autonomous. Autonomous groups will automatically attack polluted areas. Autonomous groups aren't considered to be [script-driven](LuaUnitGroup::is_script_driven).
--- 
----
function LuaUnitGroup.set_autonomous() end

----
--- Give this group a command.
--- 
--- @param command Command 
----
function LuaUnitGroup.set_command(command) end

----
--- Give this group a distraction command.
--- 
--- @param command Command 
----
function LuaUnitGroup.set_distraction_command(command) end

----
--- Make the group start moving even if some of its members haven't yet arrived.
--- 
----
function LuaUnitGroup.start_moving() end


----
--- @class LuaVirtualSignalPrototype
--- Prototype of a virtual signal.
--- 
--- @field name string Name of this prototype.
--- @field order string The string used to alphabetically sort these prototypes. It is a simple string that has no additional semantic meaning.
--- @field localised_name LocalisedString 
--- @field localised_description LocalisedString 
--- @field special boolean Whether this is a special signal. The `everything`, `anything`, `each`, and `unknown` signals are considered special.
--- @field subgroup LuaGroup 
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaVirtualSignalPrototype = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaVirtualSignalPrototype.help() end


----
--- @class LuaVoidEnergySourcePrototype
--- Prototype of a void energy source.
--- 
--- @field emissions double The emissions of this energy source in `pollution/Joule`. Multiplying it by energy consumption in `Watt` gives `pollution/second`.
--- @field render_no_network_icon boolean 
--- @field render_no_power_icon boolean 
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaVoidEnergySourcePrototype = { }

