require('.classes-1')
require('.classes-2')

----
--- Get the rail signal or train stop at the start/end of the rail segment this rail is in.
--- 
--- @param direction defines.rail_direction The direction of travel relative to this rail.
--- @param in_else_out boolean If true, gets the entity at the entrance of the rail segment, otherwise gets the entity at the exit of the rail segment.
--- @return LuaEntity? `nil` if the rail segment doesn't start/end with a signal nor a train stop.
----
function LuaEntity.get_rail_segment_entity(direction, in_else_out) end

----
--- Get the length of the rail segment this rail is in.
--- 
--- @return double 
----
function LuaEntity.get_rail_segment_length() end

----
--- Get a rail from each rail segment that overlaps with this rail's rail segment.
--- 
--- @return LuaEntity[] 
----
function LuaEntity.get_rail_segment_overlaps() end

----
--- Get all rails of a rail segment this rail is in
--- 
--- @param direction defines.rail_direction Selects end of this rail that points to a rail segment end from which to start returning rails
--- @return LuaEntity[] Rails of this rail segment
----
function LuaEntity.get_rail_segment_rails(direction) end

----
--- Current recipe being assembled by this machine, if any.
--- 
--- @return LuaRecipe? 
----
function LuaEntity.get_recipe() end

----
--- Get a logistic requester slot.
--- 
--- @param slot uint The slot index.
--- @return SimpleItemStack? Contents of the specified slot; `nil` if the given slot contains no request.
----
function LuaEntity.get_request_slot(slot) end

----
--- Gets legs of given SpiderVehicle.
--- 
--- @return LuaEntity[] 
----
function LuaEntity.get_spider_legs() end

----
--- The train currently stopped at this train stop, if any.
--- 
--- @return LuaTrain? 
----
function LuaEntity.get_stopped_train() end

----
--- The trains scheduled to stop at this train stop.
--- 
--- @return LuaTrain[] 
----
function LuaEntity.get_train_stop_trains() end

----
--- Get a transport line of a belt or belt connectable entity.
--- 
--- @param index uint Index of the requested transport line. Transport lines are 1-indexed.
--- @return LuaTransportLine 
----
function LuaEntity.get_transport_line(index) end

----
--- Returns the new entity direction after upgrading.
--- 
--- @return defines.direction? `nil` if this entity is not marked for upgrade.
----
function LuaEntity.get_upgrade_direction() end

----
--- Returns the new entity prototype.
--- 
--- @return LuaEntityPrototype? `nil` if this entity is not marked for upgrade.
----
function LuaEntity.get_upgrade_target() end

----
--- Same as [LuaEntity::has_flag](LuaEntity::has_flag), but targets the inner entity on a entity ghost.
--- 
--- @param flag string The flag to test. See [EntityPrototypeFlags](EntityPrototypeFlags) for a list of flags.
--- @return boolean `true` if the entity has the given flag set.
----
function LuaEntity.ghost_has_flag(flag) end

----
--- Has this unit been assigned a command?
--- 
--- @return boolean 
----
function LuaEntity.has_command() end

----
--- Test whether this entity's prototype has a certain flag set.
--- 
--- @param flag string The flag to test. See [EntityPrototypeFlags](EntityPrototypeFlags) for a list of flags.
--- @return boolean `true` if this entity has the given flag set.
----
function LuaEntity.has_flag(flag) end

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaEntity.help() end

----
--- Insert fluid into this entity. Fluidbox is chosen automatically.
--- 
--- @param fluid Fluid Fluid to insert.
--- @return double Amount of fluid actually inserted.
----
function LuaEntity.insert_fluid(fluid) end

----
--- 
--- @return boolean `true` if this gate is currently closed.
----
function LuaEntity.is_closed() end

----
--- 
--- @return boolean `true` if this gate is currently closing
----
function LuaEntity.is_closing() end

----
--- Returns `true` if this entity produces or consumes electricity and is connected to an electric network that has at least one entity that can produce power.
--- 
--- @return boolean 
----
function LuaEntity.is_connected_to_electric_network() end

----
--- Returns whether a craft is currently in process. It does not indicate whether progress is currently being made, but whether a crafting process has been started in this machine.
--- 
--- @return boolean 
----
function LuaEntity.is_crafting() end

----
--- 
--- @return boolean `true` if this gate is currently opened.
----
function LuaEntity.is_opened() end

----
--- 
--- @return boolean `true` if this gate is currently opening.
----
function LuaEntity.is_opening() end

----
--- Checks if this rail and other rail both belong to the same rail block.
--- 
--- @param other_rail LuaEntity 
--- @return boolean 
----
function LuaEntity.is_rail_in_same_rail_block_as(other_rail) end

----
--- Checks if this rail and other rail both belong to the same rail segment.
--- 
--- @param other_rail LuaEntity 
--- @return boolean 
----
function LuaEntity.is_rail_in_same_rail_segment_as(other_rail) end

----
--- Is this entity or tile ghost or item request proxy registered for construction? If false, it means a construction robot has been dispatched to build the entity, or it is not an entity that can be constructed.
--- 
--- @return boolean 
----
function LuaEntity.is_registered_for_construction() end

----
--- Is this entity registered for deconstruction with this force? If false, it means a construction robot has been dispatched to deconstruct it, or it is not marked for deconstruction. The complexity is effectively O(1) - it depends on the number of objects targeting this entity which should be small enough.
--- 
--- @param force ForceIdentification The force construction manager to check.
--- @return boolean 
----
function LuaEntity.is_registered_for_deconstruction(force) end

----
--- Is this entity registered for repair? If false, it means a construction robot has been dispatched to upgrade it, or it is not damaged. This is worst-case O(N) complexity where N is the current number of things in the repair queue.
--- 
--- @return boolean 
----
function LuaEntity.is_registered_for_repair() end

----
--- Is this entity registered for upgrade? If false, it means a construction robot has been dispatched to upgrade it, or it is not marked for upgrade. This is worst-case O(N) complexity where N is the current number of things in the upgrade queue.
--- 
--- @return boolean 
----
function LuaEntity.is_registered_for_upgrade() end

----
--- 
--- @return boolean `true` if the rocket was successfully launched. Return value of `false` means the silo is not ready for launch.
----
function LuaEntity.launch_rocket() end

----
--- @class LuaEntity.mine_parameters
--- Parameters class for the method `LuaEntity.mine`
--- 
--- @field inventory LuaInventory? If provided the item(s) will be transferred into this inventory. If provided, this must be an inventory created with [LuaGameScript::create_inventory](LuaGameScript::create_inventory) or be a basic inventory owned by some entity.
--- @field force boolean? If true, when the item(s) don't fit into the given inventory the entity is force mined. If false, the mining operation fails when there isn't enough room to transfer all of the items into the inventory. Defaults to false. This is ignored and acts as `true` if no inventory is provided.
--- @field raise_destroyed boolean? If true, [script_raised_destroy](script_raised_destroy) will be raised. Defaults to `true`.
--- @field ignore_minable boolean? If true, the minable state of the entity is ignored. Defaults to `false`. If false, an entity that isn't minable (set as not-minable in the prototype or isn't minable for other reasons) will fail to be mined.
----

----
--- Mines this entity.
--- 
--- @param args LuaEntity.mine_parameters? Table containing arguments for the method.
--- @see LuaEntity.mine_parameters
--- @return boolean Whether mining succeeded.
----
function LuaEntity.mine(args) end

----
--- Sets the entity to be deconstructed by construction robots.
--- 
--- @param force ForceIdentification The force whose robots are supposed to do the deconstruction.
--- @param player PlayerIdentification? The player to set the `last_user` to if any.
--- @return boolean if the entity was marked for deconstruction.
----
function LuaEntity.order_deconstruction(force, player) end

----
--- @class LuaEntity.order_upgrade_parameters
--- Parameters class for the method `LuaEntity.order_upgrade`
--- 
--- @field force ForceIdentification The force whose robots are supposed to do the upgrade.
--- @field target EntityPrototypeIdentification The prototype of the entity to upgrade to.
--- @field player PlayerIdentification? 
--- @field direction defines.direction? The new direction if any.
----

----
--- Sets the entity to be upgraded by construction robots.
--- 
--- @param args LuaEntity.order_upgrade_parameters Table containing arguments for the method.
--- @see LuaEntity.order_upgrade_parameters
--- @return boolean Whether the entity was marked for upgrade.
----
function LuaEntity.order_upgrade(args) end

----
--- Plays a note with the given instrument and note.
--- 
--- @param instrument uint 
--- @param note uint 
--- @return boolean Whether the request is valid. The sound may or may not be played depending on polyphony settings.
----
function LuaEntity.play_note(instrument, note) end

----
--- Release the unit from the spawner which spawned it. This allows the spawner to continue spawning additional units.
--- 
----
function LuaEntity.release_from_spawner() end

----
--- @class LuaEntity.remove_fluid_parameters
--- Parameters class for the method `LuaEntity.remove_fluid`
--- 
--- @field name string Fluid prototype name.
--- @field amount double Amount to remove
--- @field minimum_temperature double? 
--- @field maximum_temperature double? 
--- @field temperature double? 
----

----
--- Remove fluid from this entity.
--- 
--- @param args LuaEntity.remove_fluid_parameters Table containing arguments for the method.
--- @see LuaEntity.remove_fluid_parameters
--- @return double Amount of fluid actually removed.
----
function LuaEntity.remove_fluid(args) end

----
--- Remove an offer from a market.
--- 
--- @param offer uint Index of offer to remove.
--- @return boolean `true` if the offer was successfully removed; `false` when the given index was not valid.
----
function LuaEntity.remove_market_item(offer) end

----
--- 
--- @param force ForceIdentification The force that requests the gate to be closed.
----
function LuaEntity.request_to_close(force) end

----
--- 
--- @param force ForceIdentification The force that requests the gate to be open.
--- @param extra_time uint? Extra ticks to stay open.
----
function LuaEntity.request_to_open(force, extra_time) end

----
--- @class LuaEntity.revive_parameters
--- Parameters class for the method `LuaEntity.revive`
--- 
--- @field return_item_request_proxy boolean? If `true` the function will return item request proxy as the third return value.
--- @field raise_revive boolean? If true, and an entity ghost; [script_raised_revive](script_raised_revive) will be called. Else if true, and a tile ghost; [script_raised_set_tiles](script_raised_set_tiles) will be called.
----

----
--- Revive a ghost. I.e. turn it from a ghost to a real entity or tile.
--- 
--- @param args LuaEntity.revive_parameters? Table containing arguments for the method.
--- @see LuaEntity.revive_parameters
--- @return table<string,uint>? Any items the new real entity collided with or `nil` if the ghost could not be revived.
--- @return LuaEntity? The revived entity if an entity ghost was successfully revived.
--- @return LuaEntity? The item request proxy if it was requested with `return_item_request_proxy`.
----
function LuaEntity.revive(args) end

----
--- @class LuaEntity.rotate_parameters
--- Parameters class for the method `LuaEntity.rotate`
--- 
--- @field reverse boolean? If `true`, rotate the entity in the counter-clockwise direction.
--- @field by_player PlayerIdentification? If not specified, the [on_player_rotated_entity](on_player_rotated_entity) event will not be fired.
--- @field spill_items boolean? If the player is not given should extra items be spilled or returned as a second return value from this.
--- @field enable_looted boolean? When true, each spilled item will be flagged with the [LuaEntity::to_be_looted](LuaEntity::to_be_looted) flag.
--- @field force LuaForce|string? When provided the spilled items will be marked for deconstruction by this force.
----

----
--- Rotates this entity as if the player rotated it.
--- 
--- @param args LuaEntity.rotate_parameters? Table containing arguments for the method.
--- @see LuaEntity.rotate_parameters
--- @return boolean Whether the rotation was successful.
--- @return table<string,uint>? Count of spilled items indexed by their prototype names if `spill_items` was `true`.
----
function LuaEntity.rotate(args) end

----
--- Set the source of this beam.
--- 
--- @param source LuaEntity|MapPosition 
----
function LuaEntity.set_beam_source(source) end

----
--- Set the target of this beam.
--- 
--- @param target LuaEntity|MapPosition 
----
function LuaEntity.set_beam_target(target) end

----
--- Give the entity a command.
--- 
--- @param command Command 
----
function LuaEntity.set_command(command) end

----
--- Give the entity a distraction command.
--- 
--- @param command Command 
----
function LuaEntity.set_distraction_command(command) end

----
--- Sets the driver of this vehicle.
--- 
--- @param driver LuaEntity|PlayerIdentification? The new driver. Writing `nil` ejects the current driver, if any.
----
function LuaEntity.set_driver(driver) end

----
--- Set the filter for a slot in an inserter, loader, or logistic storage container.
--- 
--- @param slot_index uint Index of the slot to set the filter for.
--- @param item nil|string Prototype name of the item to filter, or `nil` to clear the filter.
----
function LuaEntity.set_filter(slot_index, item) end

----
--- Sets the heat setting for this heat interface.
--- 
--- @param filter HeatSetting The new setting.
----
function LuaEntity.set_heat_setting(filter) end

----
--- Sets the filter for this infinity container at the given index.
--- 
--- @param index uint The index to set.
--- @param filter InfinityInventoryFilter|nil The new filter, or `nil` to clear the filter.
----
function LuaEntity.set_infinity_container_filter(index, filter) end

----
--- Sets the filter for this infinity pipe.
--- 
--- @param filter InfinityPipeFilter|nil The new filter, or `nil` to clear the filter.
----
function LuaEntity.set_infinity_pipe_filter(filter) end

----
--- Sets the passenger of this car or spidertron.
--- 
--- @param passenger LuaEntity|PlayerIdentification? The new passenger. Writing `nil` ejects the current passenger, if any.
----
function LuaEntity.set_passenger(passenger) end

----
--- Sets the given recipe in this assembly machine.
--- 
--- @param recipe LuaRecipe|string? The new recipe. Writing `nil` clears the recipe, if any.
--- @return table<string,uint> Any items removed from this entity as a result of setting the recipe.
----
function LuaEntity.set_recipe(recipe) end

----
--- Set a logistic requester slot.
--- 
--- @param request ItemStackIdentification What to request.
--- @param slot uint The slot index.
--- @return boolean Whether the slot was set.
----
function LuaEntity.set_request_slot(request, slot) end

----
--- @class LuaEntity.silent_revive_parameters
--- Parameters class for the method `LuaEntity.silent_revive`
--- 
--- @field return_item_request_proxy boolean? If `true` the function will return item request proxy as the third parameter.
--- @field raise_revive boolean? If true, and an entity ghost; [script_raised_revive](script_raised_revive) will be called. Else if true, and a tile ghost; [script_raised_set_tiles](script_raised_set_tiles) will be called.
----

----
--- Revives a ghost silently.
--- 
--- @param args LuaEntity.silent_revive_parameters? Table containing arguments for the method.
--- @see LuaEntity.silent_revive_parameters
--- @return table<string,uint>? Any items the new real entity collided with or `nil` if the ghost could not be revived.
--- @return LuaEntity? The revived entity if an entity ghost was successfully revived.
--- @return LuaEntity? The item request proxy if it was requested with `return_item_request_proxy`.
----
function LuaEntity.silent_revive(args) end

----
--- Triggers spawn_decoration actions defined in the entity prototype or does nothing if entity is not "turret" or "unit-spawner".
--- 
----
function LuaEntity.spawn_decorations() end

----
--- Only works if the entity is a speech-bubble, with an "effect" defined in its wrapper_flow_style. Starts animating the opacity of the speech bubble towards zero, and destroys the entity when it hits zero.
--- 
----
function LuaEntity.start_fading_out() end

----
--- Sets the [speed](LuaEntity::speed) of the given SpiderVehicle to zero. Notably does not clear its [autopilot_destination](LuaEntity::autopilot_destination), which it will continue moving towards if set.
--- 
----
function LuaEntity.stop_spider() end

----
--- Whether this entity supports a backer name.
--- 
--- @return boolean 
----
function LuaEntity.supports_backer_name() end

----
--- Is this entity marked for deconstruction?
--- 
--- @return boolean 
----
function LuaEntity.to_be_deconstructed() end

----
--- Is this entity marked for upgrade?
--- 
--- @return boolean 
----
function LuaEntity.to_be_upgraded() end

----
--- Toggle this entity's equipment movement bonus. Does nothing if the entity does not have an equipment grid.
--- 
----
function LuaEntity.toggle_equipment_movement_bonus() end

----
--- Reconnect loader, beacon, cliff and mining drill connections to entities that might have been teleported out or in by the script. The game doesn't do this automatically as we don't want to loose performance by checking this in normal games.
--- 
----
function LuaEntity.update_connections() end


----
--- @class LuaEntityPrototype
--- Prototype of an entity.
--- 
--- @field type string Type of this prototype.
--- @field name string Name of this prototype.
--- @field localised_name LocalisedString 
--- @field localised_description LocalisedString 
--- @field max_health float Max health of this entity. Will be `0` if this is not an entity with health.
--- @field infinite_resource boolean? Whether this resource is infinite.
--- @field minimum_resource_amount uint? Minimum amount of this resource.
--- @field normal_resource_amount uint? The normal amount for this resource.
--- @field infinite_depletion_resource_amount uint? Every time this infinite resource 'ticks' down, it is reduced by this amount. Meaningless if this isn't an infinite resource.
--- @field resource_category string? Name of the category of this resource.
--- @field mineable_properties { fluid_amount: double, minable: boolean, mining_particle: string, mining_time: double, mining_trigger: TriggerItem[], products: Product[], required_fluid: string } Whether this entity is minable and what can be obtained by mining it.
--- @field items_to_place_this ItemStackDefinition[]? Items that when placed will produce this entity, if any. Construction bots will choose the first item in the list to build this entity.
--- @field collision_box BoundingBox The bounding box used for collision checking.
--- @field secondary_collision_box BoundingBox? The secondary bounding box used for collision checking, if any. This is only used in rails and rail remnants.
--- @field map_generator_bounding_box BoundingBox The bounding box used for map generator collision checking.
--- @field selection_box BoundingBox The bounding box used for drawing selection.
--- @field drawing_box BoundingBox The bounding box used for drawing the entity icon.
--- @field sticker_box BoundingBox The bounding box used to attach sticker type entities.
--- @field collision_mask CollisionMask The collision masks this entity uses
--- @field collision_mask_with_flags CollisionMaskWithFlags 
--- @field default_collision_mask_with_flags CollisionMaskWithFlags The hardcoded default collision mask (with flags) for this entity prototype type.
--- @field order string The string used to alphabetically sort these prototypes. It is a simple string that has no additional semantic meaning.
--- @field group LuaGroup Group of this entity.
--- @field subgroup LuaGroup Subgroup of this entity.
--- @field healing_per_tick float? Amount this entity can heal per tick, if any.
--- @field emissions_per_second double Amount of pollution emissions per second this entity will create.
--- @field corpses table<string,LuaEntityPrototype>? Corpses used when this entity is destroyed. It is a dictionary indexed by the corpse's prototype name.
--- @field selectable_in_game boolean Is this entity selectable?
--- @field selection_priority uint The selection priority of this entity - a value between 0 and 255
--- @field weight double? The weight of this vehicle prototype.
--- @field resistances table<string,Resistance>? List of resistances towards each damage type. It is a dictionary indexed by damage type names (see `data/base/prototypes/damage-type.lua`).
--- @field fast_replaceable_group string? The group of mutually fast-replaceable entities, if any.
--- @field next_upgrade LuaEntityPrototype? The next upgrade for this entity, if any.
--- @field loot Loot[]? Loot that will be dropped when this entity is killed, if any.
--- @field repair_speed_modifier uint? Repair-speed modifier for this entity, if any. Actual repair speed will be `tool_repair_speed * entity_repair_speed_modifier`.
--- @field turret_range uint? The range of this turret.
--- @field autoplace_specification AutoplaceSpecification? Autoplace specification for this entity prototype, if any.
--- @field belt_speed double? The speed of this transport belt.
--- @field result_units UnitSpawnDefinition[]? The result units and spawn points with weight and evolution factor for a biter spawner entity.
--- @field attack_result TriggerItem[]? The attack result of this entity, if any.
--- @field final_attack_result TriggerItem[]? The final attack result for this projectile.
--- @field attack_parameters AttackParameters? The attack parameters for this entity, if any.
--- @field spawn_cooldown { max: double, min: double }? The spawning cooldown for this enemy spawner prototype.
--- @field mining_drill_radius double? The mining radius of this mining drill prototype.
--- @field mining_speed double? The mining speed of this mining drill/character prototype.
--- @field logistic_mode string? The logistic mode of this logistic container. One of `"requester"`, `"active-provider"`, `"passive-provider"`, `"buffer"`, `"storage"`, `"none"`.
--- @field max_underground_distance uint8? The max underground distance for underground belts and underground pipes.
--- @field flags EntityPrototypeFlags The flags for this entity prototype.
--- @field remains_when_mined LuaEntityPrototype[] The remains left behind when this entity is mined.
--- @field additional_pastable_entities LuaEntityPrototype[] Entities this entity can be pasted onto in addition to the normal allowed ones.
--- @field allow_copy_paste boolean When false copy-paste is not allowed for this entity.
--- @field shooting_cursor_size double The cursor size used when shooting at this entity.
--- @field created_smoke { initial_height: float, max_radius: float, offset_deviation: BoundingBox, offsets: Vector[], smoke_name: string, speed: Vector, speed_from_center: float, speed_from_center_deviation: float, speed_multiplier: float, speed_multiplier_deviation: float, starting_frame: float, starting_frame_deviation: float, starting_frame_speed: float, starting_frame_speed_deviation: float }? The smoke trigger run when this entity is built, if any.
--- @field created_effect TriggerItem[]? The trigger to run when this entity is created, if any.
--- @field map_color Color? The map color used when charting this entity if a friendly or enemy color isn't defined, if any.
--- @field friendly_map_color Color The friendly map color used when charting this entity.
--- @field enemy_map_color Color The enemy map color used when charting this entity.
--- @field build_base_evolution_requirement double The evolution requirement to build this entity as a base when expanding enemy bases.
--- @field instruments ProgrammableSpeakerInstrument[]? The instruments for this programmable speaker.
--- @field max_polyphony uint? The maximum polyphony for this programmable speaker.
--- @field module_inventory_size uint? The module inventory size. `nil` if this entity doesn't support modules.
--- @field ingredient_count uint? The max number of ingredients this crafting machine prototype supports.
--- @field crafting_speed double? The crafting speed..
--- @field crafting_categories table<string,boolean>? The crafting categories this entity prototype supports.
--- @field resource_categories table<string,boolean>? The resource categories this character or mining drill supports.
--- @field supply_area_distance double? The supply area of this electric pole or beacon prototype.
--- @field max_wire_distance double The maximum wire distance for this entity. 0 if the entity doesn't support wires.
--- @field max_circuit_wire_distance double The maximum circuit wire distance for this entity. 0 if the entity doesn't support circuit wires.
--- @field energy_usage double? The direct energy usage of this entity, if any.
--- @field max_energy_usage double The theoretical maximum energy usage for this entity.
--- @field max_energy_production double The theoretical maximum energy production for this this entity.
--- @field effectivity double? The effectivity of this car prototype, generator prototype.
--- @field consumption double? The energy consumption of this car prototype.
--- @field friction_force double? The friction of this vehicle prototype.
--- @field braking_force double? The braking force of this vehicle prototype.
--- @field air_resistance double? The air resistance of this rolling stock prototype.
--- @field tank_driving boolean? If this car prototype uses tank controls to drive.
--- @field rotation_speed double? The rotation speed of this car prototype.
--- @field turret_rotation_speed double? The turret rotation speed of this car prototype.
--- @field guns table<string,LuaItemPrototype>? A mapping of the gun name to the gun prototype this prototype uses. `nil` if this entity prototype doesn't use guns.
--- @field indexed_guns LuaItemPrototype[]? A vector of the gun prototypes of this car, spider vehicle, artillery wagon, or turret.
--- @field speed double? The default speed of this flying robot, rolling stock or unit. For rolling stocks, this is their `max_speed`.
--- @field speed_multiplier_when_out_of_energy float? The speed multiplier when this flying robot is out of energy.
--- @field max_payload_size uint? The max payload size of this logistics or construction robot.
--- @field draw_cargo boolean? Whether this logistics or construction robot renders its cargo when flying.
--- @field energy_per_move double? The energy consumed per tile moved for this flying robot.
--- @field energy_per_tick double? The energy consumed per tick for this flying robot.
--- @field max_energy double? The max energy for this flying robot.
--- @field min_to_charge float? The minimum energy for this flying robot before it tries to recharge.
--- @field max_to_charge float? The maximum energy for this flying robot above which it won't try to recharge when stationing.
--- @field burner_prototype LuaBurnerPrototype? The burner energy source prototype this entity uses, if any.
--- @field electric_energy_source_prototype LuaElectricEnergySourcePrototype? The electric energy source prototype this entity uses, if any.
--- @field heat_energy_source_prototype LuaHeatEnergySourcePrototype? The heat energy source prototype this entity uses, if any.
--- @field fluid_energy_source_prototype LuaFluidEnergySourcePrototype? The fluid energy source prototype this entity uses, if any.
--- @field void_energy_source_prototype LuaVoidEnergySourcePrototype? The void energy source prototype this entity uses, if any.
--- @field heat_buffer_prototype LuaHeatBufferPrototype? The heat buffer prototype this entity uses, if any.
--- @field building_grid_bit_shift uint The log2 of grid size of the building
--- @field fluid_usage_per_tick double? The fluid usage of this generator prototype.
--- @field maximum_temperature double? The maximum fluid temperature of this generator prototype.
--- @field burns_fluid boolean? Whether this generator prototype burns fluid.
--- @field scale_fluid_usage boolean? Whether this generator prototype scales fluid usage.
--- @field destroy_non_fuel_fluid boolean? Whether this generator prototype destroys non-fuel fluids.
--- @field max_power_output double? The default maximum power output of this generator prototype.
--- @field target_temperature double? The target temperature of this boiler prototype.
--- @field fluid LuaFluidPrototype? The fluid this offshore pump produces.
--- @field fluid_capacity double The fluid capacity of this entity or 0 if this entity doesn't support fluids.
--- @field pumping_speed double? The pumping speed of this offshore or normal pump.
--- @field stack boolean? Whether this inserter is a stack-type.
--- @field allow_custom_vectors boolean? Whether this inserter allows custom pickup and drop vectors.
--- @field allow_burner_leech boolean? Whether this inserter allows burner leeching.
--- @field inserter_extension_speed double? The extension speed of this inserter.
--- @field inserter_rotation_speed double? The rotation speed of this inserter.
--- @field inserter_pickup_position Vector? The pickup position for this inserter.
--- @field inserter_drop_position Vector? The drop position for this inserter.
--- @field inserter_chases_belt_items boolean? True if this inserter chases items on belts for pickup.
--- @field count_as_rock_for_filtered_deconstruction boolean? If this simple-entity is counted as a rock for the deconstruction planner "trees and rocks only" filter.
--- @field filter_count uint? The filter count of this inserter, loader, or logistic chest. For logistic containers, `nil` means no limit.
--- @field time_to_live uint The time to live for this prototype or `0` if prototype doesn't have time_to_live or time_before_removed.
--- @field distribution_effectivity double? The distribution effectivity for this beacon prototype.
--- @field explosion_beam double? Whether this explosion has a beam.
--- @field explosion_rotate double? Whether this explosion rotates.
--- @field tree_color_count uint8? If it is a tree, return the number of colors it supports.
--- @field alert_when_damaged boolean? Whether this entity raises an alert when damaged.
--- @field alert_when_attacking boolean? Whether this turret raises an alert when attacking
--- @field color Color? The color of the prototype, if any.
--- @field collision_mask_collides_with_self boolean Does this prototype collision mask collide with itself?
--- @field collision_mask_collides_with_tiles_only boolean Does this prototype collision mask collide with tiles only?
--- @field collision_mask_considers_tile_transitions boolean Does this prototype collision mask consider tile transitions?
--- @field allowed_effects table<string,boolean>? The allowed module effects for this entity, if any.
--- @field rocket_parts_required uint? The rocket parts required for this rocket silo prototype.
--- @field rocket_rising_delay uint8? The rocket rising delay for this rocket silo prototype.
--- @field launch_wait_time uint8? The rocket launch delay for this rocket silo prototype.
--- @field light_blinking_speed double? The light blinking speed for this rocket silo prototype.
--- @field door_opening_speed double? The door opening speed for this rocket silo prototype.
--- @field rising_speed double? The rising speed for this rocket silo rocket prototype.
--- @field engine_starting_speed double? The engine starting speed for this rocket silo rocket prototype.
--- @field flying_speed double? The flying speed for this rocket silo rocket prototype.
--- @field flying_acceleration double? The flying acceleration for this rocket silo rocket prototype.
--- @field fixed_recipe string? The fixed recipe name for this assembling machine prototype, if any.
--- @field construction_radius double? The construction radius for this roboport prototype.
--- @field logistic_radius double? The logistic radius for this roboport prototype.
--- @field energy_per_hit_point double? The energy used per hitpoint taken for this vehicle during collisions.
--- @field create_ghost_on_death boolean If this prototype will attempt to create a ghost of itself on death.
--- @field ammo_category string? Name of the ammo category of this land mine.
--- @field timeout uint? The time it takes this land mine to arm.
--- @field trigger_collision_mask CollisionMaskWithFlags? The collision mask entities must collide with to make this landmine blow up.
--- @field fluidbox_prototypes LuaFluidBoxPrototype[] The fluidbox prototypes for this entity.
--- @field neighbour_bonus double? 
--- @field container_distance double? 
--- @field belt_distance double? 
--- @field belt_length double? 
--- @field is_building boolean Everything in the following list is considered a building.  - AccumulatorPrototype - ArtilleryTurretPrototype - BeaconPrototype - BoilerPrototype - BurnerGeneratorPrototype - CombinatorPrototype → ArithmeticCombinator, DeciderCombinator - ConstantCombinatorPrototype - ContainerPrototype → LogisticContainer, InfinityContainer - CraftingMachinePrototype → AssemblingMachine, RocketSilo, Furnace - ElectricEnergyInterfacePrototype - ElectricPolePrototype - EnemySpawnerPrototype - GatePrototype - GeneratorPrototype - HeatInterfacePrototype - HeatPipePrototype - InserterPrototype - LabPrototype - LampPrototype - LinkedContainerPrototype - MarketPrototype - MiningDrillPrototype - OffshorePumpPrototype - PipePrototype → InfinityPipe - PipeToGroundPrototype - PlayerPortPrototype - PowerSwitchPrototype - ProgrammableSpeakerPrototype - PumpPrototype - RadarPrototype - RailPrototype → CurvedRail, StraightRail - RailSignalBasePrototype → RailChainSignal, RailSignal - ReactorPrototype - RoboportPrototype - SimpleEntityPrototype - SimpleEntityWithOwnerPrototype → SimpleEntityWithForce - SolarPanelPrototype - StorageTankPrototype - TrainStopPrototype - TransportBeltConnectablePrototype → LinkedBelt, Loader1x1, Loader1x2, Splitter, TransportBelt, UndergroundBelt - TurretPrototype → AmmoTurret, ElectricTurret, FluidTurret - WallPrototype
--- @field automated_ammo_count uint? The amount of ammo that inserters automatically insert into this ammo-turret or artillery-turret.
--- @field max_speed double? The max speed of this projectile or flying robot prototype.
--- @field darkness_for_all_lamps_on float? Value between 0 and 1 darkness where all lamps of this lamp prototype are on.
--- @field darkness_for_all_lamps_off float? Value between 0 and 1 darkness where all lamps of this lamp prototype are off.
--- @field always_on boolean? Whether the lamp is always on (except when out of power or turned off by the circuit network).
--- @field min_darkness_to_spawn float? The minimum darkness at which this unit spawner can spawn entities.
--- @field max_darkness_to_spawn float? The maximum darkness at which this unit spawner can spawn entities.
--- @field call_for_help_radius double? 
--- @field max_count_of_owned_units double? Count of enemies this spawner can sustain.
--- @field max_friends_around_to_spawn double? How many friendly units are required within the spawning_radius of this spawner for it to stop producing more units.
--- @field spawning_radius double? How far from the spawner can the units be spawned.
--- @field spawning_spacing double? What spaces should be between the spawned units.
--- @field radius double The radius of this entity prototype.
--- @field cliff_explosive_prototype string? The item prototype name used to destroy this cliff.
--- @field rocket_entity_prototype LuaEntityPrototype? The rocket entity prototype associated with this rocket silo prototype.
--- @field has_belt_immunity boolean? Whether this unit, car, or character prototype has belt immunity.
--- @field vision_distance double? The vision distance of this unit prototype.
--- @field pollution_to_join_attack float? The amount of pollution that has to be absorbed by the unit's spawner before the unit will leave the spawner and attack the source of the pollution.
--- @field min_pursue_time uint? The minimum pursue time of this unit prototype.
--- @field max_pursue_distance double? The maximum pursue distance of this unit prototype.
--- @field radar_range uint? The radar range of this unit prototype.
--- @field move_while_shooting boolean? Whether this unit prototype can move while shooting.
--- @field can_open_gates boolean? Whether this unit prototype can open gates.
--- @field affected_by_tiles boolean? Whether this unit prototype is affected by tile walking speed modifiers.
--- @field distraction_cooldown uint? The distraction cooldown of this unit prototype.
--- @field spawning_time_modifier double? The spawning time modifier of this unit prototype.
--- @field alert_icon_shift Vector The alert icon shift of this entity prototype.
--- @field alert_icon_scale float The alert icon scale of this entity prototype.
--- @field lab_inputs string[]? The item prototype names that are the inputs of this lab prototype.
--- @field researching_speed double? The base researching speed of this lab prototype.
--- @field item_slot_count uint? The item slot count of this constant combinator prototype.
--- @field base_productivity double? The base productivity of this crafting machine, lab, or mining drill.
--- @field allow_access_to_all_forces boolean? Whether this market allows access to all forces or just friendly ones.
--- @field supports_direction boolean Whether this entity prototype could possibly ever be rotated.
--- @field terrain_friction_modifier float? The terrain friction modifier for this vehicle.
--- @field allow_passengers boolean? Whether this vehicle allows passengers.
--- @field max_distance_of_sector_revealed uint? The radius of the area this radar can chart, in chunks.
--- @field max_distance_of_nearby_sector_revealed uint? The radius of the area constantly revealed by this radar, in chunks.
--- @field adjacent_tile_collision_box BoundingBox? The bounding box that specifies which tiles adjacent to the offshore pump should be checked.
--- @field adjacent_tile_collision_mask CollisionMask? Tiles adjacent to the offshore pump must not collide with this collision mask.
--- @field adjacent_tile_collision_test CollisionMask? If this mask is not empty, tiles adjacent to the offshore pump must not collide with this collision mask.
--- @field center_collision_mask CollisionMask? The collision mask used only for collision test with tile directly at offshore pump position.
--- @field grid_prototype LuaEquipmentGridPrototype? The equipment grid prototype for this entity, if any.
--- @field remove_decoratives string 
--- @field related_underground_belt LuaEntityPrototype? 
--- @field inserter_stack_size_bonus double? The built-in stack size bonus of this inserter prototype.
--- @field protected_from_tile_building boolean True if this entity prototype should be included during tile collision checks with [LuaTilePrototype::check_collision_with_entities](LuaTilePrototype::check_collision_with_entities) enabled.
--- @field is_entity_with_owner boolean True if this is entity-with-owner
--- @field is_military_target boolean? True if this entity-with-owner is military target
--- @field allow_run_time_change_of_is_military_target boolean? True if this entity-with-owner's is_military_target can be changed run-time (on the entity, not on the prototype itself)
--- @field logistic_parameters { charge_approach_distance: float, charging_distance: float, charging_energy: double, charging_station_count: uint, charging_station_shift: Vector, charging_threshold_distance: float, construction_radius: float, logistic_radius: float, logistics_connection_distance: float, robot_limit: uint, robot_vertical_acceleration: float, robots_shrink_when_entering_and_exiting: boolean, spawn_and_station_height: float, spawn_and_station_shadow_height_offset: float, stationing_offset: Vector }? The logistic parameters for this roboport.
--- @field height double? The height of this spider vehicle prototype.
--- @field torso_rotation_speed double? The torso rotation speed of this spider vehicle prototype.
--- @field torso_bob_speed double? The torso bob speed of this spider vehicle prototype.
--- @field automatic_weapon_cycling boolean? Whether this spider vehicle prototype automatically cycles weapons.
--- @field chain_shooting_cooldown_modifier double? The chain shooting cooldown modifier of this spider vehicle prototype.
--- @field chunk_exploration_radius double? The chunk exploration radius of this spider vehicle prototype.
--- @field animation_speed_coefficient double? The animation speed coefficient of this belt connectable prototype.
--- @field manual_range_modifier double? The manual range modifier for this artillery turret or wagon prototype.  subclass(ArtilleryWagon, ArtilleryTurret)
--- @field dying_speed float? The dying time of this corpse prototype.
--- @field active_energy_usage double? The active energy usage of this rocket silo or combinator prototype.
--- @field idle_energy_usage double? The idle energy usage of this rocket silo prototype.
--- @field lamp_energy_usage double? The lamp energy usage of this rocket silo prototype.
--- @field use_exact_mode boolean? Whether this logistic container prototype uses exact mode
--- @field tile_width uint Specifies the tiling size of the entity, is used to decide, if the center should be in the center of the tile (odd tile size dimension) or on the tile border (even tile size dimension)
--- @field tile_height uint Specifies the tiling size of the entity, is used to decide, if the center should be in the center of the tile (odd tile size dimension) or on the tile border (even tile size dimension)
--- @field running_speed double? The current movement speed of this character, including effects from exoskeletons, tiles, stickers and shooting.
--- @field maximum_corner_sliding_distance double? 
--- @field build_distance uint? 
--- @field drop_item_distance uint? 
--- @field reach_distance uint? 
--- @field reach_resource_distance double? 
--- @field item_pickup_distance double? 
--- @field loot_pickup_distance double? 
--- @field enter_vehicle_distance double? 
--- @field ticks_to_keep_gun uint? 
--- @field ticks_to_keep_aiming_direction uint? 
--- @field ticks_to_stay_in_combat uint? 
--- @field respawn_time uint? 
--- @field damage_hit_tint Color? 
--- @field character_corpse LuaEntityPrototype? 
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaEntityPrototype = { }

----
--- Gets the base size of the given inventory on this entity or `nil` if the given inventory doesn't exist.
--- 
--- @param index defines.inventory 
--- @return uint? 
----
function LuaEntityPrototype.get_inventory_size(index) end

----
--- Test whether this entity prototype has a certain flag set.
--- 
--- @param flag string The flag to test. See [EntityPrototypeFlags](EntityPrototypeFlags) for a list of flags.
--- @return boolean `true` if this prototype has the given flag set.
----
function LuaEntityPrototype.has_flag(flag) end

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaEntityPrototype.help() end


----
--- @class LuaEquipment
--- An item in a [LuaEquipmentGrid](LuaEquipmentGrid), for example a fusion reactor placed in one's power armor.
--- 
--- An equipment reference becomes invalid once the equipment is removed or the equipment grid it resides in is destroyed.
--- 
--- @field name string Name of this equipment.
--- @field type string Type of this equipment.
--- @field position EquipmentPosition Position of this equipment in the equipment grid.
--- @field shape { height: uint, width: uint } Shape of this equipment.
--- @field shield double Current shield value of the equipment.
--- @field max_shield double Maximum shield value.
--- @field max_solar_power double Maximum solar power generated.
--- @field movement_bonus double Movement speed bonus.
--- @field generator_power double Energy generated per tick.
--- @field energy double Current available energy.
--- @field max_energy double Maximum amount of energy that can be stored in this equipment.
--- @field prototype LuaEquipmentPrototype 
--- @field burner LuaBurner? The burner energy source for this equipment, if any.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaEquipment = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaEquipment.help() end


----
--- @class LuaEquipmentCategoryPrototype
--- Prototype of an equipment category.
--- 
--- @field name string Name of this prototype.
--- @field order string The string used to alphabetically sort these prototypes. It is a simple string that has no additional semantic meaning.
--- @field localised_name LocalisedString 
--- @field localised_description LocalisedString 
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaEquipmentCategoryPrototype = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaEquipmentCategoryPrototype.help() end


----
--- @class LuaEquipmentGrid
--- An equipment grid is for example the inside of a power armor.
--- 
--- @field prototype LuaEquipmentGridPrototype 
--- @field width uint Width of the equipment grid.
--- @field height uint Height of the equipment grid.
--- @field equipment LuaEquipment[] All the equipment in this grid.
--- @field generator_energy double Total energy per tick generated by the equipment inside this grid.
--- @field max_solar_energy double Maximum energy per tick that can be created by any solar panels in the equipment grid. Actual generated energy varies depending on the daylight levels.
--- @field available_in_batteries double The total energy stored in all batteries in the equipment grid.
--- @field battery_capacity double Total energy storage capacity of all batteries in the equipment grid.
--- @field shield float The amount of shields this equipment grid has.
--- @field max_shield float The maximum amount of shields this equipment grid has.
--- @field inhibit_movement_bonus boolean Whether this grid's equipment movement bonus is active.
--- @field unique_id uint Unique identifier of this equipment grid.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaEquipmentGrid = { }

----
--- @class LuaEquipmentGrid.can_move_parameters
--- Parameters class for the method `LuaEquipmentGrid.can_move`
--- 
--- @field equipment LuaEquipment The equipment to move
--- @field position EquipmentPosition Where to put it
----

----
--- Check whether moving an equipment would succeed.
--- 
--- @param args LuaEquipmentGrid.can_move_parameters Table containing arguments for the method.
--- @see LuaEquipmentGrid.can_move_parameters
--- @return boolean 
----
function LuaEquipmentGrid.can_move(args) end

----
--- Clear all equipment from the grid, removing it without actually returning it.
--- 
--- @param by_player PlayerIdentification? If provided, the action is done 'as' this player and [on_player_removed_equipment](on_player_removed_equipment) is triggered.
----
function LuaEquipmentGrid.clear(by_player) end

----
--- Get the number of all or some equipment in this grid.
--- 
--- @param equipment string? Prototype name of the equipment to count. If not specified, count all equipment.
--- @return uint 
----
function LuaEquipmentGrid.count(equipment) end

----
--- Find equipment by name.
--- 
--- @param equipment string Prototype name of the equipment to find.
--- @return LuaEquipment? The first found equipment, or `nil` if equipment could not be found.
----
function LuaEquipmentGrid.find(equipment) end

----
--- Find equipment in the Equipment Grid based off a position.
--- 
--- @param position EquipmentPosition The position
--- @return LuaEquipment? The found equipment, or `nil` if equipment could not be found at the given position.
----
function LuaEquipmentGrid.get(position) end

----
--- Get counts of all equipment in this grid.
--- 
--- @return table<string,uint> The counts, indexed by equipment names.
----
function LuaEquipmentGrid.get_contents() end

