require('.classes-1')
require('.classes-2')
require('.classes-3')
require('.classes-4')
require('.classes-5')

----
--- Remove children of this element. Any [LuaGuiElement](LuaGuiElement) objects referring to the destroyed elements become invalid after this operation.
--- 
----
function LuaGuiElement.clear() end

----
--- Removes the items in this dropdown or listbox.
--- 
----
function LuaGuiElement.clear_items() end

----
--- Closes the dropdown list if this is a dropdown and it is open.
--- 
----
function LuaGuiElement.close_dropdown() end

----
--- Remove this element, along with its children. Any [LuaGuiElement](LuaGuiElement) objects referring to the destroyed elements become invalid after this operation.
--- 
----
function LuaGuiElement.destroy() end

----
--- Focuses this GUI element if possible.
--- 
----
function LuaGuiElement.focus() end

----
--- Forces this frame to re-auto-center. Only works on frames stored directly in [LuaGui::screen](LuaGui::screen).
--- 
----
function LuaGuiElement.force_auto_center() end

----
--- Gets the index that this element has in its parent element.
--- 
--- @return uint 
----
function LuaGuiElement.get_index_in_parent() end

----
--- Gets the item at the given index from this dropdown or listbox.
--- 
--- @param index uint The index to get
--- @return LocalisedString 
----
function LuaGuiElement.get_item(index) end

----
--- The mod that owns this Gui element or `nil` if it's owned by the scenario script.
--- 
--- @return string? 
----
function LuaGuiElement.get_mod() end

----
--- Returns whether this slider only allows being moved to discrete positions.
--- 
--- @return boolean 
----
function LuaGuiElement.get_slider_discrete_slider() end

----
--- Returns whether this slider only allows discrete values.
--- 
--- @return boolean 
----
function LuaGuiElement.get_slider_discrete_values() end

----
--- Gets this sliders maximum value.
--- 
--- @return double 
----
function LuaGuiElement.get_slider_maximum() end

----
--- Gets this sliders minimum value.
--- 
--- @return double 
----
function LuaGuiElement.get_slider_minimum() end

----
--- Gets the minimum distance this slider can move.
--- 
--- @return double 
----
function LuaGuiElement.get_slider_value_step() end

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaGuiElement.help() end

----
--- Removes the item at the given index from this dropdown or listbox.
--- 
--- @param index uint The index
----
function LuaGuiElement.remove_item(index) end

----
--- Removes the given tab and its associated content from this tabbed pane.
--- 
--- @param tab LuaGuiElement The tab to remove. If not given, it removes all tabs.
----
function LuaGuiElement.remove_tab(tab) end

----
--- Scrolls this scroll bar to the bottom.
--- 
----
function LuaGuiElement.scroll_to_bottom() end

----
--- Scrolls this scroll bar such that the specified GUI element is visible to the player.
--- 
--- @param element LuaGuiElement The element to scroll to.
--- @param scroll_mode string? Where the element should be positioned in the scroll-pane. Must be either `"in-view"` or `"top-third"`. Defaults to `"in-view"`.
----
function LuaGuiElement.scroll_to_element(element, scroll_mode) end

----
--- Scrolls the scroll bar such that the specified listbox item is visible to the player.
--- 
--- @param index int The item index to scroll to.
--- @param scroll_mode string? Where the item should be positioned in the list-box. Must be either `"in-view"` or `"top-third"`. Defaults to `"in-view"`.
----
function LuaGuiElement.scroll_to_item(index, scroll_mode) end

----
--- Scrolls this scroll bar to the left.
--- 
----
function LuaGuiElement.scroll_to_left() end

----
--- Scrolls this scroll bar to the right.
--- 
----
function LuaGuiElement.scroll_to_right() end

----
--- Scrolls this scroll bar to the top.
--- 
----
function LuaGuiElement.scroll_to_top() end

----
--- Selects a range of text in this textbox.
--- 
--- @param start int The index of the first character to select
--- @param end_ int The index of the last character to select
----
function LuaGuiElement.select(start, end_) end

----
--- Selects all the text in this textbox.
--- 
----
function LuaGuiElement.select_all() end

----
--- Sets the given string at the given index in this dropdown or listbox.
--- 
--- @param index uint The index whose text to replace.
--- @param string LocalisedString The text to set at the given index.
----
function LuaGuiElement.set_item(index, string) end

----
--- Sets whether this slider only allows being moved to discrete positions.
--- 
--- @param value boolean 
----
function LuaGuiElement.set_slider_discrete_slider(value) end

----
--- Sets whether this slider only allows discrete values.
--- 
--- @param value boolean 
----
function LuaGuiElement.set_slider_discrete_values(value) end

----
--- Sets this sliders minimum and maximum values.
--- 
--- @param minimum double 
--- @param maximum double 
----
function LuaGuiElement.set_slider_minimum_maximum(minimum, maximum) end

----
--- Sets the minimum distance this slider can move.
--- 
--- @param value double 
----
function LuaGuiElement.set_slider_value_step(value) end

----
--- Swaps the children at the given indices in this element.
--- 
--- @param index_1 uint The index of the first child.
--- @param index_2 uint The index of the second child.
----
function LuaGuiElement.swap_children(index_1, index_2) end


----
--- @class LuaHeatBufferPrototype
--- Prototype of a heat buffer.
--- 
--- @field max_temperature double 
--- @field default_temperature double 
--- @field specific_heat double 
--- @field max_transfer double 
--- @field min_temperature_gradient double 
--- @field min_working_temperature double 
--- @field minimum_glow_temperature double 
--- @field connections HeatConnection[] 
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaHeatBufferPrototype = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaHeatBufferPrototype.help() end


----
--- @class LuaHeatEnergySourcePrototype
--- Prototype of a heat energy source.
--- 
--- @field emissions double The emissions of this energy source in `pollution/Joule`. Multiplying it by energy consumption in `Watt` gives `pollution/second`.
--- @field render_no_network_icon boolean 
--- @field render_no_power_icon boolean 
--- @field max_temperature double 
--- @field default_temperature double 
--- @field specific_heat double 
--- @field max_transfer double 
--- @field min_temperature_gradient double 
--- @field min_working_temperature double 
--- @field minimum_glow_temperature double 
--- @field connections HeatConnection[] 
--- @field heat_buffer_prototype LuaHeatBufferPrototype 
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaHeatEnergySourcePrototype = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaHeatEnergySourcePrototype.help() end


----
--- @class LuaInserterControlBehavior: LuaGenericOnOffControlBehavior
--- Control behavior for inserters.
--- 
--- @field circuit_read_hand_contents boolean `true` if the contents of the inserter hand should be sent to the circuit network
--- @field circuit_mode_of_operation defines.control_behavior.inserter.circuit_mode_of_operation The circuit mode of operations for the inserter.
--- @field circuit_hand_read_mode defines.control_behavior.inserter.hand_read_mode The hand read mode for the inserter.
--- @field circuit_set_stack_size boolean If the stack size of the inserter is set through the circuit network or not.
--- @field circuit_stack_control_signal SignalID The signal used to set the stack size of the inserter.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaInserterControlBehavior = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaInserterControlBehavior.help() end


----
--- @class LuaInventory
--- A storage of item stacks.
--- 
--- @field index defines.inventory? The inventory index this inventory uses, if any.
--- @field entity_owner LuaEntity? The entity that owns this inventory, if any.
--- @field player_owner LuaPlayer? The player that owns this inventory, if any.
--- @field equipment_owner LuaEquipment? The equipment that owns this inventory, if any.
--- @field mod_owner string? The mod that owns this inventory, if any.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaInventory = { }

----
--- Can at least some items be inserted?
--- 
--- @param items ItemStackIdentification Items that would be inserted.
--- @return boolean `true` if at least a part of the given items could be inserted into this inventory.
----
function LuaInventory.can_insert(items) end

----
--- If the given inventory slot filter can be set to the given filter.
--- 
--- @param index uint The item stack index
--- @param filter string The item name of the filter
--- @return boolean 
----
function LuaInventory.can_set_filter(index, filter) end

----
--- Make this inventory empty.
--- 
----
function LuaInventory.clear() end

----
--- Counts the number of empty stacks.
--- 
--- @param include_filtered boolean? If true, filtered slots will be included. Defaults to false.
--- @param include_bar boolean? If true, slots blocked by the current bar will be included. Defaults to true.
--- @return uint 
----
function LuaInventory.count_empty_stacks(include_filtered, include_bar) end

----
--- Destroys this inventory.
--- 
----
function LuaInventory.destroy() end

----
--- Finds the first empty stack. Filtered slots are excluded unless a filter item is given.
--- 
--- @param item string? If given, empty stacks that are filtered for this item will be included.
--- @return LuaItemStack? The first empty stack, or `nil` if there aren't any empty stacks.
--- @return uint? The stack index of the matching stack, if any is found.
----
function LuaInventory.find_empty_stack(item) end

----
--- Finds the first LuaItemStack in the inventory that matches the given item name.
--- 
--- @param item string The item name to find
--- @return LuaItemStack? The first matching stack, or `nil` if none match.
--- @return uint? The stack index of the matching stack, if any is found.
----
function LuaInventory.find_item_stack(item) end

----
--- Get the current bar. This is the index at which the red area starts.
--- 
--- @return uint 
----
function LuaInventory.get_bar() end

----
--- Get counts of all items in this inventory.
--- 
--- @return table<string,uint> The counts, indexed by item names.
----
function LuaInventory.get_contents() end

----
--- Gets the filter for the given item stack index.
--- 
--- @param index uint The item stack index
--- @return string? The current filter or `nil` if none.
----
function LuaInventory.get_filter(index) end

----
--- Gets the number of the given item that can be inserted into this inventory.
--- 
--- @param item string The item to check.
----
function LuaInventory.get_insertable_count(item) end

----
--- Get the number of all or some items in this inventory.
--- 
--- @param item string? Prototype name of the item to count. If not specified, count all items.
--- @return uint 
----
function LuaInventory.get_item_count(item) end

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaInventory.help() end

----
--- Insert items into this inventory.
--- 
--- @param items ItemStackIdentification Items to insert.
--- @return uint Number of items actually inserted.
----
function LuaInventory.insert(items) end

----
--- Does this inventory contain nothing?
--- 
--- @return boolean 
----
function LuaInventory.is_empty() end

----
--- If this inventory supports filters and has at least 1 filter set.
--- 
--- @return boolean 
----
function LuaInventory.is_filtered() end

----
--- Is every stack in this inventory full? Ignores stacks blocked by the current bar.
--- 
--- @return boolean 
----
function LuaInventory.is_full() end

----
--- Remove items from this inventory.
--- 
--- @param items ItemStackIdentification Items to remove.
--- @return uint Number of items actually removed.
----
function LuaInventory.remove(items) end

----
--- Resizes the inventory.
--- 
--- @param size uint16 New size of a inventory
----
function LuaInventory.resize(size) end

----
--- Set the current bar.
--- 
--- @param bar uint? The new limit. Omitting this parameter will clear the limit.
----
function LuaInventory.set_bar(bar) end

----
--- Sets the filter for the given item stack index.
--- 
--- @param index uint The item stack index.
--- @param filter nil|string The new filter. `nil` erases any existing filter.
--- @return boolean If the filter was allowed to be set.
----
function LuaInventory.set_filter(index, filter) end

----
--- Sorts and merges the items in this inventory.
--- 
----
function LuaInventory.sort_and_merge() end

----
--- Does this inventory support a bar? Bar is the draggable red thing, found for example on chests, that limits the portion of the inventory that may be manipulated by machines.
--- 
--- @return boolean 
----
function LuaInventory.supports_bar() end

----
--- If this inventory supports filters.
--- 
--- @return boolean 
----
function LuaInventory.supports_filters() end


----
--- @class LuaItemPrototype
--- Prototype of an item.
--- 
--- @field type string Type of this prototype. E.g. `"gun"` or `"mining-tool"`.
--- @field name string Name of this prototype.
--- @field localised_name LocalisedString 
--- @field localised_description LocalisedString 
--- @field order string The string used to alphabetically sort these prototypes. It is a simple string that has no additional semantic meaning.
--- @field place_result LuaEntityPrototype? Prototype of the entity that will be created by placing this item, if any.
--- @field place_as_equipment_result LuaEquipmentPrototype? Prototype of the equipment that will be created by placing this item in an equipment grid, if any.
--- @field place_as_tile_result PlaceAsTileResult? The place-as-tile result if one is defined, if any.
--- @field stackable boolean Is this item allowed to stack at all?
--- @field default_request_amount uint The default request value.
--- @field stack_size uint Maximum stack size of the item specified by this prototype.
--- @field wire_count uint The number of items needed to connect two entities with this as wire.
--- @field fuel_category string? The fuel category of this item prototype, if any.
--- @field burnt_result LuaItemPrototype? The result of burning this item as fuel, if any.
--- @field fuel_value float Fuel value when burned.
--- @field fuel_acceleration_multiplier double The acceleration multiplier when this item is used as fuel in a vehicle.
--- @field fuel_top_speed_multiplier double The fuel top speed multiplier when this item is used as fuel in a vehicle.
--- @field fuel_emissions_multiplier double The emissions multiplier if this is used as fuel.
--- @field subgroup LuaGroup The subgroup this prototype belongs to.
--- @field group LuaGroup The group this prototype belongs to.
--- @field flags ItemPrototypeFlags The flags for this item prototype.
--- @field rocket_launch_products Product[] The results of launching this item in a rocket.
--- @field can_be_mod_opened boolean If this item can be mod-opened.
--- @field magazine_size float? Size of full magazine.
--- @field reload_time float? Amount of extra time (in ticks) it takes to reload the weapon after depleting the magazine.
--- @field equipment_grid LuaEquipmentGridPrototype? The prototype of this armor equipment grid, if any.
--- @field resistances table<string,Resistance>? Resistances of this armor item, if any, indexed by damage type name.
--- @field inventory_size_bonus uint? The inventory size bonus for this armor prototype.
--- @field capsule_action CapsuleAction? The capsule action for this capsule item prototype.
--- @field attack_parameters AttackParameters? The gun attack parameters.
--- @field inventory_size uint? The main inventory size for item-with-inventory-prototype.
--- @field item_filters table<string,LuaItemPrototype>? 
--- @field item_group_filters table<string,LuaGroup>? 
--- @field item_subgroup_filters table<string,LuaGroup>? 
--- @field filter_mode string? The filter mode used by this item with inventory.
--- @field insertion_priority_mode string? The insertion priority mode used by this item with inventory.
--- @field localised_filter_message LocalisedString? The localised string used when the player attempts to put items into this item with inventory that aren't allowed.
--- @field extend_inventory_by_default boolean? If this item with inventory extends the inventory it resides in by default.
--- @field default_label_color Color? The default label color used for this item with label, if any.
--- @field draw_label_for_cursor_render boolean? If true, and this item with label has a label it is drawn in place of the normal number when held in the cursor.
--- @field speed float? The repairing speed if this is a repairing tool.
--- @field module_effects ModuleEffects? Effects of this module.
--- @field category string? The name of a [LuaModuleCategoryPrototype](LuaModuleCategoryPrototype). Used when upgrading modules: Ctrl + click modules into an entity and it will replace lower tier modules of the same category with higher tier modules.
--- @field tier uint? Tier of the module inside its category. Used when upgrading modules: Ctrl + click modules into an entity and it will replace lower tier modules with higher tier modules if they have the same category.
--- @field limitations string[]? An array of recipe names this module is allowed to work with. Empty when all recipes are allowed.
--- @field limitation_message_key string? The limitation message key used when the player attempts to use this modules in some place it's not allowed.
--- @field straight_rail LuaEntityPrototype? The straight rail prototype used for this rail planner prototype.
--- @field curved_rail LuaEntityPrototype? The curved rail prototype used for this rail planner prototype.
--- @field repair_result TriggerItem[]? The repair result of this repair tool prototype.
--- @field selection_border_color Color? The color used when doing normal selection with this selection tool prototype.
--- @field alt_selection_border_color Color? The color used when doing alt selection with this selection tool prototype.
--- @field reverse_selection_border_color Color? The color used when doing reverse selection with this selection tool prototype.
--- @field alt_reverse_selection_border_color Color? The color used when doing alt reverse selection with this selection tool prototype.
--- @field selection_mode_flags SelectionModeFlags? Flags that affect which entities will be selected.
--- @field alt_selection_mode_flags SelectionModeFlags? Flags that affect which entities will be selected during alternate selection.
--- @field reverse_selection_mode_flags SelectionModeFlags? Flags that affect which entities will be selected during reverse selection.
--- @field alt_reverse_selection_mode_flags SelectionModeFlags? Flags that affect which entities will be selected during alt reverse selection.
--- @field selection_cursor_box_type string? 
--- @field alt_selection_cursor_box_type string? 
--- @field reverse_selection_cursor_box_type string? 
--- @field alt_reverse_selection_cursor_box_type string? 
--- @field always_include_tiles boolean? If tiles area always included when doing selection with this selection tool prototype.
--- @field entity_filter_mode string? The entity filter mode used by this selection tool.
--- @field alt_entity_filter_mode string? The alt entity filter mode used by this selection tool.
--- @field reverse_alt_entity_filter_mode string? The reverse entity filter mode used by this selection tool.
--- @field alt_reverse_alt_entity_filter_mode string? The alt reverse entity filter mode used by this selection tool.
--- @field tile_filter_mode string? The tile filter mode used by this selection tool.
--- @field alt_tile_filter_mode string? The alt tile filter mode used by this selection tool.
--- @field reverse_tile_filter_mode string? The reverse tile filter mode used by this selection tool.
--- @field alt_reverse_tile_filter_mode string? The alt reverse tile filter mode used by this selection tool.
--- @field entity_filters table<string,LuaEntityPrototype>? The entity filters used by this selection tool indexed by entity name.
--- @field alt_entity_filters table<string,LuaEntityPrototype>? The alt entity filters used by this selection tool indexed by entity name.
--- @field reverse_entity_filters table<string,LuaEntityPrototype>? The reverse entity filters used by this selection tool indexed by entity name.
--- @field alt_reverse_entity_filters table<string,LuaEntityPrototype>? The alt reverse entity filters used by this selection tool indexed by entity name.
--- @field entity_type_filters table<string,boolean>? The entity type filters used by this selection tool indexed by entity type.
--- @field alt_entity_type_filters table<string,boolean>? The alt entity type filters used by this selection tool indexed by entity type.
--- @field reverse_entity_type_filters table<string,boolean>? The reverse entity type filters used by this selection tool indexed by entity type.
--- @field alt_reverse_entity_type_filters table<string,boolean>? The alt reverse entity type filters used by this selection tool indexed by entity type.
--- @field tile_filters table<string,LuaTilePrototype>? The tile filters used by this selection tool indexed by tile name.
--- @field alt_tile_filters table<string,LuaTilePrototype>? The alt tile filters used by this selection tool indexed by tile name.
--- @field reverse_tile_filters table<string,LuaTilePrototype>? The reverse tile filters used by this selection tool indexed by tile name.
--- @field alt_reverse_tile_filters table<string,LuaTilePrototype>? The alt reverse tile filters used by this selection tool indexed by tile name.
--- @field entity_filter_slots uint? The number of entity filters this deconstruction item has.
--- @field tile_filter_slots uint? The number of tile filters this deconstruction item has.
--- @field durability_description_key string? The durability message key used when displaying the durability of this tool.
--- @field durability double? The durability of this tool item.
--- @field infinite boolean? If this tool item has infinite durability.
--- @field mapper_count uint? How many filters an upgrade item has.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaItemPrototype = { }

----
--- The type of this ammo prototype.
--- 
--- @param ammo_source_type string? One of `"default"`, `"player"`, `"turret"`, or `"vehicle"`. Defaults to `"default"`.
--- @return AmmoType? 
----
function LuaItemPrototype.get_ammo_type(ammo_source_type) end

----
--- Does this prototype have a flag enabled?
--- 
--- @param flag string The flag to check. Can be one of [ItemPrototypeFlags](ItemPrototypeFlags). Any other value will cause an error.
--- @return boolean 
----
function LuaItemPrototype.has_flag(flag) end

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaItemPrototype.help() end


----
--- @class LuaItemStack
--- A reference to an item and count owned by some external entity.
--- 
--- @field valid_for_read boolean Is this valid for reading? Differs from the usual `valid` in that `valid` will be `true` even if the item stack is blank but the entity that holds it is still valid.
--- @field prototype LuaItemPrototype Prototype of the item held in this stack.
--- @field name string Prototype name of the item held in this stack.
--- @field type string Type of the item prototype.
--- @field count uint Number of items in this stack.
--- @field grid LuaEquipmentGrid? The equipment grid of this item, if any.
--- @field health float How much health the item has, as a number in range [0, 1].
--- @field durability double? Durability of the contained item. Automatically capped at the item's maximum durability.
--- @field ammo uint Number of bullets left in the magazine.
--- @field blueprint_icons BlueprintSignalIcon[]? Icons of this blueprint item, blueprint book, deconstruction item or upgrade planner. An item that doesn't have icons returns `nil` on read and throws error on write.
--- @field blueprint_snap_to_grid TilePosition? The snapping grid size in this blueprint item. `nil` if snapping is not enabled.
--- @field blueprint_position_relative_to_grid TilePosition? The offset from the absolute grid. `nil` if absolute snapping is not enabled.
--- @field blueprint_absolute_snapping boolean If absolute snapping is enabled on this blueprint item.
--- @field label string? The current label for this item, if any.
--- @field label_color Color? The current label color for this item, if any.
--- @field allow_manual_label_change boolean Whether the label for this item can be manually changed. When false the label can only be changed through the API.
--- @field cost_to_build table<string,uint> Raw materials required to build this blueprint. Result is a dictionary mapping each item prototype name to the required count.
--- @field extends_inventory boolean If this item extends the inventory it resides in (provides its contents for counts, crafting, insertion). Only callable on items with inventories.
--- @field prioritize_insertion_mode string The insertion mode priority this ItemWithInventory uses when items are inserted into an inventory it resides in. Only callable on items with inventories.
--- @field default_icons BlueprintSignalIcon[] The default icons for a blueprint item.
--- @field tags Tags 
--- @field custom_description LocalisedString The custom description this item-with-tags. This is shown over the normal item description if this is set to a non-empty value.
--- @field entity_filters string[] The entity filters for this deconstruction item. The attribute is a sparse array with the keys representing the index of the filter. All strings in this array must be entity prototype names that don't have the `"not-deconstructable"` flag set and are either a `cliff` or marked as `minable`.
--- @field tile_filters string[] The tile filters for this deconstruction item. The attribute is a sparse array with the keys representing the index of the filter. All strings in this array must be tile prototype names.
--- @field entity_filter_mode defines.deconstruction_item.entity_filter_mode The blacklist/whitelist entity filter mode for this deconstruction item.
--- @field tile_filter_mode defines.deconstruction_item.tile_filter_mode The blacklist/whitelist tile filter mode for this deconstruction item.
--- @field tile_selection_mode defines.deconstruction_item.tile_selection_mode The tile selection mode for this deconstruction item.
--- @field trees_and_rocks_only boolean If this deconstruction item is set to allow trees and rocks only.
--- @field entity_filter_count uint The number of entity filters this deconstruction item supports.
--- @field tile_filter_count uint The number of tile filters this deconstruction item supports.
--- @field active_index uint? The active blueprint index for this blueprint book. `nil` if this blueprint book is empty.
--- @field item_number uint? The unique identifier for this item , if any. Note that this ID stays the same no matter where the item is moved to.  Only these types of items have unique IDs: - `"armor"` - `"spidertron-remote"` - `"selection-tool"` - `"copy-paste-tool"` - `"upgrade-item"` - `"deconstruction-item"` - `"blueprint"` - `"blueprint-book"` - `"item-with-entity-data"` - `"item-with-inventory"` - `"item-with-tags"`
--- @field connected_entity LuaEntity? If this item is a spidertron remote that has a spidertron bound to it, it returns the connected spider-vehicle entity.
--- @field entity_label string? If this is an item with entity data, get the stored entity label.
--- @field entity_color Color? If this is an item with entity data, get the stored entity color.
--- @field is_blueprint boolean If this is a blueprint item.
--- @field is_blueprint_book boolean If this is a blueprint book item.
--- @field is_module boolean If this is a module item.
--- @field is_tool boolean If this is a tool item.
--- @field is_mining_tool boolean If this is a mining tool item.
--- @field is_armor boolean If this is an armor item.
--- @field is_repair_tool boolean If this is a repair tool item.
--- @field is_item_with_label boolean If this is an item with label item.
--- @field is_item_with_inventory boolean If this is an item with inventory item.
--- @field is_item_with_entity_data boolean If this is an item with entity data item.
--- @field is_selection_tool boolean If this is a selection tool item.
--- @field is_item_with_tags boolean If this is an item with tags item.
--- @field is_deconstruction_item boolean If this is a deconstruction tool item.
--- @field is_upgrade_item boolean If this is a upgrade item.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaItemStack = { }

----
--- Add ammo to this ammo item.
--- 
--- @param amount float Amount of ammo to add.
----
function LuaItemStack.add_ammo(amount) end

----
--- Add durability to this tool item.
--- 
--- @param amount double Amount of durability to add.
----
function LuaItemStack.add_durability(amount) end

----
--- @class LuaItemStack.build_blueprint_parameters
--- Parameters class for the method `LuaItemStack.build_blueprint`
--- 
--- @field surface SurfaceIdentification Surface to build on
--- @field force ForceIdentification Force to use for the building
--- @field position MapPosition The position to build at
--- @field force_build boolean? When true, anything that can be built is else nothing is built if any one thing can't be built
--- @field direction defines.direction? The direction to use when building
--- @field skip_fog_of_war boolean? If chunks covered by fog-of-war are skipped.
--- @field by_player PlayerIdentification? The player to use if any. If provided [defines.events.on_built_entity](defines.events.on_built_entity) will also be fired on successful entity creation.
--- @field raise_built boolean? If true; [defines.events.script_raised_built](defines.events.script_raised_built) will be fired on successful entity creation. Note: this is ignored if by_player is provided.
----

----
--- 
--- @param args LuaItemStack.build_blueprint_parameters Table containing arguments for the method.
--- @see LuaItemStack.build_blueprint_parameters
--- @return LuaEntity[] Array of created ghosts
----
function LuaItemStack.build_blueprint(args) end

----
--- Would a call to [LuaItemStack::set_stack](LuaItemStack::set_stack) succeed?
--- 
--- @param stack ItemStackIdentification? Stack that would be set, possibly `nil`.
--- @return boolean 
----
function LuaItemStack.can_set_stack(stack) end

----
--- @class LuaItemStack.cancel_deconstruct_area_parameters
--- Parameters class for the method `LuaItemStack.cancel_deconstruct_area`
--- 
--- @field surface SurfaceIdentification Surface to cancel deconstruct on
--- @field force ForceIdentification Force to use for canceling deconstruction
--- @field area BoundingBox The area to deconstruct
--- @field skip_fog_of_war boolean? If chunks covered by fog-of-war are skipped.
--- @field by_player PlayerIdentification? The player to use if any.
----

----
--- Cancel deconstruct the given area with this deconstruction item.
--- 
--- @param args LuaItemStack.cancel_deconstruct_area_parameters Table containing arguments for the method.
--- @see LuaItemStack.cancel_deconstruct_area_parameters
----
function LuaItemStack.cancel_deconstruct_area(args) end

----
--- Clear this item stack.
--- 
----
function LuaItemStack.clear() end

----
--- Clears this blueprint item.
--- 
----
function LuaItemStack.clear_blueprint() end

----
--- Clears all settings/filters on this deconstruction item resetting it to default values.
--- 
----
function LuaItemStack.clear_deconstruction_item() end

----
--- Clears all settings/filters on this upgrade item resetting it to default values.
--- 
----
function LuaItemStack.clear_upgrade_item() end

----
--- @class LuaItemStack.create_blueprint_parameters
--- Parameters class for the method `LuaItemStack.create_blueprint`
--- 
--- @field surface SurfaceIdentification Surface to create from
--- @field force ForceIdentification Force to use for the creation
--- @field area BoundingBox The bounding box
--- @field always_include_tiles boolean? When true, blueprintable tiles are always included in the blueprint. When false they're only included if no entities exist in the setup area.
--- @field include_entities boolean? When true, entities are included in the blueprint. Defaults to true.
--- @field include_modules boolean? When true, modules are included in the blueprint. Defaults to true.
--- @field include_station_names boolean? When true, station names are included in the blueprint. Defaults to false.
--- @field include_trains boolean? When true, trains are included in the blueprint. Defaults to false.
--- @field include_fuel boolean? When true, train fuel is included in the blueprint, Defaults to true.
----

----
--- Sets up this blueprint using the found blueprintable entities/tiles on the surface.
--- 
--- @param args LuaItemStack.create_blueprint_parameters Table containing arguments for the method.
--- @see LuaItemStack.create_blueprint_parameters
--- @return table<uint,LuaEntity> The blueprint entity index to source entity mapping.
----
function LuaItemStack.create_blueprint(args) end

----
--- Creates the equipment grid for this item if it doesn't exist and this is an item-with-entity-data that supports equipment grids.
--- 
--- @return LuaEquipmentGrid 
----
function LuaItemStack.create_grid() end

----
--- @class LuaItemStack.deconstruct_area_parameters
--- Parameters class for the method `LuaItemStack.deconstruct_area`
--- 
--- @field surface SurfaceIdentification Surface to deconstruct on
--- @field force ForceIdentification Force to use for the deconstruction
--- @field area BoundingBox The area to deconstruct
--- @field skip_fog_of_war boolean? If chunks covered by fog-of-war are skipped.
--- @field by_player PlayerIdentification? The player to use if any.
----

----
--- Deconstruct the given area with this deconstruction item.
--- 
--- @param args LuaItemStack.deconstruct_area_parameters Table containing arguments for the method.
--- @see LuaItemStack.deconstruct_area_parameters
----
function LuaItemStack.deconstruct_area(args) end

----
--- Remove ammo from this ammo item.
--- 
--- @param amount float Amount of ammo to remove.
----
function LuaItemStack.drain_ammo(amount) end

----
--- Remove durability from this tool item.
--- 
--- @param amount double Amount of durability to remove.
----
function LuaItemStack.drain_durability(amount) end

----
--- Export a supported item (blueprint, blueprint-book, deconstruction-planner, upgrade-planner, item-with-tags) to a string.
--- 
--- @return string The exported string
----
function LuaItemStack.export_stack() end

----
--- The entities in this blueprint.
--- 
--- @return BlueprintEntity[]? 
----
function LuaItemStack.get_blueprint_entities() end

----
--- Gets the number of entities in this blueprint item.
--- 
--- @return uint 
----
function LuaItemStack.get_blueprint_entity_count() end

----
--- Gets the given tag on the given blueprint entity index in this blueprint item.
--- 
--- @param index uint The entity index.
--- @param tag string The tag to get.
--- @return AnyBasic? 
----
function LuaItemStack.get_blueprint_entity_tag(index, tag) end

----
--- Gets the tags for the given blueprint entity index in this blueprint item.
--- 
--- @param index uint 
--- @return Tags 
----
function LuaItemStack.get_blueprint_entity_tags(index) end

----
--- A list of the tiles in this blueprint.
--- 
--- @return Tile[]? 
----
function LuaItemStack.get_blueprint_tiles() end

----
--- Gets the entity filter at the given index for this deconstruction item.
--- 
--- @param index uint 
--- @return string? 
----
function LuaItemStack.get_entity_filter(index) end

----
--- Access the inner inventory of an item.
--- 
--- @param inventory defines.inventory Index of the inventory to access, which can only be [defines.inventory.item_main](defines.inventory.item_main).
--- @return LuaInventory? `nil` if there is no inventory with the given index.
----
function LuaItemStack.get_inventory(inventory) end

----
--- Gets the filter at the given index for this upgrade item.
--- 
--- @param index uint The index of the mapper to read.
--- @param type string `"from"` or `"to"`.
--- @return UpgradeFilter 
----
function LuaItemStack.get_mapper(index, type) end

----
--- Gets the tag with the given name or returns `nil` if it doesn't exist.
--- 
--- @param tag_name string 
--- @return AnyBasic? 
----
function LuaItemStack.get_tag(tag_name) end

----
--- Gets the tile filter at the given index for this deconstruction item.
--- 
--- @param index uint 
--- @return string? 
----
function LuaItemStack.get_tile_filter(index) end

