require('.classes-1')
require('.classes-2')
require('.classes-3')
require('.classes-4')
require('.classes-5')
require('.classes-6')
require('.classes-7')
require('.classes-8')

----
--- @class LuaRendering.draw_polygon_parameters
--- Parameters class for the method `LuaRendering.draw_polygon`
--- 
--- @field color Color 
--- @field vertices ScriptRenderVertexTarget[] 
--- @field target LuaEntity|MapPosition? Acts like an offset applied to all vertices that are not set to an entity.
--- @field target_offset Vector? Only used if `target` is a LuaEntity.
--- @field orientation RealOrientation? The orientation applied to all vertices. Default is 0.
--- @field orientation_target LuaEntity|MapPosition? If given, the vertices (that are not set to an entity) rotate so that it faces this target. Note that `orientation` is still applied.
--- @field orientation_target_offset Vector? Only used if `orientation_target` is a LuaEntity.
--- @field use_target_orientation boolean? Only used if `orientation_target` is a LuaEntity.
--- @field surface SurfaceIdentification 
--- @field time_to_live uint? In ticks. Defaults to living forever.
--- @field forces ForceIdentification[]? The forces that this object is rendered to. Passing `nil` or an empty table will render it to all forces.
--- @field players PlayerIdentification[]? The players that this object is rendered to. Passing `nil` or an empty table will render it to all players.
--- @field visible boolean? If this is rendered to anyone at all. Defaults to true.
--- @field draw_on_ground boolean? If this should be drawn below sprites and entities.
--- @field only_in_alt_mode boolean? If this should only be rendered in alt mode. Defaults to false.
----

----
--- Create a triangle mesh defined by a triangle strip.
--- 
--- @param args LuaRendering.draw_polygon_parameters Table containing arguments for the method.
--- @see LuaRendering.draw_polygon_parameters
--- @return uint64 Id of the render object
----
function LuaRendering.draw_polygon(args) end

----
--- @class LuaRendering.draw_rectangle_parameters
--- Parameters class for the method `LuaRendering.draw_rectangle`
--- 
--- @field color Color 
--- @field width float? Width of the outline, used only if filled = false. Value is in pixels (32 per tile).
--- @field filled boolean If the rectangle should be filled.
--- @field left_top LuaEntity|MapPosition 
--- @field left_top_offset Vector? Only used if `left_top` is a LuaEntity.
--- @field right_bottom LuaEntity|MapPosition 
--- @field right_bottom_offset Vector? Only used if `right_bottom` is a LuaEntity.
--- @field surface SurfaceIdentification 
--- @field time_to_live uint? In ticks. Defaults to living forever.
--- @field forces ForceIdentification[]? The forces that this object is rendered to. Passing `nil` or an empty table will render it to all forces.
--- @field players PlayerIdentification[]? The players that this object is rendered to. Passing `nil` or an empty table will render it to all players.
--- @field visible boolean? If this is rendered to anyone at all. Defaults to true.
--- @field draw_on_ground boolean? If this should be drawn below sprites and entities.
--- @field only_in_alt_mode boolean? If this should only be rendered in alt mode. Defaults to false.
----

----
--- Create a rectangle.
--- 
--- @param args LuaRendering.draw_rectangle_parameters Table containing arguments for the method.
--- @see LuaRendering.draw_rectangle_parameters
--- @return uint64 Id of the render object
----
function LuaRendering.draw_rectangle(args) end

----
--- @class LuaRendering.draw_sprite_parameters
--- Parameters class for the method `LuaRendering.draw_sprite`
--- 
--- @field sprite SpritePath 
--- @field orientation RealOrientation? The orientation of the sprite. Default is 0.
--- @field x_scale double? Horizontal scale of the sprite. Default is 1.
--- @field y_scale double? Vertical scale of the sprite. Default is 1.
--- @field tint Color? 
--- @field render_layer RenderLayer? 
--- @field orientation_target LuaEntity|MapPosition? If given, the sprite rotates so that it faces this target. Note that `orientation` is still applied to the sprite.
--- @field orientation_target_offset Vector? Only used if `orientation_target` is a LuaEntity.
--- @field use_target_orientation boolean? Only used if `orientation_target` is a LuaEntity.
--- @field oriented_offset Vector? Offsets the center of the sprite if `orientation_target` is given. This offset will rotate together with the sprite.
--- @field target LuaEntity|MapPosition Center of the sprite.
--- @field target_offset Vector? Only used if `target` is a LuaEntity.
--- @field surface SurfaceIdentification 
--- @field time_to_live uint? In ticks. Defaults to living forever.
--- @field forces ForceIdentification[]? The forces that this object is rendered to. Passing `nil` or an empty table will render it to all forces.
--- @field players PlayerIdentification[]? The players that this object is rendered to. Passing `nil` or an empty table will render it to all players.
--- @field visible boolean? If this is rendered to anyone at all. Defaults to true.
--- @field only_in_alt_mode boolean? If this should only be rendered in alt mode. Defaults to false.
----

----
--- Create a sprite.
--- 
--- @param args LuaRendering.draw_sprite_parameters Table containing arguments for the method.
--- @see LuaRendering.draw_sprite_parameters
--- @return uint64 Id of the render object
----
function LuaRendering.draw_sprite(args) end

----
--- @class LuaRendering.draw_text_parameters
--- Parameters class for the method `LuaRendering.draw_text`
--- 
--- @field text LocalisedString The text to display.
--- @field surface SurfaceIdentification 
--- @field target LuaEntity|MapPosition 
--- @field target_offset Vector? Only used if `target` is a LuaEntity.
--- @field color Color 
--- @field scale double? 
--- @field font string? Name of font to use. Defaults to the same font as flying-text.
--- @field time_to_live uint? In ticks. Defaults to living forever.
--- @field forces ForceIdentification[]? The forces that this object is rendered to. Passing `nil` or an empty table will render it to all forces.
--- @field players PlayerIdentification[]? The players that this object is rendered to. Passing `nil` or an empty table will render it to all players.
--- @field visible boolean? If this is rendered to anyone at all. Defaults to true.
--- @field draw_on_ground boolean? If this should be drawn below sprites and entities. Rich text does not support this option.
--- @field orientation RealOrientation? The orientation of the text. Default is 0.
--- @field alignment string? Defaults to "left". Other options are "right" and "center".
--- @field vertical_alignment string? Defaults to "top". Other options are "middle", "baseline" and "bottom".
--- @field scale_with_zoom boolean? Defaults to false. If true, the text scales with player zoom, resulting in it always being the same size on screen, and the size compared to the game world changes.
--- @field only_in_alt_mode boolean? If this should only be rendered in alt mode. Defaults to false.
--- @field use_rich_text boolean? If rich text rendering is enabled. Defaults to false.
----

----
--- Create a text.
--- 
--- @param args LuaRendering.draw_text_parameters Table containing arguments for the method.
--- @see LuaRendering.draw_text_parameters
--- @return uint64 Id of the render object
----
function LuaRendering.draw_text(args) end

----
--- Get the alignment of the text with this id.
--- 
--- @param id uint64 
--- @return string? `nil` if the object is not a text.
----
function LuaRendering.get_alignment(id) end

----
--- Gets an array of all valid object ids.
--- 
--- @param mod_name string? If provided, get only the render objects created by this mod. An empty string (`""`) refers to all objects not belonging to a mod, such as those created using console commands.
--- @return uint64[] 
----
function LuaRendering.get_all_ids(mod_name) end

----
--- Get the angle of the arc with this id.
--- 
--- @param id uint64 
--- @return float? Angle in radian. `nil` if the object is not a arc.
----
function LuaRendering.get_angle(id) end

----
--- Get the animation prototype name of the animation with this id.
--- 
--- @param id uint64 
--- @return string? `nil` if the object is not an animation.
----
function LuaRendering.get_animation(id) end

----
--- Get the animation offset of the animation with this id.
--- 
--- @param id uint64 
--- @return double? Animation offset in frames. `nil` if the object is not an animation.
----
function LuaRendering.get_animation_offset(id) end

----
--- Get the animation speed of the animation with this id.
--- 
--- @param id uint64 
--- @return double? Animation speed in frames per tick. `nil` if the object is not an animation.
----
function LuaRendering.get_animation_speed(id) end

----
--- Get the color or tint of the object with this id.
--- 
--- @param id uint64 
--- @return Color? `nil` if the object does not support color.
----
function LuaRendering.get_color(id) end

----
--- Get the dash length of the line with this id.
--- 
--- @param id uint64 
--- @return double? `nil` if the object is not a line.
----
function LuaRendering.get_dash_length(id) end

----
--- Get whether this is being drawn on the ground, under most entities and sprites.
--- 
--- @param id uint64 
--- @return boolean 
----
function LuaRendering.get_draw_on_ground(id) end

----
--- Get if the circle or rectangle with this id is filled.
--- 
--- @param id uint64 
--- @return boolean? `nil` if the object is not a circle or rectangle.
----
function LuaRendering.get_filled(id) end

----
--- Get the font of the text with this id.
--- 
--- @param id uint64 
--- @return string? `nil` if the object is not a text.
----
function LuaRendering.get_font(id) end

----
--- Get the forces that the object with this id is rendered to or `nil` if visible to all forces.
--- 
--- @param id uint64 
--- @return LuaForce[]? 
----
function LuaRendering.get_forces(id) end

----
--- Get from where the line with this id is drawn.
--- 
--- @param id uint64 
--- @return ScriptRenderTarget? `nil` if this object is not a line.
----
function LuaRendering.get_from(id) end

----
--- Get the length of the gaps in the line with this id.
--- 
--- @param id uint64 
--- @return double? `nil` if the object is not a line.
----
function LuaRendering.get_gap_length(id) end

----
--- Get the intensity of the light with this id.
--- 
--- @param id uint64 
--- @return float? `nil` if the object is not a light.
----
function LuaRendering.get_intensity(id) end

----
--- Get where top left corner of the rectangle with this id is drawn.
--- 
--- @param id uint64 
--- @return ScriptRenderTarget? `nil` if the object is not a rectangle.
----
function LuaRendering.get_left_top(id) end

----
--- Get the radius of the outer edge of the arc with this id.
--- 
--- @param id uint64 
--- @return double? `nil` if the object is not a arc.
----
function LuaRendering.get_max_radius(id) end

----
--- Get the radius of the inner edge of the arc with this id.
--- 
--- @param id uint64 
--- @return double? `nil` if the object is not a arc.
----
function LuaRendering.get_min_radius(id) end

----
--- Get the minimum darkness at which the light with this id is rendered.
--- 
--- @param id uint64 
--- @return float? `nil` if the object is not a light.
----
function LuaRendering.get_minimum_darkness(id) end

----
--- Get whether this is only rendered in alt-mode.
--- 
--- @param id uint64 
--- @return boolean 
----
function LuaRendering.get_only_in_alt_mode(id) end

----
--- Get the orientation of the object with this id.
--- 
--- @param id uint64 
--- @return RealOrientation? `nil` if the object is not a text, polygon, sprite, light or animation.
----
function LuaRendering.get_orientation(id) end

----
--- The object rotates so that it faces this target. Note that `orientation` is still applied to the object. Get the orientation_target of the object with this id.
--- 
--- @param id uint64 
--- @return ScriptRenderTarget? `nil` if no target or if this object is not a polygon, sprite, or animation.
----
function LuaRendering.get_orientation_target(id) end

----
--- Get if the light with this id is rendered has the same orientation as the target entity. Note that `orientation` is still applied to the sprite.
--- 
--- @param id uint64 
--- @return boolean? `nil` if the object is not a light.
----
function LuaRendering.get_oriented(id) end

----
--- Offsets the center of the sprite or animation if `orientation_target` is given. This offset will rotate together with the sprite or animation. Get the oriented_offset of the sprite or animation with this id.
--- 
--- @param id uint64 
--- @return Vector? `nil` if this object is not a sprite or animation.
----
function LuaRendering.get_oriented_offset(id) end

----
--- Get the players that the object with this id is rendered to or `nil` if visible to all players.
--- 
--- @param id uint64 
--- @return LuaPlayer[]? 
----
function LuaRendering.get_players(id) end

----
--- Get the radius of the circle with this id.
--- 
--- @param id uint64 
--- @return double? `nil` if the object is not a circle.
----
function LuaRendering.get_radius(id) end

----
--- Get the render layer of the sprite or animation with this id.
--- 
--- @param id uint64 
--- @return RenderLayer? `nil` if the object is not a sprite or animation.
----
function LuaRendering.get_render_layer(id) end

----
--- Get where bottom right corner of the rectangle with this id is drawn.
--- 
--- @param id uint64 
--- @return ScriptRenderTarget? `nil` if the object is not a rectangle.
----
function LuaRendering.get_right_bottom(id) end

----
--- Get the scale of the text or light with this id.
--- 
--- @param id uint64 
--- @return double? `nil` if the object is not a text or light.
----
function LuaRendering.get_scale(id) end

----
--- Get if the text with this id scales with player zoom.
--- 
--- @param id uint64 
--- @return boolean? `nil` if the object is not a text.
----
function LuaRendering.get_scale_with_zoom(id) end

----
--- Get the sprite of the sprite or light with this id.
--- 
--- @param id uint64 
--- @return SpritePath? `nil` if the object is not a sprite or light.
----
function LuaRendering.get_sprite(id) end

----
--- Get where the arc with this id starts.
--- 
--- @param id uint64 
--- @return float? Angle in radian. `nil` if the object is not a arc.
----
function LuaRendering.get_start_angle(id) end

----
--- The surface the object with this id is rendered on.
--- 
--- @param id uint64 
--- @return LuaSurface 
----
function LuaRendering.get_surface(id) end

----
--- Get where the object with this id is drawn.
--- 
--- @param id uint64 
--- @return ScriptRenderTarget? `nil` if the object does not support target.
----
function LuaRendering.get_target(id) end

----
--- Get the text that is displayed by the text with this id.
--- 
--- @param id uint64 
--- @return LocalisedString? `nil` if the object is not a text.
----
function LuaRendering.get_text(id) end

----
--- Get the time to live of the object with this id. This will be 0 if the object does not expire.
--- 
--- @param id uint64 
--- @return uint 
----
function LuaRendering.get_time_to_live(id) end

----
--- Get where the line with this id is drawn to.
--- 
--- @param id uint64 
--- @return ScriptRenderTarget? `nil` if the object is not a line.
----
function LuaRendering.get_to(id) end

----
--- Gets the type of the given object. The types are "text", "line", "circle", "rectangle", "arc", "polygon", "sprite", "light" and "animation".
--- 
--- @param id uint64 
--- @return string 
----
function LuaRendering.get_type(id) end

----
--- Get if the text with this id parses rich text tags.
--- 
--- @param id uint64 
--- @return boolean? `nil` if the object is not a text.
----
function LuaRendering.get_use_rich_text(id) end

----
--- Get whether this uses the target orientation.
--- 
--- @param id uint64 
--- @return boolean `nil` if the object is not a sprite, polygon, or animation.
----
function LuaRendering.get_use_target_orientation(id) end

----
--- Get the vertical alignment of the text with this id.
--- 
--- @param id uint64 
--- @return string? `nil` if the object is not a text.
----
function LuaRendering.get_vertical_alignment(id) end

----
--- Get the vertices of the polygon with this id.
--- 
--- @param id uint64 
--- @return ScriptRenderTarget[]? `nil` if the object is not a polygon.
----
function LuaRendering.get_vertices(id) end

----
--- Get whether this is rendered to anyone at all.
--- 
--- @param id uint64 
--- @return boolean 
----
function LuaRendering.get_visible(id) end

----
--- Get the width of the object with this id. Value is in pixels (32 per tile).
--- 
--- @param id uint64 
--- @return float? `nil` if the object does not support width.
----
function LuaRendering.get_width(id) end

----
--- Get the horizontal scale of the sprite or animation with this id.
--- 
--- @param id uint64 
--- @return double? `nil` if the object is not a sprite or animation.
----
function LuaRendering.get_x_scale(id) end

----
--- Get the vertical scale of the sprite or animation with this id.
--- 
--- @param id uint64 
--- @return double? `nil` if the object is not a sprite or animation.
----
function LuaRendering.get_y_scale(id) end

----
--- Does a font with this name exist?
--- 
--- @param font_name string 
--- @return boolean 
----
function LuaRendering.is_font_valid(font_name) end

----
--- Does a valid object with this id exist?
--- 
--- @param id uint64 
--- @return boolean 
----
function LuaRendering.is_valid(id) end

----
--- Set the alignment of the text with this id. Does nothing if this object is not a text.
--- 
--- @param id uint64 
--- @param alignment string "left", "right" or "center".
----
function LuaRendering.set_alignment(id, alignment) end

----
--- Set the angle of the arc with this id. Does nothing if this object is not a arc.
--- 
--- @param id uint64 
--- @param angle float angle in radian
----
function LuaRendering.set_angle(id, angle) end

----
--- Set the animation prototype name of the animation with this id. Does nothing if this object is not an animation.
--- 
--- @param id uint64 
--- @param animation string 
----
function LuaRendering.set_animation(id, animation) end

----
--- Set the animation offset of the animation with this id. Does nothing if this object is not an animation.
--- 
--- @param id uint64 
--- @param animation_offset double Animation offset in frames.
----
function LuaRendering.set_animation_offset(id, animation_offset) end

----
--- Set the animation speed of the animation with this id. Does nothing if this object is not an animation.
--- 
--- @param id uint64 
--- @param animation_speed double Animation speed in frames per tick.
----
function LuaRendering.set_animation_speed(id, animation_speed) end

----
--- Set the color or tint of the object with this id. Does nothing if this object does not support color.
--- 
--- @param id uint64 
--- @param color Color 
----
function LuaRendering.set_color(id, color) end

----
--- Set the corners of the rectangle with this id. Does nothing if this object is not a rectangle.
--- 
--- @param id uint64 
--- @param left_top LuaEntity|MapPosition 
--- @param left_top_offset Vector 
--- @param right_bottom LuaEntity|MapPosition 
--- @param right_bottom_offset Vector 
----
function LuaRendering.set_corners(id, left_top, left_top_offset, right_bottom, right_bottom_offset) end

----
--- Set the dash length of the line with this id. Does nothing if this object is not a line.
--- 
--- @param id uint64 
--- @param dash_length double 
----
function LuaRendering.set_dash_length(id, dash_length) end

----
--- Set the length of the dashes and the length of the gaps in the line with this id. Does nothing if this object is not a line.
--- 
--- @param id uint64 
--- @param dash_length double 
--- @param gap_length double 
----
function LuaRendering.set_dashes(id, dash_length, gap_length) end

----
--- Set whether this is being drawn on the ground, under most entities and sprites.
--- 
--- @param id uint64 
--- @param draw_on_ground boolean 
----
function LuaRendering.set_draw_on_ground(id, draw_on_ground) end

----
--- Set if the circle or rectangle with this id is filled. Does nothing if this object is not a circle or rectangle.
--- 
--- @param id uint64 
--- @param filled boolean 
----
function LuaRendering.set_filled(id, filled) end

----
--- Set the font of the text with this id. Does nothing if this object is not a text.
--- 
--- @param id uint64 
--- @param font string 
----
function LuaRendering.set_font(id, font) end

----
--- Set the forces that the object with this id is rendered to.
--- 
--- @param id uint64 
--- @param forces ForceIdentification[] Providing an empty array will set the object to be visible to all forces.
----
function LuaRendering.set_forces(id, forces) end

----
--- Set from where the line with this id is drawn. Does nothing if the object is not a line.
--- 
--- @param id uint64 
--- @param from LuaEntity|MapPosition 
--- @param from_offset Vector? 
----
function LuaRendering.set_from(id, from, from_offset) end

----
--- Set the length of the gaps in the line with this id. Does nothing if this object is not a line.
--- 
--- @param id uint64 
--- @param gap_length double 
----
function LuaRendering.set_gap_length(id, gap_length) end

----
--- Set the intensity of the light with this id. Does nothing if this object is not a light.
--- 
--- @param id uint64 
--- @param intensity float 
----
function LuaRendering.set_intensity(id, intensity) end

----
--- Set where top left corner of the rectangle with this id is drawn. Does nothing if this object is not a rectangle.
--- 
--- @param id uint64 
--- @param left_top LuaEntity|MapPosition 
--- @param left_top_offset Vector? 
----
function LuaRendering.set_left_top(id, left_top, left_top_offset) end

----
--- Set the radius of the outer edge of the arc with this id. Does nothing if this object is not a arc.
--- 
--- @param id uint64 
--- @param max_radius double 
----
function LuaRendering.set_max_radius(id, max_radius) end

----
--- Set the radius of the inner edge of the arc with this id. Does nothing if this object is not a arc.
--- 
--- @param id uint64 
--- @param min_radius double 
----
function LuaRendering.set_min_radius(id, min_radius) end

----
--- Set the minimum darkness at which the light with this id is rendered. Does nothing if this object is not a light.
--- 
--- @param id uint64 
--- @param minimum_darkness float 
----
function LuaRendering.set_minimum_darkness(id, minimum_darkness) end

----
--- Set whether this is only rendered in alt-mode.
--- 
--- @param id uint64 
--- @param only_in_alt_mode boolean 
----
function LuaRendering.set_only_in_alt_mode(id, only_in_alt_mode) end

----
--- Set the orientation of the object with this id. Does nothing if this object is not a text, polygon, sprite, light or animation.
--- 
--- @param id uint64 
--- @param orientation RealOrientation 
----
function LuaRendering.set_orientation(id, orientation) end

----
--- The object rotates so that it faces this target. Note that `orientation` is still applied to the object. Set the orientation_target of the object with this id. Does nothing if this object is not a polygon, sprite, or animation. Set to `nil` if the object should not have an orientation_target.
--- 
--- @param id uint64 
--- @param orientation_target LuaEntity|MapPosition 
--- @param orientation_target_offset Vector? 
----
function LuaRendering.set_orientation_target(id, orientation_target, orientation_target_offset) end

----
--- Set if the light with this id is rendered has the same orientation as the target entity. Does nothing if this object is not a light. Note that `orientation` is still applied to the sprite.
--- 
--- @param id uint64 
--- @param oriented boolean 
----
function LuaRendering.set_oriented(id, oriented) end

----
--- Offsets the center of the sprite or animation if `orientation_target` is given. This offset will rotate together with the sprite or animation. Set the oriented_offset of the sprite or animation with this id. Does nothing if this object is not a sprite or animation.
--- 
--- @param id uint64 
--- @param oriented_offset Vector 
----
function LuaRendering.set_oriented_offset(id, oriented_offset) end

----
--- Set the players that the object with this id is rendered to.
--- 
--- @param id uint64 
--- @param players PlayerIdentification[] Providing an empty array will set the object to be visible to all players.
----
function LuaRendering.set_players(id, players) end

----
--- Set the radius of the circle with this id. Does nothing if this object is not a circle.
--- 
--- @param id uint64 
--- @param radius double 
----
function LuaRendering.set_radius(id, radius) end

----
--- Set the render layer of the sprite or animation with this id. Does nothing if this object is not a sprite or animation.
--- 
--- @param id uint64 
--- @param render_layer RenderLayer 
----
function LuaRendering.set_render_layer(id, render_layer) end

----
--- Set where top bottom right of the rectangle with this id is drawn. Does nothing if this object is not a rectangle.
--- 
--- @param id uint64 
--- @param right_bottom LuaEntity|MapPosition 
--- @param right_bottom_offset Vector? 
----
function LuaRendering.set_right_bottom(id, right_bottom, right_bottom_offset) end

----
--- Set the scale of the text or light with this id. Does nothing if this object is not a text or light.
--- 
--- @param id uint64 
--- @param scale double 
----
function LuaRendering.set_scale(id, scale) end

----
--- Set if the text with this id scales with player zoom, resulting in it always being the same size on screen, and the size compared to the game world changes. Does nothing if this object is not a text.
--- 
--- @param id uint64 
--- @param scale_with_zoom boolean 
----
function LuaRendering.set_scale_with_zoom(id, scale_with_zoom) end

----
--- Set the sprite of the sprite or light with this id. Does nothing if this object is not a sprite or light.
--- 
--- @param id uint64 
--- @param sprite SpritePath 
----
function LuaRendering.set_sprite(id, sprite) end

----
--- Set where the arc with this id starts. Does nothing if this object is not a arc.
--- 
--- @param id uint64 
--- @param start_angle float angle in radian
----
function LuaRendering.set_start_angle(id, start_angle) end

----
--- Set where the object with this id is drawn. Does nothing if this object does not support target.
--- 
--- @param id uint64 
--- @param target LuaEntity|MapPosition 
--- @param target_offset Vector? 
----
function LuaRendering.set_target(id, target, target_offset) end

----
--- Set the text that is displayed by the text with this id. Does nothing if this object is not a text.
--- 
--- @param id uint64 
--- @param text LocalisedString 
----
function LuaRendering.set_text(id, text) end

----
--- Set the time to live of the object with this id. Set to 0 if the object should not expire.
--- 
--- @param id uint64 
--- @param time_to_live uint 
----
function LuaRendering.set_time_to_live(id, time_to_live) end

----
--- Set where the line with this id is drawn to. Does nothing if this object is not a line.
--- 
--- @param id uint64 
--- @param to LuaEntity|MapPosition 
--- @param to_offset Vector? 
----
function LuaRendering.set_to(id, to, to_offset) end

----
--- Set if the text with this id parses rich text tags.
--- 
--- @param id uint64 
--- @param use_rich_text boolean 
----
function LuaRendering.set_use_rich_text(id, use_rich_text) end

----
--- Set whether this uses the target orientation.
--- 
--- @param id uint64 
--- @param use_target_orientation boolean 
----
function LuaRendering.set_use_target_orientation(id, use_target_orientation) end

----
--- Set the vertical alignment of the text with this id. Does nothing if this object is not a text.
--- 
--- @param id uint64 
--- @param alignment string "top", "middle", "baseline" or "bottom"
----
function LuaRendering.set_vertical_alignment(id, alignment) end

----
--- Set the vertices of the polygon with this id. Does nothing if this object is not a polygon.
--- 
--- @param id uint64 
--- @param vertices ScriptRenderVertexTarget[] 
----
function LuaRendering.set_vertices(id, vertices) end

----
--- Set whether this is rendered to anyone at all.
--- 
--- @param id uint64 
--- @param visible boolean 
----
function LuaRendering.set_visible(id, visible) end

----
--- Set the width of the object with this id. Does nothing if this object does not support width. Value is in pixels (32 per tile).
--- 
--- @param id uint64 
--- @param width float 
----
function LuaRendering.set_width(id, width) end

----
--- Set the horizontal scale of the sprite or animation with this id. Does nothing if this object is not a sprite or animation.
--- 
--- @param id uint64 
--- @param x_scale double 
----
function LuaRendering.set_x_scale(id, x_scale) end

----
--- Set the vertical scale of the sprite or animation with this id. Does nothing if this object is not a sprite or animation.
--- 
--- @param id uint64 
--- @param y_scale double 
----
function LuaRendering.set_y_scale(id, y_scale) end


----
--- @class LuaResourceCategoryPrototype
--- Prototype of a resource category.
--- 
--- @field name string Name of this prototype.
--- @field order string The string used to alphabetically sort these prototypes. It is a simple string that has no additional semantic meaning.
--- @field localised_name LocalisedString 
--- @field localised_description LocalisedString 
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaResourceCategoryPrototype = { }

