
----
--- @class AchievementPrototypeFilter
--- Depending on the value of `filter`, the table may take additional fields. `filter` may be one of the following:
--- 
--- @field filter string The condition to filter on. One of `"allowed-without-fight"`, `"type"`.
--- @field mode string? How to combine this with the previous filter. Must be `"or"` or `"and"`. Defaults to `"or"`. When evaluating the filters, `"and"` has higher precedence than `"or"`.
--- @field invert boolean? Inverts the condition. Default is `false`.
----

----
--- @class AdvancedMapGenSettings
--- 
--- @field pollution PollutionMapSettings 
--- @field enemy_evolution EnemyEvolutionMapSettings 
--- @field enemy_expansion EnemyExpansionMapSettings 
--- @field difficulty_settings DifficultySettings 
----

----
--- @class Alert
--- 
--- @field tick uint The tick this alert was created.
--- @field target LuaEntity? 
--- @field prototype LuaEntityPrototype? 
--- @field position MapPosition? 
--- @field icon SignalID? The SignalID used for a custom alert. Only present for custom alerts.
--- @field message LocalisedString? The message for a custom alert. Only present for custom alerts.
----

----
--- @alias Alignment string
--- A [string](string) that specifies where a GUI element should be.
----

----
--- @class AmmoType
--- 
--- @field action TriggerItem[]? 
--- @field target_type string One of `"entity"` (fires at an entity), `"position"` (fires directly at a position), or `"direction"` (fires in a direction).
--- @field clamp_position boolean? When `true`, the gun will be able to shoot even when the target is out of range. Only applies when `target_type` is `position`. The gun will fire at the maximum range in the direction of the target position. Defaults to `false`.
--- @field category string Ammo category of this ammo.
--- @field energy_consumption double? Energy consumption of a single shot, if applicable. Defaults to `0`.
--- @field range_modifier double? 
--- @field cooldown_modifier double? 
--- @field consumption_modifier double? 
----

----
--- @alias Any boolean|table|string|number|LuaObject
--- Any basic type (string, number, boolean), table, or LuaObject.
----

----
--- @alias AnyBasic number|boolean|table|string
--- Any basic type (string, number, boolean) or table.
----

----
--- @class ArithmeticCombinatorParameters
--- 
--- @field first_signal SignalID? First signal to use in an operation. If not specified, the second argument will be the value of `first_constant`.
--- @field second_signal SignalID? Second signal to use in an operation. If not specified, the second argument will be the value of `second_constant`.
--- @field first_constant int? Constant to use as the first argument of the operation. Has no effect when `first_signal` is set. Defaults to `0`.
--- @field second_constant int? Constant to use as the second argument of the operation. Has no effect when `second_signal` is set. Defaults to `0`.
--- @field operation string? Must be one of `"*"`, `"/"`, `"+"`, `"-"`, `"%"`, `"^"`, `"<<"`, `">>"`, `"AND"`, `"OR"`, `"XOR"`. When not specified, defaults to `"*"`.
--- @field output_signal SignalID? Specifies the signal to output.
----

----
--- @class AttackParameterFluid
--- 
--- @field type string Name of the [LuaFluidPrototype](LuaFluidPrototype).
--- @field damage_modifier double Multiplier applied to the damage of an attack.
----

----
--- @class AttackParameters
--- 
--- @field type string The type of AttackParameter. One of `'projectile'`, `'stream'` or `'beam'`.
--- @field range float Maximum range of attack.
--- @field min_range float Minimum range of attack. Used with flamethrower turrets to prevent self-immolation.
--- @field range_mode string Defines how the range is determined. Either `'center-to-center'` or `'bounding-box-to-bounding-box'`.
--- @field fire_penalty float When searching for the nearest enemy to attack, `fire_penalty` is added to the enemy's distance if they are on fire.
--- @field rotate_penalty float When searching for an enemy to attack, a higher `rotate_penalty` will discourage targeting enemies that would take longer to turn to face.
--- @field health_penalty float When searching for an enemy to attack, a higher `health_penalty` will discourage targeting enemies with high health. A negative penalty will do the opposite.
--- @field min_attack_distance float If less than `range`, the entity will choose a random distance between `range` and `min_attack_distance` and attack from that distance. Used for spitters.
--- @field turn_range float The arc that the entity can attack in as a fraction of a circle. A value of `1` means the full 360 degrees.
--- @field damage_modifier float Multiplier applied to the damage of an attack.
--- @field ammo_consumption_modifier float Multiplier applied to the ammo consumption of an attack.
--- @field cooldown float Minimum amount of ticks between shots. If this is less than `1`, multiple shots can be performed per tick.
--- @field warmup uint Number of ticks it takes for the weapon to actually shoot after it has been ordered to do so.
--- @field movement_slow_down_factor double 
--- @field movement_slow_down_cooldown float 
--- @field ammo_type AmmoType? 
--- @field ammo_categories string[]? List of the names of compatible [LuaAmmoCategoryPrototypes](LuaAmmoCategoryPrototype).
----

----
--- @class AutoplaceControl
--- 
--- @field frequency MapGenSize For things that are placed as spots such as ores and enemy bases, frequency is generally proportional to number of spots placed per unit area. For continuous features such as forests, frequency is how compressed the probability function is over distance, i.e. the inverse of 'scale' (similar to terrain_segmentation). When the [LuaAutoplaceControlPrototype](LuaAutoplaceControlPrototype) is of the category `"terrain"`, then scale is shown in the map generator GUI instead of frequency.
--- @field size MapGenSize For things that are placed as spots, size is proportional to the area of each spot. For continuous features, size affects how much of the map is covered with the thing, and is called 'coverage' in the GUI.
--- @field richness MapGenSize Has different effects for different things, but generally affects the 'health' or density of a thing that is placed without affecting where it is placed. For trees, richness affects tree health. For ores, richness multiplies the amount of ore at any given tile in a patch. Metadata about autoplace controls (such as whether or not 'richness' does anything for them) can be found in the [LuaAutoplaceControlPrototype](LuaAutoplaceControlPrototype) by looking up `game.autoplace_control_prototypes[(control prototype name)]`, e.g. `game.autoplace_control_prototypes["enemy-base"].richness` is false, because enemy base autoplacement doesn't use richness.
----

----
--- @class AutoplaceSettings
--- 
--- @field treat_missing_as_default boolean Whether missing autoplace names for this type should be default enabled.
--- @field settings table<string,AutoplaceControl> 
----

----
--- @class AutoplaceSpecification
--- Specifies how probability and richness are calculated when placing something on the map. Can be specified either using `probability_expression` and `richness_expression` or by using all the other fields.
--- 
--- @field probability_expression NoiseExpression 
--- @field richness_expression NoiseExpression 
--- @field coverage double 
--- @field sharpness double 
--- @field max_probability double 
--- @field placement_density uint 
--- @field richness_base double 
--- @field richness_multiplier double 
--- @field richness_multiplier_distance_bonus double 
--- @field starting_area_size uint 
--- @field order string 
--- @field default_enabled boolean 
--- @field peaks AutoplaceSpecificationPeak[]? 
--- @field control string? Control prototype name.
--- @field tile_restriction AutoplaceSpecificationRestriction[]? 
--- @field force string 
--- @field random_probability_penalty double 
----

----
--- @class AutoplaceSpecificationPeak
--- 
--- @field influence double 
--- @field max_influence double 
--- @field min_influence double 
--- @field richness_influence double 
--- @field noisePersistence double 
--- @field noise_layer string? Prototype name of the noise layer.
--- @field noise_octaves_difference double 
--- @field water_optimal double 
--- @field water_range double 
--- @field water_max_range double 
--- @field water_top_property_limit double 
--- @field elevation_optimal double 
--- @field elevation_range double 
--- @field elevation_max_range double 
--- @field elevation_top_property_limit double 
--- @field temperature_optimal double 
--- @field temperature_range double 
--- @field temperature_max_range double 
--- @field temperature_top_property_limit double 
--- @field starting_area_weight_optimal double 
--- @field starting_area_weight_range double 
--- @field starting_area_weight_max_range double 
--- @field starting_area_weight_top_property_limit double 
--- @field tier_from_start_optimal double 
--- @field tier_from_start_range double 
--- @field tier_from_start_max_range double 
--- @field tier_from_start_top_property_limit double 
--- @field distance_optimal double 
--- @field distance_range double 
--- @field distance_max_range double 
--- @field distance_top_property_limit double 
--- @field aux_optimal double 
--- @field aux_range double 
--- @field aux_max_range double 
--- @field aux_top_property_limit double 
----

----
--- @class AutoplaceSpecificationRestriction
--- 
--- @field first string? Tile prototype name
--- @field second string? Second prototype name
----

----
--- @class BeamTarget
--- 
--- @field entity LuaEntity? The target entity.
--- @field position MapPosition? The target position.
----

----
--- @class BlueprintEntity
--- The representation of an entity inside of a blueprint. It has at least these fields, but can contain additional ones depending on the kind of entity.
--- 
--- @field entity_number uint The entity's unique identifier in the blueprint.
--- @field name string The prototype name of the entity.
--- @field position MapPosition The position of the entity.
--- @field direction defines.direction? The direction the entity is facing. Only present for entities that can face in different directions and when the entity is not facing north.
--- @field tags Tags? The entity tags of the entity, if there are any. Only relevant for entity ghosts.
--- @field items table<string,uint>? The items that the entity will request when revived, if there are any. It's a mapping of prototype names to amounts. Only relevant for entity ghosts.
--- @field connections BlueprintCircuitConnection? The circuit network connections of the entity, if there are any. Only relevant for entities that support circuit connections.
--- @field control_behavior BlueprintControlBehavior? The control behavior of the entity, if it has one. The format of the control behavior depends on the entity's type. Only relevant for entities that support control behaviors.
--- @field schedule TrainScheduleRecord[]? The schedule of the entity, if it has one. Only relevant for locomotives.
----

----
--- @class BlueprintSignalIcon
--- 
--- @field signal SignalID The icon to use. It can be any item icon as well as any virtual signal icon.
--- @field index uint Index of the icon in the blueprint icons slots. Has to be an integer in the range [1, 4].
----

----
--- @alias BoundingBox { left_top: MapPosition, orientation: RealOrientation, right_bottom: MapPosition }
--- Two positions, specifying the top-left and bottom-right corner of the box respectively. Like with [MapPosition](MapPosition), the names of the members may be omitted. When read from the game, the third member `orientation` is present if it is non-zero, however it is ignored when provided to the game.
----

----
--- @class CapsuleAction
--- 
--- @field type string One of `"throw"`, `"equipment-remote"`, `"use-on-self"`, `"artillery-remote"`, `"destroy-cliffs"`.
----

----
--- @class ChartTagSpec
--- 
--- @field position MapPosition 
--- @field icon SignalID? 
--- @field text string? 
--- @field last_user PlayerIdentification? 
----

----
--- @alias ChunkPosition { x: int, y: int }
--- Coordinates of a chunk in a [LuaSurface](LuaSurface) where each integer `x`/`y` represents a different chunk. This uses the same format as [MapPosition](MapPosition), meaning it can be specified either with or without explicit keys. A [MapPosition](MapPosition) can be translated to a ChunkPosition by dividing the `x`/`y` values by 32.
----

----
--- @class ChunkPositionAndArea
--- A [ChunkPosition](ChunkPosition) with an added bounding box for the area of the chunk.
--- 
--- @field x int 
--- @field y int 
--- @field area BoundingBox 
----

----
--- @class CircuitCondition
--- 
--- @field comparator ComparatorString? Specifies how the inputs should be compared. If not specified, defaults to `"<"`.
--- @field first_signal SignalID? Defaults to blank
--- @field second_signal SignalID? What to compare `first_signal` to. If not specified, `first_signal` will be compared to `constant`.
--- @field constant int? Constant to compare `first_signal` to. Has no effect when `second_signal` is set. When neither `second_signal` nor `constant` are specified, the effect is as though `constant` were specified with the value `0`.
----

----
--- @class CircuitConditionDefinition
--- 
--- @field condition CircuitCondition 
--- @field fulfilled boolean? Whether the condition is currently fulfilled
----

----
--- @class CircuitConnectionDefinition
--- 
--- @field wire defines.wire_type Wire color, either [defines.wire_type.red](defines.wire_type.red) or [defines.wire_type.green](defines.wire_type.green).
--- @field target_entity LuaEntity 
--- @field source_circuit_id defines.circuit_connector_id 
--- @field target_circuit_id defines.circuit_connector_id 
----

----
--- @class CircularParticleCreationSpecification
--- 
--- @field name string Name of the [LuaEntityPrototype](LuaEntityPrototype)
--- @field direction float 
--- @field direction_deviation float 
--- @field speed float 
--- @field speed_deviation float 
--- @field starting_frame_speed float 
--- @field starting_frame_speed_deviation float 
--- @field height float 
--- @field height_deviation float 
--- @field vertical_speed float 
--- @field vertical_speed_deviation float 
--- @field center Vector This vector is a table with `x` and `y` keys instead of an array.
--- @field creation_distance double 
--- @field creation_distance_orientation double 
--- @field use_source_position boolean 
----

----
--- @class CircularProjectileCreationSpecification
--- 
--- @field _ RealOrientation 
--- @field _ Vector 
----

----
--- @alias CliffOrientation string
----

----
--- @class CliffPlacementSettings
--- 
--- @field name string Name of the cliff prototype.
--- @field cliff_elevation_0 float Elevation at which the first row of cliffs is placed. The default is `10`, and this cannot be set from the map generation GUI.
--- @field cliff_elevation_interval float Elevation difference between successive rows of cliffs. This is inversely proportional to 'frequency' in the map generation GUI. Specifically, when set from the GUI the value is `40 / frequency`.
--- @field richness MapGenSize Corresponds to 'continuity' in the GUI. This value is not used directly, but is used by the 'cliffiness' noise expression, which in combination with elevation and the two cliff elevation properties drives cliff placement (cliffs are placed when elevation crosses the elevation contours defined by `cliff_elevation_0` and `cliff_elevation_interval` when 'cliffiness' is greater than `0.5`). The default 'cliffiness' expression interprets this value such that larger values result in longer unbroken walls of cliffs, and smaller values (between `0` and `1`) result in larger gaps in cliff walls.
----

----
--- @alias CollisionMask table<CollisionMaskLayer,bool>
--- A set of flags. Active flags are in the dictionary as `true`, while inactive flags aren't present at all.
----

----
--- @alias CollisionMaskLayer string
--- A [string](string) specifying a collision mask layer.
--- 
--- In addition to the listed layers, there is `"layer-13"` through `"layer-55"`. These layers are currently unused by the game but may change. If a mod is going to use one of the unused layers it's recommended to start at the higher layers because the base game will take from the lower ones.
----

----
--- @alias CollisionMaskWithFlags table<CollisionMaskLayer|string,bool>
--- A [CollisionMask](CollisionMask) which also includes any flags this mask has.
----

----
--- @alias Color { a: float, b: float, g: float, r: float }
--- Red, green, blue and alpha values, all in range [0, 1] or all in range [0, 255] if any value is > 1. All values here are optional. Color channels default to `0`, the alpha channel defaults to `1`.
--- 
--- Similar to [MapPosition](MapPosition), Color allows the short-hand notation of passing an array of exactly 3 or 4 numbers. The game usually expects colors to be in pre-multiplied form (color channels are pre-multiplied by alpha).
----

----
--- @alias ColorModifier { a: float, b: float, g: float, r: float }
--- Same as [Color](Color), but red, green, blue and alpha values can be any floating point number, without any special handling of the range [1, 255].
----

----
--- @class Command
--- Commands can be given to enemies and unit groups.
--- 
--- @field type defines.command Type of command. The remaining fields depend on the value of this field.
----

----
--- @alias ComparatorString string
--- A string that specifies how the inputs should be compared
----

----
--- @class ConfigurationChangedData
--- 
--- @field old_version string? Old version of the map. Present only when loading map version other than the current version.
--- @field new_version string? New version of the map. Present only when loading map version other than the current version.
--- @field mod_changes table<string,ModChangeData> Dictionary of mod changes. It is indexed by mod name.
--- @field mod_startup_settings_changed boolean `true` when mod startup settings have changed since the last time this save was loaded.
--- @field migration_applied boolean `true` when mod prototype migrations have been applied since the last time this save was loaded.
----

----
--- @class ConstantCombinatorParameters
--- 
--- @field signal SignalID Signal to emit.
--- @field count int Value of the signal to emit.
--- @field index uint Index of the constant combinator's slot to set this signal to.
----

----
--- @class CraftingQueueItem
--- 
--- @field index uint The index of the item in the crafting queue.
--- @field recipe string The recipe being crafted.
--- @field count uint The amount of items being crafted.
--- @field prerequisite boolean The item is a prerequisite for another item in the queue.
----

----
--- @alias CursorBoxRenderType string
----

----
--- @class CustomCommandData
--- 
--- @field name string The name of the command.
--- @field tick uint The tick the command was used in.
--- @field player_index uint? The player who issued the command, or `nil` if it was issued from the server console.
--- @field parameter string? The parameter passed after the command, if there is one.
----

----
--- @class CutsceneWaypoint
--- 
--- @field position MapPosition? Position to pan the camera to.
--- @field target LuaEntity|LuaUnitGroup? Entity or unit group to pan the camera to.
--- @field transition_time uint How many ticks it will take to reach this waypoint from the previous one.
--- @field time_to_wait uint Time in ticks to wait before moving to the next waypoint.
--- @field zoom double? Zoom level to be set when the waypoint is reached. When not specified, the previous waypoint's zoom is used.
----

----
--- @class DeciderCombinatorParameters
--- 
--- @field first_signal SignalID? Defaults to blank.
--- @field second_signal SignalID? Second signal to use in an operation, if any. If this is not specified, the second argument to a decider combinator's operation is assumed to be the value of `constant`.
--- @field constant uint? Constant to use as the second argument of operation. Defaults to `0`.
--- @field comparator ComparatorString? Specifies how the inputs should be compared. If not specified, defaults to `"<"`.
--- @field output_signal SignalID? Defaults to blank.
--- @field copy_count_from_input boolean? Defaults to `true`. When `false`, will output a value of `1` for the given `output_signal`.
----

----
--- @class Decorative
--- 
--- @field name string The name of the decorative prototype.
--- @field position TilePosition 
--- @field amount uint8 
----

----
--- @class DecorativePrototypeFilter
--- Depending on the value of `filter`, the table may take additional fields. `filter` may be one of the following:
--- 
--- @field filter string The condition to filter on. One of `"decal"`, `"autoplace"`, `"collision-mask"`.
--- @field mode string? How to combine this with the previous filter. Must be `"or"` or `"and"`. Defaults to `"or"`. When evaluating the filters, `"and"` has higher precedence than `"or"`.
--- @field invert boolean? Inverts the condition. Default is `false`.
----

----
--- @class DecorativeResult
--- 
--- @field position TilePosition 
--- @field decorative LuaDecorativePrototype 
--- @field amount uint 
----

----
--- @class DifficultySettings
--- Technology and recipe difficulty settings. Updating any of the attributes will immediately take effect in the game engine.
--- 
--- @field recipe_difficulty defines.difficulty_settings.recipe_difficulty 
--- @field technology_difficulty defines.difficulty_settings.technology_difficulty 
--- @field technology_price_multiplier double A value in range [0.001, 1000].
--- @field research_queue_setting string Either `"after-victory"`, `"always"` or `"never"`. Changing this to `"always"` or `"after-victory"` does not automatically unlock the research queue. See [LuaForce](LuaForce) for that.
----

----
--- @class DisplayResolution
--- 
--- @field width uint 
--- @field height uint 
----

----
--- @class DragTarget
--- 
--- @field target_entity LuaEntity 
--- @field target_circuit_id defines.circuit_connector_id? If the wire being dragged is a circuit wire this is the connector id.
--- @field target_wire_id defines.wire_connection_id? If the wire being dragged is copper wire this is the wire id.
----

----
--- @class EnemyEvolutionMapSettings
--- These values represent a percentual increase in evolution. This means a value of `0.1` would increase evolution by 10%.
--- 
--- @field enabled boolean Whether enemy evolution is enabled at all.
--- @field time_factor double The amount evolution naturally progresses by every second. Defaults to `0.000004`.
--- @field destroy_factor double The amount evolution progresses for every destroyed spawner. Defaults to `0.002`.
--- @field pollution_factor double The amount evolution progresses for every unit of pollution. Defaults to `0.0000009`.
----

----
--- @class EnemyExpansionMapSettings
--- Candidate chunks are given scores to determine which one of them should be expanded into. This score takes into account various settings noted below. The iteration is over a square region centered around the chunk for which the calculation is done, and includes the central chunk as well. Distances are calculated as [Manhattan distance](https://en.wikipedia.org/wiki/Taxicab_geometry).
--- 
--- The pseudocode algorithm to determine a chunk's score is as follows:
--- 
--- ```
--- player = 0
--- for neighbour in all chunks within enemy_building_influence_radius from chunk:
---   player += number of player buildings on neighbour
---           * building_coefficient
---           * neighbouring_chunk_coefficient^distance(chunk, neighbour)
--- 
--- base = 0
--- for neighbour in all chunk within friendly_base_influence_radius from chunk:
---   base += num of enemy bases on neighbour
---           * other_base_coefficient
---           * neighbouring_base_chunk_coefficient^distance(chunk, neighbour)
--- 
--- score(chunk) = 1 / (1 + player + base)
--- ```
--- 
--- @field enabled boolean Whether enemy expansion is enabled at all.
--- @field max_expansion_distance uint Distance in chunks from the furthest base around to prevent expansions from reaching too far into the player's territory. Defaults to `7`.
--- @field friendly_base_influence_radius uint Defaults to `2`.
--- @field enemy_building_influence_radius uint Defaults to `2`.
--- @field building_coefficient double Defaults to `0.1`.
--- @field other_base_coefficient double Defaults to `2.0`.
--- @field neighbouring_chunk_coefficient double Defaults to `0.5`.
--- @field neighbouring_base_chunk_coefficient double Defaults to `0.4`.
--- @field max_colliding_tiles_coefficient double A chunk has to have at most this high of a percentage of unbuildable tiles for it to be considered a candidate to avoid chunks full of water as candidates. Defaults to `0.9`, or 90%.
--- @field settler_group_min_size uint The minimum size of a biter group that goes to build a new base. This is multiplied by the evolution factor. Defaults to `5`.
--- @field settler_group_max_size uint The maximum size of a biter group that goes to build a new base. This is multiplied by the evolution factor. Defaults to `20`.
--- @field min_expansion_cooldown uint The minimum time between expansions in ticks. The actual cooldown is adjusted to the current evolution levels. Defaults to `4*3,600=14,400` ticks.
--- @field max_expansion_cooldown uint The maximum time between expansions in ticks. The actual cooldown is adjusted to the current evolution levels. Defaults to `60*3,600=216,000` ticks.
----

----
--- @class EntityPrototypeFilter
--- Depending on the value of `filter`, the table may take additional fields. `filter` may be one of the following:
--- 
--- @field filter string The condition to filter on. One of `"flying-robot"`, `"robot-with-logistics-interface"`, `"rail"`, `"ghost"`, `"explosion"`, `"vehicle"`, `"crafting-machine"`, `"rolling-stock"`, `"turret"`, `"transport-belt-connectable"`, `"wall-connectable"`, `"buildable"`, `"placable-in-editor"`, `"clonable"`, `"selectable"`, `"hidden"`, `"entity-with-health"`, `"building"`, `"fast-replaceable"`, `"uses-direction"`, `"minable"`, `"circuit-connectable"`, `"autoplace"`, `"blueprintable"`, `"item-to-place"`, `"name"`, `"type"`, `"collision-mask"`, `"flag"`, `"build-base-evolution-requirement"`, `"selection-priority"`, `"emissions"`, `"crafting-category"`.
--- @field mode string? How to combine this with the previous filter. Must be `"or"` or `"and"`. Defaults to `"or"`. When evaluating the filters, `"and"` has higher precedence than `"or"`.
--- @field invert boolean? Inverts the condition. Default is `false`.
----

----
--- @class EntityPrototypeFlags
--- A set of flags. Active flags are in the dictionary as `true`, while inactive flags aren't present at all.
--- 
--- By default, none of these flags are set.
--- 
--- @field not-rotatable bool Prevents the entity from being rotated before or after placement.
--- @field placeable-neutral bool Determines the default force when placing entities in the map editor and using the "AUTO" option for the force.
--- @field placeable-player bool Determines the default force when placing entities in the map editor and using the "AUTO" option for the force.
--- @field placeable-enemy bool Determines the default force when placing entities in the map editor and using the "AUTO" option for the force.
--- @field placeable-off-grid bool Determines whether the entity needs to be aligned with the invisible grid within the world. Most entities are confined in this way, with a few exceptions such as trees and land mines.
--- @field player-creation bool Makes it possible to blueprint, deconstruct, and repair the entity (which can be turned off again using the specific flags). Makes it possible for the biter AI to target the entity as a distraction. Enables dust to automatically be created when building the entity. If the entity does not have a `map_color` set, this flag makes the entity appear on the map with the default color specified by the UtilityConstants.
--- @field building-direction-8-way bool Uses 45 degree angle increments when selecting direction.
--- @field filter-directions bool Used to automatically detect the proper direction of the entity if possible. Used by the pump, train stop, and train signal by default.
--- @field fast-replaceable-no-build-while-moving bool Fast replace will not apply when building while moving.
--- @field breaths-air bool Used to specify that the entity breathes air, and is thus affected by poison.
--- @field not-repairable bool Used to specify that the entity can not be 'healed' by repair packs.
--- @field not-on-map bool Prevents the entity from being drawn on the map.
--- @field not-deconstructable bool Prevents the entity from being deconstructed.
--- @field not-blueprintable bool Prevents the entity from being part of a blueprint.
--- @field hidden bool Hides the entity from the bonus GUI and from the "made in"-property of recipe tooltips.
--- @field hide-alt-info bool Hides the alt-info of this entity when in alt-mode.
--- @field fast-replaceable-no-cross-type-while-moving bool Does not fast replace this entity over other entity types when building while moving.
--- @field no-gap-fill-while-building bool 
--- @field not-flammable bool Does not apply fire stickers to the entity.
--- @field no-automated-item-removal bool Prevents inserters and loaders from taking items from this entity.
--- @field no-automated-item-insertion bool Prevents inserters and loaders from inserting items into this entity.
--- @field no-copy-paste bool Prevents the entity from being copy-pasted.
--- @field not-selectable-in-game bool Disallows selection of the entity even when a selection box is specified for other reasons. For example, selection boxes are used to determine the size of outlines to be shown when highlighting entities inside electric pole ranges.
--- @field not-upgradable bool Prevents the entity from being selected by the upgrade planner.
--- @field not-in-kill-statistics bool Prevents the entity from being shown in the kill statistics.
--- @field not-in-made-in bool Prevents the entity from being shown in the "made in" list in recipe tooltips.
----

----
--- @alias EntityPrototypeIdentification LuaEntity|LuaEntityPrototype|string
--- An entity prototype may be specified in one of three ways.
----

----
--- @class EquipmentPoint
--- A table used to define a manual shape for a piece of equipment.
--- 
--- @field x uint 
--- @field y uint 
----

----
--- @alias EquipmentPosition { x: int, y: int }
--- Position inside an equipment grid. This uses the same format as [MapPosition](MapPosition), meaning it can be specified either with or without explicit keys.
----

----
--- @class EquipmentPrototypeFilter
--- Depending on the value of `filter`, the table may take additional fields. `filter` may be one of the following:
--- 
--- @field filter string The condition to filter on. One of `"item-to-place"`, `"type"`.
--- @field mode string? How to combine this with the previous filter. Must be `"or"` or `"and"`. Defaults to `"or"`. When evaluating the filters, `"and"` has higher precedence than `"or"`.
--- @field invert boolean? Inverts the condition. Default is `false`.
----

----
--- @class EventData
--- Information about the event that has been raised. The table can also contain other fields depending on the type of event. See [the list of Factorio events](events.html) for more information on these.
--- 
--- @field name defines.events The identifier of the event this handler was registered to.
--- @field tick uint The tick during which the event happened.
--- @field mod_name string? The name of the mod that raised the event if it was raised using [LuaBootstrap::raise_event](LuaBootstrap::raise_event).
----

----
--- @alias EventFilter LuaPlayerBuiltEntityEventFilter|LuaEntityDeconstructionCancelledEventFilter|LuaPreGhostDeconstructedEventFilter|LuaPrePlayerMinedEntityEventFilter|LuaEntityDiedEventFilter|LuaScriptRaisedBuiltEventFilter|LuaPlayerRepairedEntityEventFilter|LuaPlayerMinedEntityEventFilter|LuaRobotMinedEntityEventFilter|LuaScriptRaisedDestroyEventFilter|LuaSectorScannedEventFilter|LuaPreRobotMinedEntityEventFilter|LuaEntityMarkedForUpgradeEventFilter|LuaPreGhostUpgradedEventFilter|LuaEntityClonedEventFilter|LuaPostEntityDiedEventFilter|LuaUpgradeCancelledEventFilter|LuaEntityMarkedForDeconstructionEventFilter|LuaEntityDamagedEventFilter|LuaRobotBuiltEntityEventFilter|LuaScriptRaisedReviveEventFilter[]
--- Used to filter out irrelevant event callbacks in a performant way.
----

----
--- @class Fluid
--- 
--- @field name string Fluid prototype name of the fluid.
--- @field amount double Amount of the fluid.
--- @field temperature double? The temperature. When reading from [LuaFluidBox](LuaFluidBox), this field will always be present. It is not necessary to specify it when writing, however. When not specified, the fluid will be set to the fluid's default temperature as specified in the fluid's prototype.
----

----
--- @class FluidBoxConnection
--- A definition of a fluidbox connection point.
--- 
--- @field type string The connection type: "input", "output", or "input-output".
--- @field positions Vector[] The 4 cardinal direction connection points for this pipe. This vector is a table with `x` and `y` keys instead of an array.
--- @field max_underground_distance uint? The maximum tile distance this underground connection can connect at if this is an underground pipe.
----

----
--- @class FluidBoxFilter
--- 
--- @field name string Fluid prototype name of the filtered fluid.
--- @field minimum_temperature double The minimum temperature allowed into the fluidbox.
--- @field maximum_temperature double The maximum temperature allowed into the fluidbox.
----

----
--- @class FluidBoxFilterSpec
--- 
--- @field name string Fluid prototype name of the filtered fluid.
--- @field minimum_temperature double? The minimum temperature allowed into the fluidbox.
--- @field maximum_temperature double? The maximum temperature allowed into the fluidbox.
--- @field force boolean? Force the filter to be set, regardless of current fluid content.
----

----
--- @alias FluidIdentification LuaFluidPrototype|Fluid|string
--- A fluid may be specified in one of three ways.
----

----
--- @class FluidPrototypeFilter
--- Depending on the value of `filter`, the table may take additional fields. `filter` may be one of the following:
--- 
--- @field filter string The condition to filter on. One of `"hidden"`, `"name"`, `"subgroup"`, `"default-temperature"`, `"max-temperature"`, `"heat-capacity"`, `"fuel-value"`, `"emissions-multiplier"`, `"gas-temperature"`.
--- @field mode string? How to combine this with the previous filter. Must be `"or"` or `"and"`. Defaults to `"or"`. When evaluating the filters, `"and"` has higher precedence than `"or"`.
--- @field invert boolean? Inverts the condition. Default is `false`.
----

----
--- @alias ForceCondition string
----

----
--- @alias ForceIdentification uint8|LuaForce|string
--- A force may be specified in one of three ways.
----

----
--- @class GameViewSettings
--- Parameters that affect the look and control of the game. Updating any of the member attributes here will immediately take effect in the game engine.
--- 
--- @field show_controller_gui boolean Show the controller GUI elements. This includes the toolbar, the selected tool slot, the armour slot, and the gun and ammunition slots.
--- @field show_minimap boolean Show the chart in the upper right-hand corner of the screen.
--- @field show_research_info boolean Show research progress and name in the upper right-hand corner of the screen.
--- @field show_entity_info boolean Show overlay icons on entities. Also known as "alt-mode".
--- @field show_alert_gui boolean Show the flashing alert icons next to the player's toolbar.
--- @field update_entity_selection boolean When `true` (the default), mousing over an entity will select it. Otherwise, moving the mouse won't update entity selection.
--- @field show_rail_block_visualisation boolean When `true` (`false` is default), the rails will always show the rail block visualisation.
--- @field show_side_menu boolean Shows or hides the buttons row.
--- @field show_map_view_options boolean Shows or hides the view options when map is opened.
--- @field show_quickbar boolean Shows or hides quickbar of shortcuts.
--- @field show_shortcut_bar boolean Shows or hides the shortcut bar.
----

----
--- @class GuiAnchor
--- 
--- @field gui defines.relative_gui_type 
--- @field position defines.relative_gui_position 
--- @field type string? If provided, only anchors the GUI element when the opened things type matches the type.
--- @field name string? If provided, only anchors the GUI element when the opened thing matches the name. `name` takes precedence over `names`.
--- @field names string[]? If provided, only anchors the GUI element when the opened thing matches one of the names. When reading an anchor, `names` is always populated.
----

----
--- @class GuiArrowSpecification
--- Used for specifying where a GUI arrow should point to.
--- 
--- @field type string This determines which of the following fields will be required. Must be one of `"nowhere"` (will remove the arrow entirely), `"goal"` (will point to the current goal), `"entity_info"`, `"active_window"`, `"entity"`, `"position"`, `"crafting_queue"` or `"item_stack"` (will point to a given item stack in an inventory). Depending on this value, other fields may have to be specified.
----

----
--- @alias GuiLocation { x: int, y: int }
--- Screen coordinates of a GUI element in a [LuaGui](LuaGui). This uses the same format as [TilePosition](TilePosition), meaning it can be specified either with or without explicit keys.
----

----
--- @class HeatConnection
--- 
--- @field position Vector 
--- @field direction defines.direction 
----

----
--- @class HeatSetting
--- The settings used by a heat-interface type entity.
--- 
--- @field temperature double? The target temperature. Defaults to the minimum temperature of the heat buffer.
--- @field mode string? `"at-least"`, `"at-most"`, `"exactly"`, `"add"`, or `"remove"`. Defaults to `"at-least"`.
----

----
--- @class InfinityInventoryFilter
--- A single filter used by an infinity-filters instance.
--- 
--- @field name string Name of the item.
--- @field count uint? The count of the filter.
--- @field mode string? `"at-least"`, `"at-most"`, or `"exactly"`. Defaults to `"at-least"`.
--- @field index uint The index of this filter in the filters list. Not required when writing a filter.
----

----
--- @class InfinityPipeFilter
--- A single filter used by an infinity-pipe type entity.
--- 
--- @field name string Name of the fluid.
--- @field percentage double? The fill percentage the pipe (e.g. 0.5 for 50%). Can't be negative.
--- @field temperature double? The temperature of the fluid. Defaults to the default/minimum temperature of the fluid.
--- @field mode string? `"at-least"`, `"at-most"`, `"exactly"`, `"add"`, or `"remove"`. Defaults to `"at-least"`.
----

----
--- @class Ingredient
--- 
--- @field type string `"item"` or `"fluid"`.
--- @field name string Prototype name of the required item or fluid.
--- @field amount double Amount of the item or fluid.
--- @field catalyst_amount double|uint? How much of this ingredient is a catalyst.
----

----
--- @class InserterCircuitConditions
--- 
--- @field circuit CircuitCondition? 
--- @field logistics CircuitCondition? 
----

----
--- @class InventoryFilter
--- 
--- @field index uint Position of the corresponding filter slot.
--- @field name string Item prototype name of the item to filter.
----

----
--- @class ItemPrototypeFilter
--- Depending on the value of `filter`, the table may take additional fields. `filter` may be one of the following:
--- 
--- @field filter string The condition to filter on. One of `"tool"`, `"mergeable"`, `"item-with-inventory"`, `"selection-tool"`, `"item-with-label"`, `"has-rocket-launch-products"`, `"fuel"`, `"place-result"`, `"burnt-result"`, `"place-as-tile"`, `"placed-as-equipment-result"`, `"name"`, `"type"`, `"flag"`, `"subgroup"`, `"fuel-category"`, `"stack-size"`, `"default-request-amount"`, `"wire-count"`, `"fuel-value"`, `"fuel-acceleration-multiplier"`, `"fuel-top-speed-multiplier"`, `"fuel-emissions-multiplier"`.
--- @field mode string? How to combine this with the previous filter. Must be `"or"` or `"and"`. Defaults to `"or"`. When evaluating the filters, `"and"` has higher precedence than `"or"`.
--- @field invert boolean? Inverts the condition. Default is `false`.
----

----
--- @class ItemPrototypeFlags
--- A set of flags. Active flags are in the dictionary as `true`, while inactive flags aren't present at all.
--- 
--- By default, none of these flags are set.
--- 
--- @field draw-logistic-overlay bool Determines whether the logistics areas of roboports should be drawn when holding this item. Used by the deconstruction planner by default.
--- @field hidden bool Hides the item in the logistic requests and filters GUIs (among others).
--- @field always-show bool Always shows the item in the logistic requests and filters GUIs (among others) even when the recipe for that item is locked.
--- @field hide-from-bonus-gui bool Hides the item from the bonus GUI.
--- @field hide-from-fuel-tooltip bool Hides the item from the tooltip that's shown when hovering over a burner inventory.
--- @field not-stackable bool Prevents the item from being stacked. It also prevents the item from stacking in assembling machine input slots, which can otherwise exceed the item stack size if required by the recipe. Additionally, the item does not show an item count when in the cursor.
--- @field can-extend-inventory bool Makes the item act as an extension to the inventory that it is placed in. Only has an effect for items with inventory.
--- @field primary-place-result bool Makes construction bots prefer this item when building the entity specified by its `place_result`.
--- @field mod-openable bool Allows the item to be opened by the player, firing the `on_mod_item_opened` event. Only has an effect for selection tool items.
--- @field only-in-cursor bool Makes it so the item is deleted when clearing the cursor, instead of being put into the player's inventory. The copy-paste tools use this by default, for example.
--- @field spawnable bool Allows the item to be spawned by a quickbar shortcut or custom input.
----

----
--- @alias ItemPrototypeIdentification LuaItemStack|LuaItemPrototype|string
--- An item prototype may be specified in one of three ways.
----

----
--- @class ItemStackDefinition
--- 
--- @field name string Prototype name of the item the stack holds.
--- @field count uint? Number of items the stack holds. If not specified, defaults to `1`.
--- @field health float? Health of the items in the stack. Defaults to `1.0`.
--- @field durability double? Durability of the tool items in the stack.
--- @field ammo double? Amount of ammo in the ammo items in the stack.
--- @field tags string[]? Tags of the items with tags in the stack.
----

----
--- @alias ItemStackIdentification LuaItemStack|SimpleItemStack
--- An item may be specified in one of two ways.
----

----
--- @class ItemStackLocation
--- 
--- @field inventory defines.inventory 
--- @field slot uint 
----

----
--- @alias LocalisedString boolean|string|LocalisedString|string[]|number|LuaObject|nil
--- Localised strings are a way to support translation of in-game text. It is an array where the first element is the key and the remaining elements are parameters that will be substituted for placeholders in the template designated by the key.
--- 
--- The key identifies the string template. For example, `"gui-alert-tooltip.attack"` (for the template `"__1__
---     objects are being damaged"`; see the file `data/core/locale/en.cfg`).
--- 
--- The template can contain placeholders such as `__1__` or `__2__`. These will be replaced by the respective parameter in the LocalisedString. The parameters themselves can be other localised strings, which will be processed recursively in the same fashion. Localised strings can not be recursed deeper than 20 levels and can not have more than 20 parameters.
--- 
--- There are two special flags for the localised string, indicated by the key being a particular string. First, if the key is the empty string (`""`), then all parameters will be concatenated (after processing, if any are localised strings themselves). Second, if the key is a question mark (`"?"`), then the first valid parameter will be used. A parameter can be invalid if its name doesn't match any string template. If no parameters are valid, the last one is returned. This is useful to implement a fallback for missing locale templates.
--- 
--- Furthermore, when an API function expects a localised string, it will also accept a regular string (i.e. not a table) which will not be translated, as well as a number, boolean or `nil`, which will be converted to their textual representation.
----

----
--- @class LogisticFilter
--- 
--- @field index uint The index this filter applies to.
--- @field name string The item name for this filter.
--- @field count uint The count for this filter.
----

----
--- @class LogisticParameters
--- 
--- @field name string? The item. `nil` clears the filter.
--- @field min uint? 
--- @field max uint? 
----

----
--- @class Loot
--- 
--- @field item string Item prototype name of the result.
--- @field probability double Probability that any loot at all will drop, as a number in range [0, 1].
--- @field count_min double Minimum amount of loot to drop.
--- @field count_max double Maximum amount of loot to drop.
----

----
--- @class LuaEntityClonedEventFilter
--- Depending on the value of `filter`, the table may take additional fields. `filter` may be one of the following:
--- 
--- @field filter string The condition to filter on. One of `"ghost"`, `"rail"`, `"rail-signal"`, `"rolling-stock"`, `"robot-with-logistics-interface"`, `"vehicle"`, `"turret"`, `"crafting-machine"`, `"wall-connectable"`, `"transport-belt-connectable"`, `"circuit-network-connectable"`, `"type"`, `"name"`, `"ghost_type"`, `"ghost_name"`.
--- @field mode string? How to combine this with the previous filter. Must be `"or"` or `"and"`. Defaults to `"or"`. When evaluating the filters, `"and"` has higher precedence than `"or"`.
--- @field invert boolean? Inverts the condition. Default is `false`.
----

----
--- @class LuaEntityDamagedEventFilter
--- Depending on the value of `filter`, the table may take additional fields. `filter` may be one of the following:
--- 
--- @field filter string The condition to filter on. One of `"ghost"`, `"rail"`, `"rail-signal"`, `"rolling-stock"`, `"robot-with-logistics-interface"`, `"vehicle"`, `"turret"`, `"crafting-machine"`, `"wall-connectable"`, `"transport-belt-connectable"`, `"circuit-network-connectable"`, `"type"`, `"name"`, `"ghost_type"`, `"ghost_name"`, `"original-damage-amount"`, `"final-damage-amount"`, `"damage-type"`, `"final-health"`.
--- @field mode string? How to combine this with the previous filter. Must be `"or"` or `"and"`. Defaults to `"or"`. When evaluating the filters, `"and"` has higher precedence than `"or"`.
--- @field invert boolean? Inverts the condition. Default is `false`.
----

----
--- @class LuaEntityDeconstructionCancelledEventFilter
--- Depending on the value of `filter`, the table may take additional fields. `filter` may be one of the following:
--- 
--- @field filter string The condition to filter on. One of `"ghost"`, `"rail"`, `"rail-signal"`, `"rolling-stock"`, `"robot-with-logistics-interface"`, `"vehicle"`, `"turret"`, `"crafting-machine"`, `"wall-connectable"`, `"transport-belt-connectable"`, `"circuit-network-connectable"`, `"type"`, `"name"`, `"ghost_type"`, `"ghost_name"`.
--- @field mode string? How to combine this with the previous filter. Must be `"or"` or `"and"`. Defaults to `"or"`. When evaluating the filters, `"and"` has higher precedence than `"or"`.
--- @field invert boolean? Inverts the condition. Default is `false`.
----

----
--- @class LuaEntityDiedEventFilter
--- Depending on the value of `filter`, the table may take additional fields. `filter` may be one of the following:
--- 
--- @field filter string The condition to filter on. One of `"ghost"`, `"rail"`, `"rail-signal"`, `"rolling-stock"`, `"robot-with-logistics-interface"`, `"vehicle"`, `"turret"`, `"crafting-machine"`, `"wall-connectable"`, `"transport-belt-connectable"`, `"circuit-network-connectable"`, `"type"`, `"name"`, `"ghost_type"`, `"ghost_name"`.
--- @field mode string? How to combine this with the previous filter. Must be `"or"` or `"and"`. Defaults to `"or"`. When evaluating the filters, `"and"` has higher precedence than `"or"`.
--- @field invert boolean? Inverts the condition. Default is `false`.
----

----
--- @class LuaEntityMarkedForDeconstructionEventFilter
--- Depending on the value of `filter`, the table may take additional fields. `filter` may be one of the following:
--- 
--- @field filter string The condition to filter on. One of `"ghost"`, `"rail"`, `"rail-signal"`, `"rolling-stock"`, `"robot-with-logistics-interface"`, `"vehicle"`, `"turret"`, `"crafting-machine"`, `"wall-connectable"`, `"transport-belt-connectable"`, `"circuit-network-connectable"`, `"type"`, `"name"`, `"ghost_type"`, `"ghost_name"`.
--- @field mode string? How to combine this with the previous filter. Must be `"or"` or `"and"`. Defaults to `"or"`. When evaluating the filters, `"and"` has higher precedence than `"or"`.
--- @field invert boolean? Inverts the condition. Default is `false`.
----

----
--- @class LuaEntityMarkedForUpgradeEventFilter
--- Depending on the value of `filter`, the table may take additional fields. `filter` may be one of the following:
--- 
--- @field filter string The condition to filter on. One of `"ghost"`, `"rail"`, `"rail-signal"`, `"rolling-stock"`, `"robot-with-logistics-interface"`, `"vehicle"`, `"turret"`, `"crafting-machine"`, `"wall-connectable"`, `"transport-belt-connectable"`, `"circuit-network-connectable"`, `"type"`, `"name"`, `"ghost_type"`, `"ghost_name"`.
--- @field mode string? How to combine this with the previous filter. Must be `"or"` or `"and"`. Defaults to `"or"`. When evaluating the filters, `"and"` has higher precedence than `"or"`.
--- @field invert boolean? Inverts the condition. Default is `false`.
----

