
----
--- @class LuaAISettings
--- Collection of settings for overriding default ai behavior.
--- 
--- @field allow_destroy_when_commands_fail boolean If enabled, units that repeatedly fail to succeed at commands will be destroyed.
--- @field allow_try_return_to_spawner boolean If enabled, units that have nothing else to do will attempt to return to a spawner.
--- @field do_separation boolean If enabled, units will try to separate themselves from nearby friendly units.
--- @field path_resolution_modifier int8 Defines how coarse the pathfinder's grid is, where smaller values mean a coarser grid. Defaults to `0`, which equals a resolution of `1x1` tiles, centered on tile centers. Values range from `-8` to `8` inclusive, where each integer increment doubles/halves the resolution. So, a resolution of `-8` equals a grid of `256x256` tiles, and a resolution of `8` equals `1/256` of a tile.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaAISettings = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaAISettings.help() end


----
--- @class LuaAccumulatorControlBehavior: LuaControlBehavior
--- Control behavior for accumulators.
--- 
--- @field output_signal SignalID 
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaAccumulatorControlBehavior = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaAccumulatorControlBehavior.help() end


----
--- @class LuaAchievementPrototype
--- Prototype of a achievement.
--- 
--- @field name string Name of this prototype.
--- @field order string The string used to alphabetically sort these prototypes. It is a simple string that has no additional semantic meaning.
--- @field localised_name LocalisedString 
--- @field localised_description LocalisedString 
--- @field allowed_without_fight boolean 
--- @field hidden boolean 
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaAchievementPrototype = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaAchievementPrototype.help() end


----
--- @class LuaAmmoCategoryPrototype
--- Prototype of a ammo category.
--- 
--- @field name string Name of this prototype.
--- @field order string The string used to alphabetically sort these prototypes. It is a simple string that has no additional semantic meaning.
--- @field localised_name LocalisedString 
--- @field localised_description LocalisedString 
--- @field bonus_gui_order string 
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaAmmoCategoryPrototype = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaAmmoCategoryPrototype.help() end


----
--- @class LuaArithmeticCombinatorControlBehavior: LuaCombinatorControlBehavior
--- Control behavior for arithmetic combinators.
--- 
--- @field parameters ArithmeticCombinatorParameters This arithmetic combinator's parameters.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaArithmeticCombinatorControlBehavior = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaArithmeticCombinatorControlBehavior.help() end


----
--- @class LuaAutoplaceControlPrototype
--- Prototype of an autoplace control.
--- 
--- @field name string Name of this prototype.
--- @field order string The string used to alphabetically sort these prototypes. It is a simple string that has no additional semantic meaning.
--- @field localised_name LocalisedString 
--- @field localised_description LocalisedString 
--- @field richness boolean 
--- @field can_be_disabled boolean 
--- @field control_order string 
--- @field category string Category name of this prototype.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaAutoplaceControlPrototype = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaAutoplaceControlPrototype.help() end


----
--- @class LuaBootstrap
--- Entry point for registering event handlers. It is accessible through the global object named `script`.
--- 
--- @field mod_name string The name of the mod from the environment this is used in.
--- @field level { campaign_name: string, is_simulation: boolean, is_tutorial: boolean, level_name: string, mod_name: string } Information about the currently running scenario/campaign/tutorial.
--- @field active_mods table<string,string> A dictionary listing the names of all currently active mods and mapping them to their version.
--- @field object_name string This object's name.
----
LuaBootstrap = { }

----
--- Generate a new, unique event ID that can be used to raise custom events with [LuaBootstrap::raise_event](LuaBootstrap::raise_event).
--- 
--- @return uint The newly generated event ID.
----
function LuaBootstrap.generate_event_name() end

----
--- Gets the filters for the given event.
--- 
--- @param event uint ID of the event to get.
--- @return EventFilter? The filters or `nil` if none are defined.
----
function LuaBootstrap.get_event_filter(event) end

----
--- Find the event handler for an event.
--- 
--- @param event uint The event identifier to get a handler for.
--- @return fun(arg_1: EventData): any? Reference to the function currently registered as the handler, if it was found.
----
function LuaBootstrap.get_event_handler(event) end

----
--- Gets the mod event order as a string.
--- 
--- @return string 
----
function LuaBootstrap.get_event_order() end

----
--- Gets the prototype history for the given type and name.
--- 
--- @param type string 
--- @param name string 
--- @return PrototypeHistory 
----
function LuaBootstrap.get_prototype_history(type, name) end

----
--- Register a function to be run when mod configuration changes. This is called when the game version or any mod version changed, when any mod was added or removed, when a startup setting has changed, when any prototypes have been added or removed, or when a migration was applied. It allows the mod to make any changes it deems appropriate to both the data structures in its [global](global) table or to the game state through [LuaGameScript](LuaGameScript).
--- 
--- @param handler nil|fun(arg_1: ConfigurationChangedData): any The handler for this event. Passing `nil` will unregister it.
----
function LuaBootstrap.on_configuration_changed(handler) end

----
--- Register a handler to run on the specified event(s). Each mod can only register once for every event, as any additional registration will overwrite the previous one. This holds true even if different filters are used for subsequent registrations.
--- 
--- @param event defines.events[]|string|defines.events The event(s) or custom-input to invoke the handler on.
--- @param handler nil|fun(arg_1: EventData): any The handler for this event. Passing `nil` will unregister it.
--- @param filters EventFilter? The filters for this event. Can only be used when registering for individual events.
----
function LuaBootstrap.on_event(event, handler, filters) end

----
--- Register a function to be run on mod initialization. This is only called when a new save game is created or when a save file is loaded that previously didn't contain the mod. During it, the mod gets the chance to set up initial values that it will use for its lifetime. It has full access to [LuaGameScript](LuaGameScript) and the [global](global) table and can change anything about them that it deems appropriate. No other events will be raised for the mod until it has finished this step.
--- 
--- @param handler fun(): any|nil The handler for this event. Passing `nil` will unregister it.
----
function LuaBootstrap.on_init(handler) end

----
--- Register a function to be run on save load. This is only called for mods that have been part of the save previously, or for players connecting to a running multiplayer session.
--- 
--- It gives the mod the opportunity to rectify potential differences in local state introduced by the save/load cycle. Doing anything other than the following three will lead to desyncs, breaking multiplayer and replay functionality. Access to [LuaGameScript](LuaGameScript) is not available. The [global](global) table can be accessed and is safe to read from, but not write to, as doing so will lead to an error.
--- 
--- The only legitimate uses of this event are these:
--- - Re-setup [metatables](https://www.lua.org/pil/13.html) as they are not persisted through the save/load cycle.
--- - Re-setup conditional event handlers, meaning subscribing to an event only when some condition is met to save processing time.
--- - Create local references to data stored in the [global](global) table.
--- 
--- For all other purposes, [LuaBootstrap::on_init](LuaBootstrap::on_init), [LuaBootstrap::on_configuration_changed](LuaBootstrap::on_configuration_changed) or [migrations](migrations) should be used instead.
--- 
--- @param handler fun(): any|nil The handler for this event. Passing `nil` will unregister it.
----
function LuaBootstrap.on_load(handler) end

----
--- Register a handler to run every nth-tick(s). When the game is on tick 0 it will trigger all registered handlers.
--- 
--- @param tick uint[]|nil|uint The nth-tick(s) to invoke the handler on. Passing `nil` as the only parameter will unregister all nth-tick handlers.
--- @param handler nil|fun(arg_1: NthTickEventData): any The handler to run. Passing `nil` will unregister it for the provided nth-tick(s).
----
function LuaBootstrap.on_nth_tick(tick, handler) end

----
--- @class LuaBootstrap.raise_biter_base_built_parameters
--- Parameters class for the method `LuaBootstrap.raise_biter_base_built`
--- 
--- @field entity LuaEntity The entity that was built.
----

----
--- 
--- @param args LuaBootstrap.raise_biter_base_built_parameters Table containing arguments for the method.
--- @see LuaBootstrap.raise_biter_base_built_parameters
----
function LuaBootstrap.raise_biter_base_built(args) end

----
--- @class LuaBootstrap.raise_console_chat_parameters
--- Parameters class for the method `LuaBootstrap.raise_console_chat`
--- 
--- @field player_index uint The player doing the chatting.
--- @field message string The chat message to send.
----

----
--- 
--- @param args LuaBootstrap.raise_console_chat_parameters Table containing arguments for the method.
--- @see LuaBootstrap.raise_console_chat_parameters
----
function LuaBootstrap.raise_console_chat(args) end

----
--- Raise an event. Only events generated with [LuaBootstrap::generate_event_name](LuaBootstrap::generate_event_name) and the following can be raised:
--- 
--- - [on_console_chat](on_console_chat)
--- - [on_player_crafted_item](on_player_crafted_item)
--- - [on_player_fast_transferred](on_player_fast_transferred)
--- - [on_biter_base_built](on_biter_base_built)
--- - [on_market_item_purchased](on_market_item_purchased)
--- - [script_raised_built](script_raised_built)
--- - [script_raised_destroy](script_raised_destroy)
--- - [script_raised_revive](script_raised_revive)
--- - [script_raised_teleported](script_raised_teleported)
--- - [script_raised_set_tiles](script_raised_set_tiles)
--- 
--- @param event uint ID of the event to raise.
--- @param data table Table with extra data that will be passed to the event handler. Any invalid LuaObjects will silently stop the event from being raised.
----
function LuaBootstrap.raise_event(event, data) end

----
--- @class LuaBootstrap.raise_market_item_purchased_parameters
--- Parameters class for the method `LuaBootstrap.raise_market_item_purchased`
--- 
--- @field player_index uint The player who did the purchasing.
--- @field market LuaEntity The market entity.
--- @field offer_index uint The index of the offer purchased.
--- @field count uint The amount of offers purchased.
----

----
--- 
--- @param args LuaBootstrap.raise_market_item_purchased_parameters Table containing arguments for the method.
--- @see LuaBootstrap.raise_market_item_purchased_parameters
----
function LuaBootstrap.raise_market_item_purchased(args) end

----
--- @class LuaBootstrap.raise_player_crafted_item_parameters
--- Parameters class for the method `LuaBootstrap.raise_player_crafted_item`
--- 
--- @field item_stack LuaItemStack The item that has been crafted.
--- @field player_index uint The player doing the crafting.
--- @field recipe LuaRecipe The recipe used to craft this item.
----

----
--- 
--- @param args LuaBootstrap.raise_player_crafted_item_parameters Table containing arguments for the method.
--- @see LuaBootstrap.raise_player_crafted_item_parameters
----
function LuaBootstrap.raise_player_crafted_item(args) end

----
--- @class LuaBootstrap.raise_player_fast_transferred_parameters
--- Parameters class for the method `LuaBootstrap.raise_player_fast_transferred`
--- 
--- @field player_index uint The player transferred from or to.
--- @field entity LuaEntity The entity transferred from or to.
--- @field from_player boolean Whether the transfer was from player to entity. If `false`, the transfer was from entity to player.
--- @field is_split boolean Whether the transfer was a split action (half stack).
----

----
--- 
--- @param args LuaBootstrap.raise_player_fast_transferred_parameters Table containing arguments for the method.
--- @see LuaBootstrap.raise_player_fast_transferred_parameters
----
function LuaBootstrap.raise_player_fast_transferred(args) end

----
--- @class LuaBootstrap.raise_script_built_parameters
--- Parameters class for the method `LuaBootstrap.raise_script_built`
--- 
--- @field entity LuaEntity The entity that has been built.
----

----
--- 
--- @param args LuaBootstrap.raise_script_built_parameters Table containing arguments for the method.
--- @see LuaBootstrap.raise_script_built_parameters
----
function LuaBootstrap.raise_script_built(args) end

----
--- @class LuaBootstrap.raise_script_destroy_parameters
--- Parameters class for the method `LuaBootstrap.raise_script_destroy`
--- 
--- @field entity LuaEntity The entity that was destroyed.
----

----
--- 
--- @param args LuaBootstrap.raise_script_destroy_parameters Table containing arguments for the method.
--- @see LuaBootstrap.raise_script_destroy_parameters
----
function LuaBootstrap.raise_script_destroy(args) end

----
--- @class LuaBootstrap.raise_script_revive_parameters
--- Parameters class for the method `LuaBootstrap.raise_script_revive`
--- 
--- @field entity LuaEntity The entity that was revived.
--- @field tags Tags? The tags associated with this entity, if any.
----

----
--- 
--- @param args LuaBootstrap.raise_script_revive_parameters Table containing arguments for the method.
--- @see LuaBootstrap.raise_script_revive_parameters
----
function LuaBootstrap.raise_script_revive(args) end

----
--- @class LuaBootstrap.raise_script_set_tiles_parameters
--- Parameters class for the method `LuaBootstrap.raise_script_set_tiles`
--- 
--- @field surface_index uint The surface whose tiles have been changed.
--- @field tiles Tile[] The tiles that have been changed.
----

----
--- 
--- @param args LuaBootstrap.raise_script_set_tiles_parameters Table containing arguments for the method.
--- @see LuaBootstrap.raise_script_set_tiles_parameters
----
function LuaBootstrap.raise_script_set_tiles(args) end

----
--- @class LuaBootstrap.raise_script_teleported_parameters
--- Parameters class for the method `LuaBootstrap.raise_script_teleported`
--- 
--- @field entity LuaEntity The entity that was teleported.
--- @field old_surface_index uint8 The entity's surface before the teleportation.
--- @field old_position MapPosition The entity's position before the teleportation.
----

----
--- 
--- @param args LuaBootstrap.raise_script_teleported_parameters Table containing arguments for the method.
--- @see LuaBootstrap.raise_script_teleported_parameters
----
function LuaBootstrap.raise_script_teleported(args) end

----
--- Register a metatable to have linkage recorded and restored when saving/loading. The metatable itself will not be saved. Instead, only the linkage to a registered metatable is saved, and the metatable registered under that name will be used when loading the table.
--- 
--- @param name string The name of this metatable. Names must be unique per mod.
--- @param metatable table The metatable to register.
----
function LuaBootstrap.register_metatable(name, metatable) end

----
--- Registers an entity so that after it's destroyed, [on_entity_destroyed](on_entity_destroyed) is called. Once an entity is registered, it stays registered until it is actually destroyed, even through save/load cycles. The registration is global across all mods, meaning once one mod registers an entity, all mods listening to [on_entity_destroyed](on_entity_destroyed) will receive the event when it is destroyed. Registering the same entity multiple times will still only fire the destruction event once, and will return the same registration number.
--- 
--- @param entity LuaEntity The entity to register.
--- @return uint64 The registration number. It is used to identify the entity in the [on_entity_destroyed](on_entity_destroyed) event.
----
function LuaBootstrap.register_on_entity_destroyed(entity) end

----
--- Sets the filters for the given event. The filters are only retained when set after the actual event registration, because registering for an event with different or no filters will overwrite previously set ones.
--- 
--- @param event uint ID of the event to filter.
--- @param filters EventFilter? The filters or `nil` to clear them.
----
function LuaBootstrap.set_event_filter(event, filters) end


----
--- @class LuaBurner
--- A reference to the burner energy source owned by a specific [LuaEntity](LuaEntity) or [LuaEquipment](LuaEquipment).
--- 
--- @field owner LuaEquipment|LuaEntity The owner of this burner energy source
--- @field inventory LuaInventory The fuel inventory.
--- @field burnt_result_inventory LuaInventory The burnt result inventory.
--- @field heat double The current heat (energy) stored in this burner.
--- @field heat_capacity double The maximum heat (maximum energy) that this burner can store.
--- @field remaining_burning_fuel double The amount of energy left in the currently-burning fuel item.
--- @field currently_burning LuaItemPrototype? The currently burning item. Writing `nil` will void the currently burning item without producing a [LuaBurner::burnt_result](LuaBurner::burnt_result).
--- @field fuel_categories table<string,boolean> The fuel categories this burner uses.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaBurner = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaBurner.help() end


----
--- @class LuaBurnerPrototype
--- Prototype of a burner energy source.
--- 
--- @field emissions double The emissions of this energy source in `pollution/Joule`. Multiplying it by energy consumption in `Watt` gives `pollution/second`.
--- @field render_no_network_icon boolean 
--- @field render_no_power_icon boolean 
--- @field effectivity double 
--- @field fuel_inventory_size uint 
--- @field burnt_inventory_size uint 
--- @field smoke SmokeSource[]? The smoke sources for this burner prototype.
--- @field light_flicker { border_fix_speed: float, color: Color, derivation_change_deviation: float, derivation_change_frequency: float, light_intensity_to_size_coefficient: float, maximum_intensity: float, minimum_intensity: float, minimum_light_size: float }? The light flicker definition for this burner prototype.
--- @field fuel_categories table<string,boolean> 
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaBurnerPrototype = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaBurnerPrototype.help() end


----
--- @class LuaChunkIterator
--- A chunk iterator can be used for iterating chunks coordinates of a surface.
--- 
--- The returned type is a [ChunkPositionAndArea](ChunkPositionAndArea) containing the chunk coordinates and its area.
--- 
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaChunkIterator = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaChunkIterator.help() end


----
--- @class LuaCircuitNetwork
--- A circuit network associated with a given entity, connector, and wire type.
--- 
--- @field entity LuaEntity The entity this circuit network reference is associated with.
--- @field wire_type defines.wire_type The wire type this network is associated with.
--- @field circuit_connector_id defines.circuit_connector_id The circuit connector ID on the associated entity this network was gotten from.
--- @field signals Signal[]? The circuit network signals last tick. `nil` if there were no signals last tick.
--- @field network_id uint The circuit networks ID.
--- @field connected_circuit_count uint The number of circuits connected to this network.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaCircuitNetwork = { }

----
--- 
--- @param signal SignalID The signal to read.
--- @return int The current value of the signal.
----
function LuaCircuitNetwork.get_signal(signal) end

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaCircuitNetwork.help() end


----
--- @class LuaCombinatorControlBehavior: LuaControlBehavior
--- 
--- @field signals_last_tick Signal[] The circuit network signals sent by this combinator last tick.
----
LuaCombinatorControlBehavior = { }

----
--- Gets the value of a specific signal sent by this combinator behavior last tick or `nil` if the signal didn't exist.
--- 
--- @param signal SignalID The signal to get
--- @return int? 
----
function LuaCombinatorControlBehavior.get_signal_last_tick(signal) end


----
--- @class LuaCommandProcessor
--- Allows for the registration of custom console commands through the global object named `commands`. Similarly to [event subscriptions](LuaBootstrap::on_event), these don't persist through a save-and-load cycle.
--- 
--- @field commands table<string,LocalisedString> Lists the custom commands registered by scripts through `LuaCommandProcessor`.
--- @field game_commands table<string,LocalisedString> Lists the built-in commands of the core game. The [wiki](https://wiki.factorio.com/Console) has an overview of these.
--- @field object_name string This object's name.
----
LuaCommandProcessor = { }

----
--- Add a custom console command.
--- 
--- @param name string The desired name of the command (case sensitive).
--- @param help LocalisedString The localised help message. It will be shown to players using the `/help` command.
--- @param function_ fun(arg_1: CustomCommandData): any The function that will be called when this command is invoked.
----
function LuaCommandProcessor.add_command(name, help, function_) end

----
--- Remove a custom console command.
--- 
--- @param name string The name of the command to remove (case sensitive).
--- @return boolean Whether the command was successfully removed. Returns `false` if the command didn't exist.
----
function LuaCommandProcessor.remove_command(name) end


----
--- @class LuaConstantCombinatorControlBehavior: LuaControlBehavior
--- Control behavior for constant combinators.
--- 
--- @field parameters ConstantCombinatorParameters[]? This constant combinator's parameters. `nil` if the [item_slot_count](LuaEntityPrototype::item_slot_count) of the combinator's prototype is `0`.  Writing `nil` clears the combinator's parameters.
--- @field enabled boolean Turns this constant combinator on and off.
--- @field signals_count uint The number of signals this constant combinator supports.
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaConstantCombinatorControlBehavior = { }

----
--- Gets the signal at the given index. Returned [Signal](Signal) will not contain signal if none is set for the index.
--- 
--- @param index uint 
--- @return Signal 
----
function LuaConstantCombinatorControlBehavior.get_signal(index) end

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaConstantCombinatorControlBehavior.help() end

----
--- Sets the signal at the given index.
--- 
--- @param index uint 
--- @param signal Signal? Passing `nil` clears the signal.
----
function LuaConstantCombinatorControlBehavior.set_signal(index, signal) end


----
--- @class LuaContainerControlBehavior: LuaControlBehavior
--- Control behavior for container entities.
--- 
--- @field valid boolean Is this object valid? This Lua object holds a reference to an object within the game engine. It is possible that the game-engine object is removed whilst a mod still holds the corresponding Lua object. If that happens, the object becomes invalid, i.e. this attribute will be `false`. Mods are advised to check for object validity if any change to the game state might have occurred between the creation of the Lua object and its access.
--- @field object_name string The class name of this object. Available even when `valid` is false. For LuaStruct objects it may also be suffixed with a dotted path to a member of the struct.
----
LuaContainerControlBehavior = { }

----
--- All methods and properties that this object supports.
--- 
--- @return string 
----
function LuaContainerControlBehavior.help() end


----
--- @class LuaControl
--- This is an abstract base class containing the common functionality between [LuaPlayer](LuaPlayer) and entities (see [LuaEntity](LuaEntity)). When accessing player-related functions through a [LuaEntity](LuaEntity), it must refer to a character entity.
--- 
--- @field surface LuaSurface The surface this entity is currently on.
--- @field surface_index uint Unique [index](LuaSurface::index) (ID) associated with the surface this entity is currently on.
--- @field position MapPosition The current position of the entity.
--- @field vehicle LuaEntity? The vehicle the player is currently sitting in.
--- @field force ForceIdentification The force of this entity. Reading will always give a [LuaForce](LuaForce), but it is possible to assign either [string](string) or [LuaForce](LuaForce) to this attribute to change the force.
--- @field force_index uint Unique [index](LuaForce::index) (ID) associated with the force of this entity.
--- @field selected LuaEntity? The currently selected entity. Assigning an entity will select it if is selectable, otherwise the selection is cleared.
--- @field opened LuaEntity|LuaEquipment|LuaTechnology|LuaEquipmentGrid|LuaItemStack|LuaPlayer|defines.gui_type|LuaGuiElement|LuaInventory? The GUI the player currently has open.  This is the GUI that will asked to close (by firing the [on_gui_closed](on_gui_closed) event) when the `Esc` or `E` keys are pressed. If this attribute is not `nil`, and a new GUI is written to it, the existing one will be asked to close.
--- @field crafting_queue_size uint Size of the crafting queue.
--- @field crafting_queue_progress double The crafting queue progress in the range `[0-1]`. `0` when no recipe is being crafted.
--- @field walking_state { direction: defines.direction, walking: boolean } Current walking state.
--- @field riding_state RidingState Current riding state of this car, or of the car this player is riding in.
--- @field mining_state { mining: boolean, position: MapPosition } Current mining state.
--- @field shooting_state { position: MapPosition, state: defines.shooting } Current shooting state.
--- @field picking_state boolean Current item-picking state.
--- @field repair_state { position: MapPosition, repairing: boolean } Current repair state.
--- @field cursor_stack LuaItemStack? The player's cursor stack. `nil` if the player controller is a spectator.
--- @field cursor_ghost ItemPrototypeIdentification? The ghost prototype in the player's cursor. When read, it will be a [LuaItemPrototype](LuaItemPrototype).
--- @field driving boolean `true` if the player is in a vehicle. Writing to this attribute puts the player in or out of a vehicle.
--- @field crafting_queue CraftingQueueItem[] The current crafting queue items.
--- @field following_robots LuaEntity[] The current combat robots following the character.
--- @field cheat_mode boolean When `true` hand crafting is free and instant.
--- @field character_crafting_speed_modifier double 
--- @field character_mining_speed_modifier double 
--- @field character_additional_mining_categories string[] 
--- @field character_running_speed_modifier double Modifies the running speed of this character by the given value as a percentage. Setting the running modifier to `0.5` makes the character run 50% faster. The minimum value of `-1` reduces the movement speed by 100%, resulting in a speed of `0`.
--- @field character_build_distance_bonus uint 
--- @field character_item_drop_distance_bonus uint 
--- @field character_reach_distance_bonus uint 
--- @field character_resource_reach_distance_bonus uint 
--- @field character_item_pickup_distance_bonus uint 
--- @field character_loot_pickup_distance_bonus uint 
--- @field character_inventory_slots_bonus uint 
--- @field character_trash_slot_count_bonus uint 
--- @field character_maximum_following_robot_count_bonus uint 
--- @field character_health_bonus float 
--- @field character_personal_logistic_requests_enabled boolean If personal logistic requests are enabled for this character or players character.
--- @field vehicle_logistic_requests_enabled boolean If personal logistic requests are enabled for this vehicle (spidertron).
--- @field opened_gui_type defines.gui_type? 
--- @field build_distance uint The build distance of this character or max uint when not a character or player connected to a character.
--- @field drop_item_distance uint The item drop distance of this character or max uint when not a character or player connected to a character.
--- @field reach_distance uint The reach distance of this character or max uint when not a character or player connected to a character.
--- @field item_pickup_distance double The item pickup distance of this character or max double when not a character or player connected to a character.
--- @field loot_pickup_distance double The loot pickup distance of this character or max double when not a character or player connected to a character.
--- @field resource_reach_distance double The resource reach distance of this character or max double when not a character or player connected to a character.
--- @field in_combat boolean Whether this character entity is in combat.
--- @field character_running_speed double The current movement speed of this character, including effects from exoskeletons, tiles, stickers and shooting.
--- @field character_mining_progress double The current mining progress between 0 and 1 of this character, or 0 if they aren't mining.
----
LuaControl = { }

----
--- @class LuaControl.begin_crafting_parameters
--- Parameters class for the method `LuaControl.begin_crafting`
--- 
--- @field count uint The count to craft.
--- @field recipe LuaRecipe|string The recipe to craft.
--- @field silent boolean? If false and the recipe can't be crafted the requested number of times printing the failure is skipped.
----

----
--- Begins crafting the given count of the given recipe.
--- 
--- @param args LuaControl.begin_crafting_parameters Table containing arguments for the method.
--- @see LuaControl.begin_crafting_parameters
--- @return uint The count that was actually started crafting.
----
function LuaControl.begin_crafting(args) end

----
--- Can at least some items be inserted?
--- 
--- @param items ItemStackIdentification Items that would be inserted.
--- @return boolean `true` if at least a part of the given items could be inserted into this inventory.
----
function LuaControl.can_insert(items) end

----
--- Can a given entity be opened or accessed?
--- 
--- @param entity LuaEntity 
--- @return boolean 
----
function LuaControl.can_reach_entity(entity) end

----
--- @class LuaControl.cancel_crafting_parameters
--- Parameters class for the method `LuaControl.cancel_crafting`
--- 
--- @field index uint The crafting queue index.
--- @field count uint The count to cancel crafting.
----

----
--- Cancels crafting the given count of the given crafting queue index.
--- 
--- @param args LuaControl.cancel_crafting_parameters Table containing arguments for the method.
--- @see LuaControl.cancel_crafting_parameters
----
function LuaControl.cancel_crafting(args) end

----
--- Removes the arrow created by `set_gui_arrow`.
--- 
----
function LuaControl.clear_gui_arrow() end

----
--- Remove all items from this entity.
--- 
----
function LuaControl.clear_items_inside() end

----
--- 
--- @param slot_index uint The slot to clear.
----
function LuaControl.clear_personal_logistic_slot(slot_index) end

----
--- Unselect any selected entity.
--- 
----
function LuaControl.clear_selected_entity() end

----
--- 
--- @param slot_index uint The slot to clear.
----
function LuaControl.clear_vehicle_logistic_slot(slot_index) end

----
--- Disable the flashlight.
--- 
----
function LuaControl.disable_flashlight() end

----
--- Enable the flashlight.
--- 
----
function LuaControl.enable_flashlight() end

----
--- Gets the entities that are part of the currently selected blueprint, regardless of it being in a blueprint book or picked from the blueprint library.
--- 
--- @return BlueprintEntity[]? Returns `nil` if there is no currently selected blueprint.
----
function LuaControl.get_blueprint_entities() end

----
--- Gets the count of the given recipe that can be crafted.
--- 
--- @param recipe LuaRecipe|string The recipe.
--- @return uint The count that can be crafted.
----
function LuaControl.get_craftable_count(recipe) end

----
--- Get an inventory belonging to this entity. This can be either the "main" inventory or some auxiliary one, like the module slots or logistic trash slots.
--- 
--- @param inventory defines.inventory 
--- @return LuaInventory? The inventory or `nil` if none with the given index was found.
----
function LuaControl.get_inventory(inventory) end

----
--- Get the number of all or some items in this entity.
--- 
--- @param item string? Prototype name of the item to count. If not specified, count all items.
--- @return uint 
----
function LuaControl.get_item_count(item) end

----
--- Gets the main inventory for this character or player if this is a character or player.
--- 
--- @return LuaInventory? The inventory or `nil` if this entity is not a character or player.
----
function LuaControl.get_main_inventory() end

----
--- The highest index of all inventories this entity can use. Allows iteration over all of them if desired.
--- 
--- @return defines.inventory 
----
function LuaControl.get_max_inventory_index() end

----
--- Gets the parameters of a personal logistic request and auto-trash slot.
--- 
--- @param slot_index uint The slot to get.
--- @return LogisticParameters The logistic parameters. If personal logistics are not researched yet, their `name` will be `nil`.
----
function LuaControl.get_personal_logistic_slot(slot_index) end

----
--- Gets the parameters of a vehicle logistic request and auto-trash slot. Only used on `spider-vehicle`.
--- 
--- @param slot_index uint The slot to get.
--- @return LogisticParameters The logistic parameters. If the vehicle does not use logistics, their `name` will be `nil`.
----
function LuaControl.get_vehicle_logistic_slot(slot_index) end

----
--- Does this entity have any item inside it?
--- 
--- @return boolean 
----
function LuaControl.has_items_inside() end

----
--- Insert items into this entity. This works the same way as inserters or shift-clicking: the "best" inventory is chosen automatically.
--- 
--- @param items ItemStackIdentification The items to insert.
--- @return uint The number of items that were actually inserted.
----
function LuaControl.insert(items) end

----
--- Returns whether the player is holding a blueprint. This takes both blueprint items as well as blueprint records from the blueprint library into account.
--- 
--- Note that both this method and [LuaControl::get_blueprint_entities](LuaControl::get_blueprint_entities) refer to the currently selected blueprint, meaning a blueprint book with a selected blueprint will return the information as well.
--- 
--- @return boolean 
----
function LuaControl.is_cursor_blueprint() end

----
--- Returns whether the player is holding something in the cursor. It takes into account items from the blueprint library, as well as items and ghost cursor.
--- 
--- @return boolean 
----
function LuaControl.is_cursor_empty() end

----
--- Is the flashlight enabled.
--- 
--- @return boolean 
----
function LuaControl.is_flashlight_enabled() end

----
--- When `true` control adapter is a LuaPlayer object, `false` for entities including characters with players.
--- 
--- @return boolean 
----
function LuaControl.is_player() end

----
--- Mines the given entity as if this player (or character) mined it.
--- 
--- @param entity LuaEntity The entity to mine
--- @param force boolean? Forces mining the entity even if the items can't fit in the player.
--- @return boolean Whether the mining succeeded.
----
function LuaControl.mine_entity(entity, force) end

----
--- Mines the given tile as if this player (or character) mined it.
--- 
--- @param tile LuaTile The tile to mine.
--- @return boolean Whether the mining succeeded.
----
function LuaControl.mine_tile(tile) end

----
--- Open the technology GUI and select a given technology.
--- 
--- @param technology TechnologyIdentification? The technology to select after opening the GUI.
----
function LuaControl.open_technology_gui(technology) end

